package actions.userstorytree.model;

import java.util.ArrayList;

import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.TreeItem;

import actions.UserStoryToolAction;
import history.HistoryManager;
import model.LinkStub;
import model.Model;
import model.UsSystem;
import model.UserStory;
import plugin.Controller;
import plugin.config.PreferencesManager;
//import plugin.config.UserStoryToolPreferenceManager;
import ui.UserStoryTableViewer;
import ui.UserStoryTreeView;
import ui.dialogs.ModelDeleteWarningDialog;

public class ModelDeleteAction extends UserStoryToolAction
{
	public static final String NAME = "Delete";
	public static final String TOOLTIP = "Delete  User Stories";
	public static final String IMAGE = "icons/delete.png";
	public static final String COMMAND_ID = "com.userstoriestool.userstorytree.modeldeleteaction";
	
	private static UserStoryTreeView treeview;
	
	public ModelDeleteAction(UserStoryTreeView ustv)
	{
		super(NAME, TOOLTIP, IMAGE, COMMAND_ID);
		treeview = ustv;
	}
	
	public void run()
	{
		if (viewer.getTree().getSelection() == null)
		{
			return;
		}
		ModelDeleteWarningDialog wDialog = null;
		if(PreferencesManager.getShowDeleteDialog())
		{
			wDialog = new ModelDeleteWarningDialog(treeview.getSite().getShell(), viewer.getTree().getSelection().length, true);
		}
		
		if( (!PreferencesManager.getShowDeleteDialog()) || wDialog.open() == Window.OK)
		{
			// Save prefs incase the user ticked the checkbox
			PreferencesManager.instance.savePluginSettings();
			
			ArrayList<Model> remove = new ArrayList<Model>();
			for(TreeItem temp : viewer.getTree().getSelection())
			{
				if(temp.getData() instanceof UserStory || temp.getData() instanceof UsSystem || temp.getData() instanceof LinkStub)
				{
					remove.add((Model) temp.getData());
				}						
			}
			
			int i = 0;
			ArrayList<Model> returnThis = new ArrayList<Model>();
			
			while(i < remove.size())
			{
				int j = 0;
				boolean hasParent = false;
				while(j < remove.size())
				{
					if(remove.get(i).getParentAbsolutePath().equals(remove.get(j).getAbsolutePath())){ //don't need to remove a child if parent will be removed								
						hasParent = true;
						break;
					}
					j++;
				}
				if(!hasParent)
				{
					returnThis.add(remove.get(i));
				}						
				i++;
			}
			//viewer.remove(remove.toArray());
			Controller.getDatabase().deleteModel(returnThis);
			HistoryManager.createHistory(returnThis, HistoryManager.deleteHistory);
			viewer.refresh();
			UserStoryTableViewer.Refresh();	
		}
	}
}