package actions.userstorytree.model;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.PartInitException;

import actions.UserStoryToolAction;
import model.UsSystem;
import plugin.Controller;
import plugin.Database;
import plugin.LogWriter;
import ui.UserStoryTreeView;

public class ModelShowOnDiskAction extends UserStoryToolAction
{
	public static final String NAME = "Show on Disk";
	public static final String TOOLTIP = "Find the file name of selected User Story";
	public static final String IMAGE = "icons/synced.gif";
	public static final String COMMAND_ID = "com.userstoriestool.userstorytree.modelshowondiskaction";
	private static UserStoryTreeView treeview;
	
	public ModelShowOnDiskAction(UserStoryTreeView ustv)
	{
		super(NAME, TOOLTIP, IMAGE, COMMAND_ID);
		treeview = ustv;
	}
	
	public void run()
	{
		if(Controller.instance == null)
		{
			return;
		}
		
		for (int i = 0; i < ResourcesPlugin.getWorkspace().getRoot().getProjects().length; i++)
		{
			if (ResourcesPlugin.getWorkspace().getRoot().getProjects()[i].isOpen()){
				try
				{
					ResourcesPlugin.getWorkspace().getRoot().getProjects()[i]
							.refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
				}
				catch (CoreException e)
				{
					LogWriter.high("ModelShowOnDiskAction: run(): CoreException occured");
					return;
				}
			}
		}
		LogWriter.info(UserStoryTreeView.lastSelectionIndex.getAbsolutePath());

		Path path = null;
		
		if(UserStoryTreeView.lastSelectionIndex instanceof UsSystem)
		{
			path = new Path(UserStoryTreeView.lastSelectionIndex.getAbsolutePath() + "\\" + Database.SYSTEM_FILE_NAME);
		}
		else
		{		
			path = new Path(UserStoryTreeView.lastSelectionIndex.getAbsolutePath());
		}
		
		IFile f = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(path);
					
		if (f != null)
		{
			if (f.exists())
			{
				try
				{
					treeview.getSite().getPage().showView("org.eclipse.ui.navigator.ProjectExplorer").getSite()
							.getSelectionProvider().setSelection(new StructuredSelection(f));
				}
				catch (PartInitException e)
				{			
					LogWriter.high("ModelShowOnDiskAction: run(): PartInitException occured");
					return;
				}
			}
		}
	}		
}
