package actions.userstorytree.userstory;

import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPropertyListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;

import actions.UserStoryToolAction;
import editor.UserStoryEditor;
import editor.UserStoryEditorInput;
import links.MemberLineFinder;
import model.LinkStub;
import model.Model;
import model.UserStory;
import plugin.Controller;
import plugin.Database;
import plugin.LogWriter;
import plugin.config.PreferencesManager;
import ui.UserStoryTableViewer;

public class UserStoryEditAction extends UserStoryToolAction
{
	public static final String NAME = "Edit User Story";
	public static final String TOOLTIP = "Edit Selected User Story";
	public static final String IMAGE = "icons/userstory_edit.png";
	public static final String COMMAND_ID = "com.userstoriestool.userstorytree.userstory.editaction";
	
	public UserStoryEditAction()
	{
		super(NAME, TOOLTIP, IMAGE, COMMAND_ID);
	}
	
	public void run()
	{
		
		ArrayList<Model> x = new ArrayList<Model>();
		for(TreeItem temp : viewer.getTree().getSelection()){
			x.add((Model) temp.getData());
		}
		
		if (PreferencesManager.getOpenEditorOnAdd())
		{
			x.addAll(Database.newModels);
		}
		Database.newModels.clear();
		for(Model temp : x){
			final Model obj = temp;
			
			// Editing a Link should open the Linked Object in its Editor.
			if (obj instanceof LinkStub)
			{
				// Cancel the Double Click action if either of these are null.
				if (PlatformUI.getWorkbench().getActiveWorkbenchWindow() == null)
				{
					return;
				}
				else if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage() == null)
				{
					return;
				}
				// Get Active Workbench Window and Page
				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				// Get File Path
				String linkedFilePath = ((LinkStub)obj).getLinkReference().getSourceFileAbsPath();
				IPath location= Path.fromOSString(linkedFilePath); 
				// Get File
				IFile file = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(location);
				IEditorDescriptor desc = PlatformUI.getWorkbench(). getEditorRegistry().getDefaultEditor(file.getName());
				try
				{
					// Open Editor for this file
					IEditorPart editor = page.openEditor(new FileEditorInput(file), desc.getId());
					if ( ((LinkStub)obj).isClassLink() )
					{
						// If the selected obj is a class, open the class in editor
						MemberLineFinder.findMember(editor, obj.getName());
					}
					else
					{
						// If the selected obj is a method, open the method in editor
						MemberLineFinder.findMember(editor, ((LinkStub)obj).getLinkReference().getClassName(), obj.getName());
					}
				}
				catch (PartInitException e)
				{
					LogWriter.high("UserStoryTreeView: DoubleClickAction on LinkStub: PartInitException occured");
				}
				return;
			}
			
			IEditorReference[] allEditors = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
					.getEditorReferences();
			IEditorReference thisEditor = null;
			boolean found = false;
			for (int i = 0; i < allEditors.length; i++) {
				if (allEditors[i].getEditor(false) instanceof UserStoryEditor) {
					if (((UserStoryEditor) (allEditors[i].getEditor(false))).getUserStory()
							.equals(obj)) {
						found = true;
						thisEditor = allEditors[i];
					}
				}
			}
			if (!found) {
				int actors = -1;
				if(obj instanceof UserStory){
					actors = ((UserStory)obj).getActors().size();
				}
				UserStoryEditorInput input = new UserStoryEditorInput(obj, actors );
				
				try {
					IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
							.openEditor(input, UserStoryEditor.ID);
					if (editor instanceof UserStoryEditor) {
						((UserStoryEditor) editor).SetEditorName(obj.getName());
					}
					editor.addPropertyListener(new IPropertyListener() {
						@Override
						public void propertyChanged(Object openEditor, int arg1) {
							if (openEditor instanceof UserStoryEditor) {
								if (!((UserStoryEditor) openEditor).isDirty()){										
									Controller.getDatabase().edit(obj,((UserStoryEditor) openEditor).getUserStory());																		
									
									viewer.update(obj, null);	
									UserStoryTableViewer.Refresh();		
									viewer.refresh();
								}
							}
						}

					});
				} catch (PartInitException e) {
					LogWriter.info(e.getMessage());
				}
				viewer.getTree().deselectAll();
			} else {
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
						.showEditor(thisEditor);
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
						.activate(thisEditor.getEditor(true));
			}
		}
	}
}