package actors;

import java.util.Comparator;

//import com.plugin.userstories.markers.MarkersManager;

public class Actor implements Comparable<Actor>
{
	private String name;
	private int id;
	
	//example line looks like this: "5,headmaster"
	public Actor(String line)
	{
		if (line.contains(","))
		{
			//Get ID
			//The number before the first comma is the ID. If it isn't a number, set it to -1
			String idStr = line.split(",")[0];
			try
			{
				this.id = Integer.valueOf(idStr);
			}
			catch (NumberFormatException e)
			{				
				this.id = -2;
				this.name = "";
				//MarkersManager.createMarker(MarkersManager.severityError, "Actors File: Error in line \"" + line + "\": Couldn't parse ID",this.id);
				return;
			}
			
			//Get Name
			//Everything after the first comma is the name. Even if the line is "1,,hey,,,", the name will be counted after the comma
			int i=0;
			boolean firstCommaFound = false;
			
			//Get index of first comma
			while(i<line.length() && !firstCommaFound)
			{
				if (line.charAt(i) == ',')
				{
					firstCommaFound = true;
				}
				else
				{
					i++;
				}
			}
			
			//i contains the position of the first comma, so take everything after that
			if (i+1<line.length())
			{
				this.name = line.substring(i+1);
			}
			else
			{				
				this.id = -1;
				this.name = "";
				//MarkersManager.createMarker(MarkersManager.severityError, "Actors File: Nothing found after comma",this.id);
				return;
			}			
		}
		else
		{
			this.id = -1;
			this.name = "";
			//MarkersManager.createMarker(MarkersManager.severityError, "Actors File: Error in line \"" + line + "\": Line has no commas",this.id);
			return;
		}
	}
	
	public Actor(int id, String name)
	{
		this.id = id;
		this.name = name;
	}
	
	@Override
	public String toString()
	{
		return this.id + "," + this.name + "\n";
	}
	
	public String toStringNoNewLine()
	{
		return this.id + "," + this.name;
	}


	public String getName() 
	{
		return this.name;
	}

	public int getId()
	{
		return this.id;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public void setId(int id) {
		this.id = id;
	}

	/*public ArrayList<Model> getModelsActorParticipates(Model parent){
		ArrayList<Model> returnThis = new ArrayList<Model>();
		
		for()
	}*/
	public int compareTo(Actor comparedActor)
	{
		int compareId = ((Actor) comparedActor).getId(); 
		return this.getId() - compareId;
	}
	
	public static Comparator<Actor> ActorIDComparator  = new Comparator<Actor>()
	{
		public int compare(Actor actor1, Actor actor2)
		{
			return actor1.compareTo(actor2);
		}
	};
}
