package editor;


import java.util.ArrayList;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import model.TestCase;
import model.UserStory;
import plugin.Activator;
import plugin.config.Preference;
import plugin.config.PreferencesManager;
import ui.dialogs.ModelDeleteWarningDialog;
import ui.dialogs.TestCaseCancelDialog;
public class EditTestCases extends Dialog
{
	private final static String LABEL_NO_TESTS = "No Test Cases available. \nSelect \"Add Test Cases\" to add some.";
	private final static String LABEL_AVAILABLE_TESTS = "Available Test Cases";
	private final static String LABEL_DESCRIPTION = "Description";
	private final static String LABEL_REMOTE_MANAGEMENT = "Remote Management";
	private final static String LABEL_EXPECTED_RESULTS = "Expected Results";
	private final static String LABEL_CANCEL_WARNING = "Are you sure you want to cancel the changes?";
	public final static String LABEL_ADD_TEST_CASES = "Add Test Cases";
	private final static String LABEL_REMOVE_TEST_CASE = "Remove Selected";
	private final static String LABEL_CONFIRM_CHANGES = "Confirm Changes";
	public final static String LABEL_CANCEL = "Cancel";
	private final static String LABEL_TEST_CASE_EDITOR = "Test Case Editor";
	
	private List listCases; //SWT.List
	private ArrayList<TestCase> dirtyTestCases; //Unsaved test cases (new or changed)
	private UserStory us;
	private int selected = -1;
	
	private Text txtRemoteManagement;
	private StyledText txtExpectedResults;
	private StyledText txtDescription;
	
	protected EditTestCases(Shell parentShell,UserStory us) {
		super(parentShell);
		this.us = us;
		this.setShellStyle(this.getShellStyle() ^ SWT.APPLICATION_MODAL | SWT.MODELESS);
		//setBlockOnOpen(false);
		dirtyTestCases  = new ArrayList<TestCase>();		
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		//Parent with 2 column each column has a Group
		Composite parentContainer = (Composite) super.createDialogArea(parent);
		
		Composite bottomContainer = new Composite(parent,SWT.BORDER);
		bottomContainer.setLayout(new GridLayout(2,false));
		bottomContainer.setLayoutData(new GridData(SWT.FILL,SWT.NONE,true,false));
		
		new Label(bottomContainer,SWT.NONE).setText(LABEL_REMOTE_MANAGEMENT);
		txtRemoteManagement = new Text(bottomContainer,SWT.BORDER);
		txtRemoteManagement.setLayoutData(new GridData(SWT.FILL,SWT.NONE,true,false));
		txtRemoteManagement.setText("");
		txtRemoteManagement.addModifyListener(new ModifyListener(){

			@Override
			public void modifyText(ModifyEvent styledText) {
				if(selected > -1 && dirtyTestCases.size() > 0)
					dirtyTestCases.get(selected).setRemoteManagement(((Text)styledText.getSource()).getText());
			}
			
		});
		
		GridLayout gl = new GridLayout();
		gl.numColumns = 2;
		gl.makeColumnsEqualWidth = true;
		parentContainer.setLayout(gl);
		//Group with a column
		Group groupClass = new Group(parentContainer, SWT.FILL);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		groupClass.setLayout(gridLayout);
		 
			Group groupTestCasesValue = new Group(parentContainer, SWT.FILL);
			groupTestCasesValue.setLayout(gridLayout);
			//Styled Texts get all the available space				
			GridData gd = new GridData();
			gd.horizontalAlignment = GridData.FILL;
			gd.verticalAlignment = GridData.FILL;
			gd.grabExcessHorizontalSpace = true;
			gd.grabExcessVerticalSpace = true;
			
			//Test cases description
			Label lblDescription = new Label(groupTestCasesValue, SWT.NONE);
			lblDescription.setText(LABEL_DESCRIPTION);
			txtDescription = new StyledText(groupTestCasesValue,  SWT.V_SCROLL | SWT.WRAP | SWT.BORDER);
			txtDescription.setLayoutData(gd);
			txtDescription.setText("");
			txtDescription.addModifyListener(new ModifyListener(){

				@Override
				public void modifyText(ModifyEvent styledText) {
					if(selected > -1 && dirtyTestCases.size() > 0)
						dirtyTestCases.get(selected).setDescription(((StyledText)styledText.getSource()).getText());
				}
				
			});
			
			//Test Case "ExpectedResults"
			Label lblExpectedResults = new Label(groupTestCasesValue, SWT.NONE);
			lblExpectedResults.setText(LABEL_EXPECTED_RESULTS);
			txtExpectedResults = new StyledText(groupTestCasesValue, SWT.V_SCROLL | SWT.WRAP | SWT.BORDER);
			txtExpectedResults.setLayoutData(gd);
			txtExpectedResults.setText("");
			txtExpectedResults.addModifyListener(new ModifyListener(){

				@Override
				public void modifyText(ModifyEvent styledText) {
					if(selected > -1 && dirtyTestCases.size() > 0){
						dirtyTestCases.get(selected).setExpectedResults(((StyledText)styledText.getSource()).getText());
					}
				}
				
			});				
			
		//All Test Cases
		Label lblTestCases = new Label(groupClass, SWT.WRAP);
		lblTestCases.setText(LABEL_AVAILABLE_TESTS);
		
		listCases = new List(groupClass,SWT.V_SCROLL);		
		listCases.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		//Each group's size
		GridData gridData = new GridData(GridData.FILL, GridData.FILL, true, true);
		gridData.widthHint = 80;
		gridData.heightHint = 80;
		gridData.verticalSpan = 3;
		groupClass.setLayoutData(gridData);
		groupTestCasesValue.setLayoutData(gridData);
		listCases.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent listCases) {
				selected = ((List)listCases.getSource()).getSelectionIndex();
				txtDescription.setText(dirtyTestCases.get(selected).getDescription());
				txtExpectedResults.setText(dirtyTestCases.get(selected).getExpectedResults());
				txtRemoteManagement.setText(dirtyTestCases.get(selected).getRemoteManagement());
			}								
		});
		if(us.getTestCases().size() > 0){
			for(TestCase i : us.getTestCases()){
				listCases.add(i.getId());
				dirtyTestCases.add(i);
			}
			txtDescription.setText(dirtyTestCases.get(0).getDescription());
			txtExpectedResults.setText(dirtyTestCases.get(0).getExpectedResults());	
			txtRemoteManagement.setText(dirtyTestCases.get(0).getRemoteManagement());
			listCases.setSelection(0);
		}else{				
			lblTestCases.setText(LABEL_NO_TESTS);
		}			
		return parent;			
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent)
	{
		Button addTestCase = createButton(parent,-1, LABEL_ADD_TEST_CASES, false);
		addTestCase.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		addTestCase.setImage(new Image(null, Activator.getImageDescriptor("icons/add.png").getImageData()));
		
		Button remove = createButton(parent,-2, LABEL_REMOVE_TEST_CASE, false);
		remove.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		remove.setImage(new Image(null, Activator.getImageDescriptor("icons/delete.png").getImageData()));
		
		Button save = createButton(parent,IDialogConstants.OK_ID, LABEL_CONFIRM_CHANGES, false);
		save.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		save.setImage(new Image(null, Activator.getImageDescriptor("icons/accept.png").getImageData()));
		
		Button cancel = createButton(parent, -3,LABEL_CANCEL, false);
		cancel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		cancel.setImage(new Image(null, Activator.getImageDescriptor("icons/cancel.png").getImageData()));
		
		cancel.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if( PreferencesManager.getShowTestCancelDialog())
				{
					TestCaseCancelDialog wn = new TestCaseCancelDialog(getShell(),LABEL_CANCEL_WARNING,false);					
					wn.create();
					if(wn.open() == Window.OK)
					{
						// Save plugin settings in case the user selected to never show this again
						PreferencesManager.instance.savePluginSettings();
						close();						
					}
				}
				else
				{
					close();
				}
			}
		});
		remove.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				
				ModelDeleteWarningDialog wn = new ModelDeleteWarningDialog(getShell(),"Delete Selected?",false);					
				wn.create();
				if(wn.open() == Window.OK){
					dirtyTestCases.remove(listCases.getSelectionIndex());
					listCases.remove(listCases.getSelectionIndex());
					
				}
			}	
		});
		addTestCase.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				
				MultiCreator mc = new MultiCreator(getShell(), MultiCreator.TYPE_TEST_CASE);
				mc.create();
				if(mc.open() == Window.OK){
					ArrayList<String> names = mc.getNames();
					for(int i = 0; i < names.size(); i++){
						TestCase newTestCase = new TestCase(names.get(i));
						listCases.add(names.get(i));
						dirtyTestCases.add(newTestCase);
						if(listCases.getSelectionIndex() < 0){
							if(listCases.getItemCount() > 0){
								listCases.select(0);
								selected = 0;
								txtDescription.setText(dirtyTestCases.get(selected).getDescription());
								txtExpectedResults.setText(dirtyTestCases.get(selected).getExpectedResults());
								txtRemoteManagement.setText(dirtyTestCases.get(selected).getRemoteManagement());
								
							}
						}
					}
					
				}
			}		
		});
		
		save.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				us.setTestCases(dirtyTestCases);
			}			
		});
	}

	@Override
	protected boolean isResizable() {
		return true;
	}

	@Override
	protected Point getInitialSize() {
		return new Point(600, 600);
	}

	@Override
	protected void okPressed() {
		super.okPressed();
	}

	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(LABEL_TEST_CASE_EDITOR);
	}

	@Override
	public boolean close() {
		return super.close();
	}
	public UserStory getUserStory(){
		return this.us;
	}
}
