package editor;

import java.util.ArrayList;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import plugin.Activator;

public class MultiCreator extends Dialog
{
	private final static String[] TEST_CASE = {"Test Case", "Test Cases"};
	private final static String[] USER_STORY = {"User Story", "User Stories"};
	private final static String[] SYSTEM = {"System", "Systems"};

	private final static String[] LABEL_DIALOG_PARTS = {"Multi ", " Creator"};
	private final static String[] LABEL_INFO_PARTS = {"Create multiple ", " separated by newline. \nSelect the \"" , "\" button when you are ready to add your "};
	private final static String LABEL_BUTTON_PART = "Add ";

	public final static String TYPE_TEST_CASE = "TYPE_TEST_CASE";
	public final static String TYPE_USER_STORY = "TYPE_USER_STORY";
	public final static String TYPE_SYSTEM = "TYPE_SYSTEM";
	
	private static String LABEL_DIALOG = "";
	private static String LABEL_INFO = "";
	private static String LABEL_BUTTON = "";
	
	private ArrayList<String> names = new ArrayList<String>();
	
	public MultiCreator(Shell parent, String type)
	{
		super(parent);	
		setDialogNameAndInfoByType(type);		
	}
	
	public void setDialogNameAndInfoByType(String type)
	{
		String[] objectNames = {"", ""};
		switch (type)
		{
			case TYPE_TEST_CASE:
				objectNames = TEST_CASE;
				break;
			case TYPE_USER_STORY:
				objectNames = USER_STORY;
				break;
			case TYPE_SYSTEM:
				objectNames = SYSTEM;
				break;
			default:
				break;
		}
		
		LABEL_DIALOG = LABEL_DIALOG_PARTS[0] + objectNames[0] + LABEL_DIALOG_PARTS[1];
		LABEL_INFO = LABEL_INFO_PARTS[0] + objectNames[1] + LABEL_INFO_PARTS[1] + objectNames[1] + LABEL_INFO_PARTS[2] + objectNames[1] + ".";
		LABEL_BUTTON = LABEL_BUTTON_PART + objectNames[1];
	}

	@Override
	protected Control createDialogArea(Composite parent)
	{
		int padding = 5;
		GridData gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.verticalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		
		
		final CLabel infoLabel = new CLabel(parent, SWT.NONE);
		infoLabel.setText(LABEL_INFO);
		infoLabel.setMargins(padding, padding, padding, padding);
		
		StyledText text = new StyledText(parent, SWT.V_SCROLL | SWT.WRAP | SWT.BORDER);
		text.setToolTipText("Create multiple objects. Add a name and then press \"Enter\" to add another ");
		text.setLayoutData(gd);
		text.addModifyListener(new ModifyListener(){
			@Override
			public void modifyText(ModifyEvent styledText)
			{
				StyledText temp = ((StyledText)styledText.getSource());
				names.clear();
				for(int i = 0; i < temp.getLineCount(); i++)
				{
					if( !temp.getLine(i).trim().equals("") )
					{
						names.add( temp.getLine(i).trim() );
					}
				}
			}			
		});
		text.setLayoutData(gd);
		text.setMargins(padding, padding, padding, padding);
		return parent;		
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent)
	{
		Button add = createButton(parent,  IDialogConstants.OK_ID, LABEL_BUTTON ,false);
		add.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		add.setImage(new Image(null, Activator.getImageDescriptor("icons/add.png").getImageData()));
		
		Button cancel = createButton(parent, IDialogConstants.CANCEL_ID, EditTestCases.LABEL_CANCEL, false);
		cancel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		cancel.setImage(new Image(null, Activator.getImageDescriptor("icons/cancel.png").getImageData()));
		
	}

	@Override
	protected boolean isResizable() {
		return true;
	}

	@Override
	protected Point getInitialSize() {
		return new Point(450, 250);
	}

	@Override
	protected void okPressed() {
		super.okPressed();
	}

	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(LABEL_DIALOG);
	}

	@Override
	public boolean close() {
		return super.close();
	}

	public ArrayList<String> getNames() {
		return names;
	}
}
