package editor;

import java.io.IOException;
import java.util.ArrayList;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import actors.ActorManager;
import model.Model;
import model.TestCase;
import model.UserStory;
import plugin.Activator;
import plugin.LogWriter;
import ui.dialogs.actors.ActorPicker;

public class UserStoryEditor extends EditorPart {

	public static final String ID = "com.userstoriestool.editor.USEditor";
	public boolean dirty = false; // If changes were happen
	
	/**
	 * Actor Stuff
	 */

	private int actors = 1;
	public static final String LABEL_ACTORS_REFRESH = "Refresh Actor List";
	public static final String LABEL_ACTORS_SELECT = "Select Actors";
	public static final String LABEL_TEST_ADD = "Add Test Case";
	public static final String LABEL_TEST_REMOVE = "Remove Test Case";
	public String LABEL_CURRENT_ACTORS = "";
	public Label currentActorsLabel;
	private ArrayList<Integer> editorActors;
	//private int numberOfActors = 0;
	public static Combo usUser;
	
	private String editorName = ""; // Name of editor
	Model us; // The User Story open in Editor
	Model newUs; // The unsaved User Story

	Text nameText;
	StyledText descriptionText;
	private boolean tableSorter = true;

	@Override
	public void doSave(IProgressMonitor arg0) {
		setPartName(editorName);
		setTitleToolTip(editorName);
		if(newUs instanceof UserStory){
			((UserStory)newUs).setActors(editorActors);
		}
		if(newUs.getName().equals("")){
			newUs.setName(us.getName());
		}
		dirty = false;
		firePropertyChange(PROP_DIRTY);
	}

	@Override
	public void doSaveAs() {
		setPartName(editorName);
		setTitleToolTip(editorName);
		if(newUs instanceof UserStory){
			((UserStory)newUs).setActors(editorActors);
		}
		if(newUs.getName().equals("")){
			newUs.setName(us.getName());
		}
		dirty = false;
		firePropertyChange(PROP_DIRTY);
	}

	@Override
	public void init(IEditorSite site, IEditorInput inp) throws PartInitException {
		if (inp instanceof UserStoryEditorInput) {
			this.us = ((UserStoryEditorInput) inp).getUserStory();
			editorName = this.us.getName();
			if(this.us instanceof UserStory){
				editorActors = ((UserStory)us).getActors();
				this.actors = ((UserStoryEditorInput) inp).getActors();
			}
		}
		setSite(site);
		setInput(inp);
		setPartName("Add new UserStory");
		setTitleToolTip(editorName);
		newUs = us.getCopy();
	}

	@Override
	public boolean isDirty() {
		return dirty;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return true;
	}

	@Override
	public void createPartControl(final Composite parent) {
		// Container Grid Layout
		GridLayout gridLayout = new GridLayout(1, false);
		
		// Container Grid Data
		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		parent.setLayoutData(gridData);
		parent.setLayout(gridLayout);
		
		//parentC.setLayout(gridLayout);		
		// Container Declaration
		/*Composite container = new Composite(parent, SWT.NONE);		
		container.setLayout(gridLayout);
		container.setLayoutData(gridData);
		 */
		// Scrolled Composite Declaration
		final ScrolledComposite scrolledComposite = new ScrolledComposite(parent,
				SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);
		scrolledComposite.setAlwaysShowScrollBars(true);
		scrolledComposite.setLayoutData(gridData);
		
		scrolledComposite.setMinSize(parent.computeSize(1000,1000));

		// Composite that contains the actual data
		final Composite composite = new Composite(scrolledComposite, SWT.NONE);
		composite.setLayout(new GridLayout(1, false));
		scrolledComposite.setContent(composite);
		scrolledComposite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));

		// User Story Name Composite
		Composite nameComposite = new Composite(composite, SWT.NONE);
		nameComposite.setLayout(new GridLayout(SWT.HORIZONTAL, false));

		// US Name Logic
		new Label(nameComposite, SWT.WRAP).setText("Name : ");
		nameText = new Text(nameComposite, SWT.WRAP|SWT.BORDER);

		GridData gd = new GridData();
		gd.horizontalAlignment = GridData.GRAB_HORIZONTAL;
		gd.grabExcessHorizontalSpace = true;
		gd.minimumWidth = SWT.HORIZONTAL;

		nameText.setLayoutData(gd);

		nameText.setText(us.getName());
		nameText.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				dirty = true;
				firePropertyChange(PROP_DIRTY);

				Text textWidget = (Text) e.getSource();
				newUs.setName(textWidget.getText());
				editorName = newUs.getName();
				if (editorName.equals(""))
					dirty = false;
			}
		});
		
		if(us instanceof UserStory){
			//Actor Composite
			Composite actorComposite = new Composite(composite,SWT.FILL);
			actorComposite.setLayout(new GridLayout(3, false));
			
			//Actor Logic for multiple Actors
			while(editorActors.size() < actors)
			{
				editorActors.add(0);
			}
			
			//Label
			new Label(actorComposite,SWT.NONE).setText("Actors");
			
			//Select Actors Button
			Button selectActors = new Button(actorComposite,SWT.PUSH);
			selectActors.setText(LABEL_ACTORS_SELECT);
			selectActors.setImage(Activator.getImageDescriptor("icons/actors_select.png").createImage());
			selectActors.setToolTipText("Shows a dialog with all Actors, to allow choosing more than one Actor for this User Story");
			selectActors.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(final SelectionEvent e)
				{
					try
					{
						ActorPicker actorPicker = new ActorPicker(getSite().getWorkbenchWindow().getShell());
						ActorPicker.givenUS = (UserStory)us;
						ActorPicker.editorActors = editorActors;
						actorPicker.create();
						
						if (actorPicker.open() == Window.OK)
						{
							if (ActorPicker.dirtyEditor)
							{
								dirty = true;
								firePropertyChange(PROP_DIRTY);
								
								//Create a string of Selected Actors
								LABEL_CURRENT_ACTORS = "";
								for (int id : editorActors)
								{
									LogWriter.info("UserStoryEditor: Actors Saved: " + ActorManager.getActorById(id).getName());
									LABEL_CURRENT_ACTORS += ActorManager.getActorById(id).getName();
									LABEL_CURRENT_ACTORS += ", ";
								}
								LABEL_CURRENT_ACTORS = LABEL_CURRENT_ACTORS.substring(0, LABEL_CURRENT_ACTORS.length()-2);
								
								//Show it
								currentActorsLabel.setText(LABEL_CURRENT_ACTORS);
								currentActorsLabel.getParent().layout();
								currentActorsLabel.getParent().getParent().layout();
							}
						}
					}
					catch (IOException e1)
					{
						LogWriter.high("UserStoryEditor: IOException occured while trying to open Actor Picker");
	
					}
				}
			});
			
			//Create a string of Selected Actors
			LABEL_CURRENT_ACTORS = "";
			for (int id : editorActors)
			{
				LogWriter.info("UserStoryEditor: Actors Before: " + ActorManager.getActorById(id).getName());
				LABEL_CURRENT_ACTORS += ActorManager.getActorById(id).getName();
				LABEL_CURRENT_ACTORS += ", ";
			}
			LABEL_CURRENT_ACTORS = LABEL_CURRENT_ACTORS.substring(0, LABEL_CURRENT_ACTORS.length()-2);
			
			//Show it
			currentActorsLabel = new Label(actorComposite,SWT.NONE);
			currentActorsLabel.setText(LABEL_CURRENT_ACTORS);
		}
		
		// Description Composite Declaration
		Composite descriptionComposite = new Composite(composite, SWT.BORDER);
		GridData descData = new GridData(SWT.FILL, SWT.DEFAULT, false,false);
		descData.heightHint = 400;
		descriptionComposite.setLayoutData(descData);
		descriptionComposite.setLayout(new GridLayout(1, false));
		descriptionComposite.setSize(SWT.HORIZONTAL,400);		
		// Description
		new Label(descriptionComposite, SWT.NONE).setText("User Story Description : ");
		descriptionText = new StyledText(descriptionComposite, SWT.H_SCROLL | SWT.V_SCROLL | SWT.WRAP | SWT.BORDER);
		descriptionText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		descriptionText.setText(us.getDescription());
		descriptionText.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				dirty = true;
				firePropertyChange(PROP_DIRTY);
				StyledText textWidget = (StyledText) e.getSource();
				newUs.setDescription(textWidget.getText());
			}

		});
		if(us instanceof UserStory){

			Composite testCasesButtons = new Composite(composite,SWT.NONE);
			testCasesButtons.setLayoutData(new GridData(SWT.NONE, SWT.NONE, false, false));
			testCasesButtons.setLayout(new GridLayout(2, false));
			
			final Composite cases = new Composite(composite,SWT.BORDER);			
			cases.setLayout(new GridLayout(1, false));	
			GridData casesData = new GridData(SWT.FILL,SWT.NONE,true,false);
			casesData.heightHint = 400;
			cases.setLayoutData(casesData);
			
			final TableViewer viewer = new TableViewer(cases,SWT.FULL_SELECTION|SWT.H_SCROLL|SWT.V_SCROLL);
			viewer.setContentProvider(new ViewContentProvider());
			viewer.setLabelProvider(new ViewLabelProvider());
			viewer.getTable().setLayout(new GridLayout());
			viewer.getTable().setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,true));
			viewer.getTable().setHeaderVisible(true);
			viewer.getTable().setLinesVisible(true);
			
			TableViewerColumn vcl = new TableViewerColumn(viewer,SWT.WRAP);
			vcl.setEditingSupport(new TestCasesEditingSupport(viewer,TestCasesEditingSupport.ChangeName,this));
			//Test case ID
			TableColumn column = vcl.getColumn();
			column.setText("ID");
			column.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent arg0) {
					sorterTableViewer(viewer,0,tableSorter);
					tableSorter = !tableSorter;
				}
			});
			column.setToolTipText("The ID of the test case");
			column.setWidth(200);
			column.setResizable(true);
			column.setMoveable(true);
			
			vcl.setLabelProvider(new ColumnLabelProvider() {
				@Override
				public String getText(Object element) {
					return ((TestCase)element).getId();
				}
			});	
			//Test case Expected Results
			vcl = new TableViewerColumn(viewer,SWT.WRAP);
			column = vcl.getColumn();
			column.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent arg0) {
					sorterTableViewer(viewer,1,tableSorter);
					tableSorter = !tableSorter;
				}
			});
			vcl.setEditingSupport(new TestCasesEditingSupport(viewer,TestCasesEditingSupport.ChangeResults,this));
			column.setWidth(200);
			column.setText("Expected Results");
			column.setToolTipText("The expected results of the Test Case");
			column.setResizable(true);
			column.setMoveable(true);
			vcl.setLabelProvider(new ColumnLabelProvider() {
				@Override
				public String getText(Object element) {
					return ((TestCase)element).getExpectedResults().replaceAll("\n"," ");
				}
			});	
			//Test case Remote Management
			vcl = new TableViewerColumn(viewer,SWT.WRAP);
			column = vcl.getColumn();
			column.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent arg0) {
					sorterTableViewer(viewer,3,tableSorter);
					tableSorter = !tableSorter;
				}
			});
			vcl.setEditingSupport(new TestCasesEditingSupport(viewer,TestCasesEditingSupport.ChangeRemoteManagement,this));
			column.setText("Remote Management");			
			column.setWidth(200);
			column.setToolTipText("The code of test case in remote systems if there are any.");
			column.setResizable(true);
			column.setMoveable(true);
			vcl.setLabelProvider(new ColumnLabelProvider() {
				@Override
				public String getText(Object element) {
					return ((TestCase)element).getRemoteManagement().replaceAll("\n"," ");
				}
			});	
			
			//Test case Description
			vcl = new TableViewerColumn(viewer,SWT.WRAP);
			column = vcl.getColumn();
			column.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent arg0) {
					sorterTableViewer(viewer,2,tableSorter);
					tableSorter = !tableSorter;
				}
			});
			vcl.setEditingSupport(new TestCasesEditingSupport(viewer,TestCasesEditingSupport.ChangeDescription,this));
			column.setText("Description");			
			column.setWidth(200);
			column.setToolTipText("The descrtiption of the Test Case");
			column.setResizable(true);
			column.setMoveable(true);
			vcl.setLabelProvider(new ColumnLabelProvider() {
				@Override
				public String getText(Object element) {
					return ((TestCase)element).getDescription().replaceAll("\n"," ");
				}
			});	
			viewer.setInput(us);
			
			// Add Test Cases Button
			Button testCaseEdit = new Button(testCasesButtons, SWT.PUSH);		
			testCaseEdit.addSelectionListener(new SelectionAdapter() {
				boolean open = false;
				@Override
				public void widgetSelected(SelectionEvent arg0) {
					if(open){ // don't open second
						return;
					}
					open = true;
					EditTestCases testCasesDialog = new EditTestCases(parent.getShell(),(UserStory)us);
					int dialogCodeExit = testCasesDialog.open();
					if (dialogCodeExit == Window.OK) {
						dirty = true;
						firePropertyChange(PROP_DIRTY);
						newUs = testCasesDialog.getUserStory();
						viewer.setInput(newUs);
						open = false;
					}
					//Be careful! This if will be true if only the user closes the dialog from the "x" button
					//The Cancel button does not have id = Window.Cancel
					if(dialogCodeExit == Window.CANCEL){
						open = false;						
					}
				}
			});
			testCaseEdit.setImage(Activator.getImageDescriptor("icons/add.png").createImage());
			testCaseEdit.setText("Edit Test Cases");
			testCaseEdit.setToolTipText(
					"Adds a new Test Case to this User Story. A Test Case consists of 3 fields (when, then, end)");
			
			Button toggle = new Button(testCasesButtons,SWT.CHECK);
			toggle.setText("Show All");
			toggle.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent event) {
					if(((Button)event.getSource()).getSelection()){
						cases.setVisible(true);
					}else{				
						cases.setVisible(false);						
					}
				}
			});
			
			
			
		/*
			cases.setVisible(ToolPreferences.getPreference(ToolPreferences.TEST_CASES_IN_EDITOR));
			toggle.setSelection(ToolPreferences.getPreference(ToolPreferences.TEST_CASES_IN_EDITOR));	
		*/
		}
	}
	private void sorterTableViewer(TableViewer viewer ,int sortBy , boolean sorter){
		if(!(us instanceof UserStory)){
			return;
		}
		ArrayList<TestCase> tests = new ArrayList<TestCase>();
		tests.addAll(((UserStory)us).getTestCases());
		ArrayList<TestCase> sortedTests = new ArrayList<TestCase>();
		
		while(tests.size() > 0){
			int i = 0;
			TestCase first = tests.get(0);
			while(i < tests.size()){
				if(sortBy == 0){ //sort by id
					if(sorter){
						if(first.getId().compareTo(tests.get(i).getId()) < 0){				
							first = tests.get(i);
						}
					}else{				
						if(first.getId().compareTo(tests.get(i).getId()) > 0){				
							first = tests.get(i);
						}
					}
				}else if(sortBy == 1){ //sort by expected results					
					if(sorter){
						if(first.getExpectedResults().compareTo(tests.get(i).getExpectedResults()) < 0){				
							first = tests.get(i);														
						}
					}else{				
						if(first.getExpectedResults().compareTo(tests.get(i).getExpectedResults()) > 0){				
							first = tests.get(i);														
						}
					}
					
				}else if(sortBy == 2){ //sort by description results					
					if(sorter){
						if(first.getDescription().compareTo(tests.get(i).getDescription()) < 0){				
							first = tests.get(i);														
						}
					}else{				
						if(first.getDescription().compareTo(tests.get(i).getDescription()) > 0){				
							first = tests.get(i);														
						}
					}
					
				}else if(sortBy == 3){ //sort by Remote Management results					
					if(sorter){
						if(first.getRemoteManagement().compareTo(tests.get(i).getRemoteManagement()) < 0){				
							first = tests.get(i);														
						}
					}else{				
						if(first.getRemoteManagement().compareTo(tests.get(i).getRemoteManagement()) > 0){				
							first = tests.get(i);														
						}
					}
					
				}
				i++;
			}
			tests.remove(first);
			sortedTests.add(first);		
		}
		((UserStory)us).getTestCases().clear();
		((UserStory)us).getTestCases().addAll(sortedTests);
		viewer.setInput(us);
	}
class ViewContentProvider implements IStructuredContentProvider {
		
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		
		public Object[] getElements(Object inputElement) {
			if(inputElement instanceof UserStory){
				ArrayList<TestCase> tsF = ((UserStory)inputElement).getTestCases();
				return tsF.toArray();
			}
			return null;			
		}
		
		public ViewContentProvider getInstance()
		{
			return this;
		}
	}
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			return getText(obj);
		}
		public Image getColumnImage(Object obj, int index) {
			return getImage(obj);
		}
		public Image getImage(Object obj) {
			return Activator.getImageDescriptor("icons/userstory_default.png").createImage();
		}
	}
	@Override
	public void setFocus() {
		if (nameText != null) {
			nameText.setFocus();
		}
	}

	public void SetEditorName(String name) {
		this.setPartName(name);
	}
	public void dirty(){
		dirty = true;
		firePropertyChange(PROP_DIRTY);
	}
	public ArrayList<Integer> getNewActors()
	{
		for(int i : editorActors){
			if( i == ActorManager.ACTOR_NONE_ID){
				editorActors.remove(new Integer(i));
			}
		}
		if(editorActors.size() == 0)
		{
			editorActors.add(ActorManager.ACTOR_NONE_ID);
		}
		((UserStory)newUs).setActors(editorActors);
		return this.editorActors;
	}
	public Model getUserStory() {
		return newUs;
	}	
}