package links;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaModelException;

/**
 * code source: https://www.eclipse.org/forums/index.php/t/82769/ 
 */
public class JavaProjectUtils
{

	public static Object[] getPackages(IJavaProject project)
	{
		Set<Object> packages = new HashSet<Object>();
		Set<String> packageNames = new HashSet<String>();

		if (project != null && project.exists())
		{
			for (IPackageFragmentRoot aSourceRoot : getSourceRoots(project))
			{
				try
				{
					for (IJavaElement aPackage : aSourceRoot.getChildren())
					{
						String packageName = aPackage.getElementName();

						if (checkDefaultPackage(packageName) && checkDuplicatePackage(packageName, packageNames))
						{
							packageNames.add(packageName);
							packages.add(aPackage);
						}
					}
				}
				catch (JavaModelException ex)
				{
					// SkywayUIPlugin.getDefault().logError("Error getting
					// packages from source root " + aSourceRoot, ex);
					// //$NON-NLS-1$
				}
			}
		}

		return packages.toArray();
	}

	@SuppressWarnings("unused")
	protected static boolean checkDefaultPackage(String packageName)
	{
		// Skip the default package unless showDefaultPacakge is true
		return true || !"".equals(packageName); //$NON-NLS-1$
	}

	@SuppressWarnings("unused")
	protected static boolean checkDuplicatePackage(String packageName, Set<String> packageNames)
	{
		// Only allow duplicates if showDuplicateNames is true
		return true || !packageNames.contains(packageName);
	}

	protected static IPackageFragmentRoot[] getSourceRoots(IJavaProject project)
	{
		ArrayList<IPackageFragmentRoot> result = new ArrayList<IPackageFragmentRoot>();
		try
		{
			IPackageFragmentRoot[] roots = project.getPackageFragmentRoots();
			for (int i = 0; i < roots.length; i++)
			{
				if (roots[i].getKind() == IPackageFragmentRoot.K_SOURCE)
				{
					result.add(roots[i]);
				}
			}
		}
		catch (JavaModelException ex)
		{

		}

		return result.toArray(new IPackageFragmentRoot[result.size()]);
	}

}
