package links;

import org.eclipse.core.resources.IFile;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;

import plugin.Controller;
import plugin.LogWriter;

/**
 * MemberLineFinder
 * Given an IEditorPart and a Member Name, this selects said member in the editor.
 * @author theo
 */
public class MemberLineFinder
{
	public static void findMember(IEditorPart editorPart, String className)
	{
		LogWriter.info("MemberLineFinder: findMember(): Awake for Type \"" + className + "\"");
		int offset = -1;
		int length = -1;
		boolean found = false;
		
		/*
		 * https://stackoverflow.com/a/33565231
		 */
		IFile file = ((FileEditorInput) editorPart.getEditorInput()).getFile();
		ICompilationUnit compilationUnit = (ICompilationUnit) JavaCore.create(file);
		LogWriter.info("MemberLineFinder: findMember(): Created CompilationUnit");
		 
		try
		{
			for (IType type : compilationUnit.getAllTypes())
			{
				if (Controller.javaElementIsType(type.getElementType()) && type.getElementName().equals(className))
				{
					LogWriter.info("MemberLineFinder: findMember(): Successfuly found Type \"" + type.getElementName() + "\"");
					found = true;
					offset = type.getNameRange().getOffset();
					length = type.getNameRange().getLength();
					break;
				}
			}
		} catch (JavaModelException e)
		{
			LogWriter.high("MemberLineFinder: findMember(): JavaModelException occured for className \"" + className + "\"");
		}
		
		if (found)
		{
			LogWriter.info("MemberLineFinder: findMember(): Selecting offset " + offset + " and length " + length);
			((ITextEditor) editorPart).selectAndReveal(offset, length);
		}
	}
	
	public static void findMember(IEditorPart editorPart, String className, String methodName)
	{
		LogWriter.info("MemberLineFinder: findMember(): Awake for Type \"" + className + "\" and Method \"" + methodName + "\"");
		int offset = -1;
		int length = -1;
		boolean foundMethod = false;
		
		/*
		 * https://stackoverflow.com/a/33565231
		 */
		IFile file = ((FileEditorInput) editorPart.getEditorInput()).getFile();
		ICompilationUnit compilationUnit = (ICompilationUnit) JavaCore.create(file);
		LogWriter.info("MemberLineFinder: findMember(): Created CompilationUnit");
		 
		try
		{
			for (IType type : compilationUnit.getAllTypes())
			{
				if (Controller.javaElementIsType(type.getElementType()) && type.getElementName().equals(className))
				{
					LogWriter.info("MemberLineFinder: findMember(): Successfuly found Type \"" + type.getElementName() + "\". About to search for method \"" + methodName + "\"");
					for (IMethod method : type.getMethods())
					{
						if (method.getElementName().equals(methodName))
						{
							LogWriter.info("MemberLineFinder: findMember(): Successfuly found Method \"" + methodName + "\"");
							offset = method.getNameRange().getOffset();
							length = method.getNameRange().getLength();
							foundMethod = true;
							break;
						}
					}
					break;
				}
			}
		} catch (JavaModelException e)
		{
			LogWriter.high("MemberLineFinder: findMember(): JavaModelException occured for className \"" + className + "\"");
		}
		
		if (foundMethod)
		{
			LogWriter.info("MemberLineFinder: findMember(): Selecting offset " + offset + " and length " + length);
			((ITextEditor) editorPart).selectAndReveal(offset, length);
		}
	}
}
