package links;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.ICodeAssist;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.javaeditor.IClassFileEditorInput;
import org.eclipse.jdt.internal.ui.javaeditor.WorkingCopyManager;
import org.eclipse.jdt.internal.ui.text.JavaWordFinder;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextOperationTarget;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import plugin.Controller;
import plugin.LogWriter;

@SuppressWarnings("restriction")
public class RightClickContextManager
{
	public static IEditorPart getActiveEditor()
	{
		IWorkbenchWindow window= PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		if (window != null) {
			IWorkbenchPage page= window.getActivePage();
			if (page != null) {
				return page.getActiveEditor();
			}
		}
		return null;
	} 

	public static ICodeAssist getCodeAssist(IEditorPart fEditor)
	{
		if (fEditor != null) {
			IEditorInput input= fEditor.getEditorInput();
			if (input instanceof IClassFileEditorInput) {
				IClassFileEditorInput cfeInput= (IClassFileEditorInput) input;
				return cfeInput.getClassFile();
			}

			WorkingCopyManager manager= JavaPlugin.getDefault().getWorkingCopyManager();
			return manager.getWorkingCopy(input, false);
		}

		return null;
	}
	
	public static FoundJavaElement getSelectedMember()
	{
		FoundJavaElement parsedElement = null;
		//Hashtable<String, String> ret = null;
		IEditorPart activeEditor = getActiveEditor();
		if (activeEditor == null)
		{
			return new FoundJavaElement("", "" , Controller.NO_MEMBER);
		}
		ITextOperationTarget target = (ITextOperationTarget)activeEditor.getAdapter(ITextOperationTarget.class);
	    if (target instanceof ITextViewer)
	    {
	        ITextViewer textViewer = (ITextViewer)target;
	        
	        LogWriter.info("RightClickContextManager: Found TextViewer");
	        
	        ITextSelection selection= (ITextSelection)textViewer.getSelectionProvider().getSelection();
	        
	        LogWriter.info("RightClickContextManager: Got Selection Provider with selection: " + selection.getText());
	        
	        IRegion hoverRegion = JavaWordFinder.findWord(textViewer.getDocument(), selection.getOffset());
	        
	        if (hoverRegion==null)
	        {
				return new FoundJavaElement("", "" , Controller.NO_MEMBER);
	        }
	        LogWriter.info("RightClickContextManager: Found Region with length: " + hoverRegion.getLength());
	        
	        ICodeAssist resolve= getCodeAssist(activeEditor);
	        IJavaElement[] javaElementArray = null;
	        
			if (resolve != null) {
				try
				{
					javaElementArray = resolve.codeSelect(hoverRegion.getOffset(), hoverRegion.getLength());
				}
				catch (JavaModelException x)
				{
					LogWriter.info("RightClickContextManager: JavaModelException occured");
				}
			}
			
	        if (javaElementArray != null)
	        {
		        LogWriter.info("RightClickContextManager: Getting Elements with size " + javaElementArray.length);
		        if (javaElementArray.length>0)
		        {
		        	IJavaElement element = javaElementArray[0];
		        	String name = javaElementArray[0].getElementName();
		        	int type = javaElementArray[0].getElementType();
		        	IPath path = ((IFileEditorInput)activeEditor.getEditorInput()).getFile().getLocation().makeAbsolute();
		        	String absolutePathToFile = path.toString();
		        	
		        	parsedElement = new FoundJavaElement(absolutePathToFile, name, type);
		        	
		        	if (type == IJavaElement.METHOD)
		        	{
		        		parsedElement.setParent(element.getAncestor(IJavaElement.TYPE));
		        	}
		        	
		        	LogWriter.info("RightClickContextManager: Found element: " + parsedElement);
		        	//ret = new Hashtable<String, String>();
		        	//ret.put(key, value)
		        }
	        }
	        else
	        {
		        LogWriter.info("RightClickContextManager: No elements found. :(");
	        }
	    }
	    return parsedElement;
	    //return ret;
	}
	
	// Use Controller.getJavaElementDict() instead
	@Deprecated
	public static String elementTypeParse(int type)
    {
    	String x = "";

    	switch (type)
    	{
    	case IJavaElement.TYPE:
    		return "Class";
    	case IJavaElement.METHOD:
    		return "Method";
		default:
			x = Integer.toString(type);
			break;
    	}
    	
    	return x;
    }
		
	
}
