package model;

import java.nio.file.InvalidPathException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import flexjson.JSON;
import plugin.Controller;
import plugin.LogWriter;

/**
 * Describes a Linked Class (and optionally an accompanying Method).
 */
public class Link
{
	private String sourceFileRelativePath; //The absolute path to the linked .java file
	private String className;
	private List<String> methodNameList; //If Method list is empty, no Method has been linked
	
	public Link()
	{
		this.methodNameList = new ArrayList<String>();
	}
	
	public Link(String sourceFileAbsPath, String className)
	{
		this.sourceFileRelativePath = sourceFileAbsPath;
		this.className = className;
		this.methodNameList = new ArrayList<String>();
	}
	
	public Link(String sourceFileAbsPath, String className, List<String> methodNameList)
	{
		this.sourceFileRelativePath = sourceFileAbsPath;
		this.className = className;
		this.methodNameList = methodNameList;
	}
	@JSON(include = true)
	public List<String> getMethodNameList() {
		return methodNameList;
	}
	
	@JSON(include=false)
	public String getSourceFileAbsPath()
	{
		LogWriter.info("Link.getSourceFileAbsPath(): Returning " + Controller.toAbsolutePath(sourceFileRelativePath));
		return Controller.toAbsolutePath(sourceFileRelativePath);
	}
	
	public String getSourceFileRelativePath()
	{
		return sourceFileRelativePath;
	}
	
	@JSON(include=false)
	public String getSourceFilename()
	{
		try
		{
			Path p = Paths.get(sourceFileRelativePath);
			String file = p.getFileName().toString();
			return file;
		}
		catch (InvalidPathException e)
		{
			LogWriter.medium("Link.getSourceFilename(): invalid absolute path " + sourceFileRelativePath);
			return sourceFileRelativePath;
		}
	}

	public void setSourceFileAbsPath(String sourceFileAbsPath)
	{
		this.sourceFileRelativePath = sourceFileAbsPath;
	}

	public String getClassName()
	{
		return className;
	}

	public void setClassName(String className)
	{
		this.className = className;
	}
	

	public void setMethodNameList(List<String> methodNameList) {
		this.methodNameList = methodNameList;
	}
		
	public void addMethod(String method)
	{
		if (this.methodNameList==null)
		{
			this.methodNameList = new ArrayList<String>();
		}
		this.methodNameList.add(method);
	}

	@Override
	public String toString() {
		return "Link [sourceFileAbsPath=" + sourceFileRelativePath + ", className=" + className + ", methodNameList="
				+ methodNameList + "]";
	}
	
	
	
}
