package model;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class LinkStub extends Model
{
	public static final int TYPE_CLASS = 0;
	public static final int TYPE_METHOD = 1;
	
	private ArrayList<LinkStub> children;
	private Link linkReference;	
	private int linkType;
	public String parentUserStoryPath; //if type == 1 the parent UserStory
	
	public LinkStub(String className, List<String> methodNameList, String parentAbsolutePath, Link linkReference)
	{
		super(className, "");
		this.parentUserStoryPath = parentAbsolutePath;
		this.children = new ArrayList<LinkStub>();
		this.linkType = TYPE_CLASS;
		this.linkReference = linkReference;
		// setParentAbsolutePath for Parent US (Class Stub)
		this.setParentAbsolutePath(parentAbsolutePath);
		// This is the absolute path of the Class Stub
		String classStubAbsolutePath = parentAbsolutePath + "\\" + className;
		this.setAbsolutePath( classStubAbsolutePath);
		
		if (methodNameList!=null)
		{
			//Create Methods if a Method Name List exists.
			if (methodNameList.size()!=0)
			{
				for (String methodName : methodNameList)
				{
					//The Method Stub gets the Class Stub Abs Path as a Parent.
					LinkStub methodLinkStub = new LinkStub(this.parentUserStoryPath,methodName, classStubAbsolutePath, linkReference);
					addChild( methodLinkStub );
				}
				
			}
		}
	}
	
	public LinkStub(String parentUserStoryPath,String methodName, String parentAbsolutePath, Link linkReference)
	{
		super(methodName, "");

		this.parentUserStoryPath = parentUserStoryPath;
		this.linkReference = linkReference;
		
		this.children = new ArrayList<LinkStub>();
		this.linkType = TYPE_METHOD;
		
		this.setParentAbsolutePath(parentAbsolutePath);
		this.setAbsolutePath( parentAbsolutePath + "\\" + methodName);
	}
	
	public void addChild(LinkStub child)
	{
		if (this.children==null)
		{
			this.children = new ArrayList<LinkStub>();
		}
		this.children.add(child);
	}

	
	/*public void generateAbsolutePath(String previousString)
	{
		String parentAbsolutePath = this.getParentAbsolutePath();
		if ( parentAbsolutePath != null )
		{
			this.setAbsolutePath( parentAbsolutePath );
		}
	}*/
	
	public Link getLinkReference() {
		return linkReference;
	}

	public void setLinkReference(Link linkReference) {
		this.linkReference = linkReference;
	}

	public int getLinkType()
	{
		return linkType;
	}
	@Override
	public Object[] getChildren()
	{
		return children.toArray();
	}
	
	public boolean isClassLink()
	{
		return (this.linkType == TYPE_CLASS);
	}
	
	public boolean isMethodLink()
	{
		return (this.linkType == TYPE_METHOD);
	}
	
	@Override
	public String toString()
	{
		String str = "LinkStub <" + getName() + ">: \'" + getDescription() + "\' - Parent: \'"+this.getParentAbsolutePath()+"\' - Children: [";
		for (LinkStub o : this.children)
		{
			str+= o.toString() + " ";				
		}
		str += "]";
		str += "reference: " + this.linkReference;
		return str;
	}

	@Override
	public void saveToDisk() { return; }

	@Override
	public void saveToDisk(String absolutePath) { return; }
	
	@Override
	public void saveToDisk(String json, String absolutePath) { return; }
	

}
