package model;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;

import flexjson.JSONSerializer;
import plugin.LogWriter;
import plugin.serialization.JsonManager;
import plugin.serialization.Jsonable;

public abstract class Model implements Jsonable
{
	/*public static final String TYPE_USERSTORY = "UserStory";
	public static final String TYPE_SYSTEM = "System";
	public static final String TYPE_LINK = "Link";
	public static final String TYPE_ACTOR = "Actor";*/
	//protected transient final String eof = "\u001a";
	protected transient final String eof = "";
	protected transient final String USERSTORY_FILE_EXTENSION = ".us";
	
	private String name;
	private String description;
	private transient String absolutePath;
	private transient String parentAbsolutePath;
	private transient boolean hasError;

	public Model()
	{
	}
	
	public Model(String name, String description)
	{
		this.name = name;
		this.description = description;
		this.absolutePath = "";
		this.parentAbsolutePath = "";
	}
	
	public Model(String name, String description, String absolutePath, String parentAbsolutePath)
	{
		this.name = name;
		this.description = description;
		this.absolutePath = absolutePath;
		this.parentAbsolutePath = parentAbsolutePath;
	}
	public String getParentAbsolutePath() {
		return parentAbsolutePath;
	}

	public void setParentAbsolutePath(String parentAbsolutePath) {
		this.parentAbsolutePath = parentAbsolutePath;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getAbsolutePath() {
		return absolutePath;
	}

	public void setAbsolutePath(String absolutePath) {
		this.absolutePath = absolutePath;
	}

	public boolean isHasError() {
		return hasError;
	}

	public void setHasError(boolean hasError) {
		this.hasError = hasError;
	}	
	
	public Object[] getChildren()
	{
		return new Object[0];
	}
	
	public String toJson()
	{		
		return JsonManager.toJson(this);
		//return new JSONSerializer().prettyPrint(true).exclude("hasError").exclude("absolutePath").exclude("parentAbsolutePath").serialize( this );
	}
	
	public void saveToDisk() throws IOException
	{
		saveToDisk(getAbsolutePath());
	}
	
	public void saveToDisk(String absolutePath) throws IOException
	{
		try
		{
			if (absolutePath != null)
			{
				if (!absolutePath.equals(""))
				{
					File f = new File(absolutePath);
					if (!f.exists())
					{
						f.createNewFile();
					}
					else if (f.isDirectory())
					{
						return;
					}
					Writer bw = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(absolutePath), "UTF-8"));
					bw.write(JsonManager.toJson(this));
					bw.close();
				}
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Critical: Couldn't save to disk at path " + this.getAbsolutePath().toString());
			e.printStackTrace();
		}
	}
	
	public void saveToDisk(String json, String absolutePath) throws IOException
	{
		try
		{
			if (absolutePath != null)
			{
				if (!absolutePath.equals(""))
				{
					File f = new File(absolutePath);
					if (!f.exists())
					{
						f.createNewFile();
					}
					else if (f.isDirectory())
					{
						return;
					}
					Writer bw = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(absolutePath), "UTF-8"));
					bw.write(json);
					bw.close();
				}
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Critical: Couldn't save to disk at path " + this.getAbsolutePath().toString());
			e.printStackTrace();
		}
	}
	
	public int hashCode()
    {
        return absolutePath.hashCode();
    }
	public Model getCopy(){
		Model copy = new UserStory();
		copy.setAbsolutePath(this.getAbsolutePath());
		copy.setDescription(this.getDescription());
		copy.setHasError(this.isHasError());
		copy.setName(this.getName());
		copy.setParentAbsolutePath(this.getParentAbsolutePath());
		return copy;
	}
    public boolean equals (Object obj)
    {    	
    	if(obj == null){
    		return false;
    	}
        if (obj instanceof Model == false)
            return false;

        return absolutePath.equals(((Model)obj).absolutePath);
    }

}
