package model;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import plugin.Database;
import plugin.LogWriter;
import plugin.serialization.JsonManager;

public class UsSystem extends Model
{
	public static final transient String ROOT_ABSPATH = "ROOT_Userstory";
	
	private transient ArrayList<Model> children;
	
	public UsSystem()
	{
		super();
		children = new ArrayList<Model>();
	}
	
	public UsSystem(String name, String description)
	{
		super(name, description);
		children = new ArrayList<Model>();
	}
	
	public UsSystem(String name, String description, String absolutePath, String parentAbsolutePath)
	{
		super(name, description, absolutePath, parentAbsolutePath);
		children = new ArrayList<Model>();
	}

	@Override
	public Object[] getChildren()
	{
		ArrayList<Model> sorted_list = new ArrayList<Model>();
		for (Model m : children)
		{
			if (m instanceof UsSystem)
			{
				sorted_list.add(m);
			}
		}
		for (Model m : children)
		{
			if (m instanceof UserStory)
			{
				sorted_list.add(m);
			}
		}
		
		return sorted_list.toArray();
	}
	
	public ArrayList<Model> getChildrenList()
	{
		return children;
	}

	public void setChildren(ArrayList<Model> children)
	{
		this.children = children;
	}
	
	public void addChild(Model child)
	{
		if (this.children==null)
		{
			this.children = new ArrayList<Model>();
		}
		this.children.add(child);
	}

	@Override
	public String toString()
	{
		String str = "UsSystem Name:" + getName() + ", Description: " + getDescription() + " Children: [";
		for (Model o : this.getChildrenList())
		{
			str+= o.toString() + " ";				
		}
		str += "]";
		return str;
	}
	@Override
	public UsSystem getCopy() {
		UsSystem copy = new UsSystem();
		copy.setChildren(this.getChildrenList());
		copy.setAbsolutePath(this.getAbsolutePath());
		copy.setDescription(this.getDescription());
		copy.setHasError(this.isHasError());
		copy.setName(this.getName());
		copy.setParentAbsolutePath(this.getParentAbsolutePath());
		return copy;
	}
	
	@Override
	public void saveToDisk() throws IOException
	{
		LogWriter.debug("UsSystem: saveToDisk(): Awake");
		try
		{
			if (this.getAbsolutePath() != null)
			{
				LogWriter.debug("UsSystem: saveToDisk(): getAbsolutePath != null");
				if (!this.getAbsolutePath().equals(""))
				{
					LogWriter.debug("UsSystem: saveToDisk(): getAbsolutePath != \"\"");
					File dir = new File(this.getAbsolutePath());
					if (!dir.exists())
					{
						if(!dir.mkdirs()){
							LogWriter.high("problima....");
						}
					}
					String system_file_path = this.getAbsolutePath() + "\\" + Database.SYSTEM_FILE_NAME;
					File f = new File(system_file_path);
					if (!f.exists())
					{
						f.createNewFile();
					}
					LogWriter.debug("UsSystem: saveToDisk(): About to open Writer");
					Writer bw = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(system_file_path), "UTF-8"));
					LogWriter.debug("UsSystem: saveToDisk(): About to write to file");
					bw.write(this.toJson() + super.eof);
					bw.close();
					/*try(Writer writer = new OutputStreamWriter(new FileOutputStream(system_file_path) , "UTF-8"))
					{
			            Gson gson = new GsonBuilder().create();
			            gson.toJson(this);
			        }
					LogWriter.debug("UsSystem: saveToDisk(): Closed Writer");*/
				}
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Critical: Couldn't save to disk at path " + this.getAbsolutePath().toString());
			e.printStackTrace();
		}
	}
	
	@Override
	public void saveToDisk(String absolutePath) throws IOException
	{
		try
		{
			if (absolutePath != null)
			{
				if (!absolutePath.equals(""))
				{
					File dir = new File(absolutePath);
					if (!dir.exists())
					{
						if(!dir.mkdirs()){
							LogWriter.info("problima....");
						}
					}
					String system_file_path = absolutePath + "\\" + Database.SYSTEM_FILE_NAME;
					File f = new File(system_file_path);
					if (!f.exists())
					{
						f.createNewFile();
					}
					Writer bw = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(system_file_path), "UTF-8"));
					bw.write(JsonManager.toJson(this));
					bw.close();
				}
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Critical: Couldn't save to disk at path " + this.getAbsolutePath().toString());
			e.printStackTrace();
		}
	}
	
	@Override
	public void saveToDisk(String json, String absolutePath) throws IOException
	{
		try
		{
			if (absolutePath != null)
			{
				if (!absolutePath.equals(""))
				{
					File dir = new File(absolutePath);
					if (!dir.exists())
					{
						if(!dir.mkdirs()){
							LogWriter.info("problima....");
						}
					}
					String system_file_path = absolutePath + "\\" + Database.SYSTEM_FILE_NAME;
					File f = new File(system_file_path);
					if (!f.exists())
					{
						f.createNewFile();
					}
					Writer bw = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(system_file_path), "UTF-8"));
					bw.write(json);
					bw.close();
				}
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Critical: Couldn't save to disk at path " + absolutePath);
			e.printStackTrace();
		}
	}
	
	public static String folderPathToSystemFile(String folderPath)
	{
		return folderPath + "\\" + Database.SYSTEM_FILE_NAME;
	}
	
	
}