package model;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;

import actors.ActorManager;
import flexjson.JSONSerializer;
import plugin.Controller;
import plugin.LogWriter;
import plugin.serialization.JsonManager;

public class UserStory extends Model
{
	private ArrayList<Integer> actors;
	private ArrayList<TestCase> testCases;
	private ArrayList<Link> links;
	
	public UserStory()
	{
		super("Untitled User Story", "");
		actors = new ArrayList<Integer>();
		actors.add(ActorManager.ACTOR_NONE_ID);
		testCases = new ArrayList<TestCase>();
		links = new ArrayList<Link>();
	}
	
	public UserStory(String name, String description)
	{
		super(name, description);
		actors = new ArrayList<Integer>();
		testCases = new ArrayList<TestCase>();
		links = new ArrayList<Link>();
		actors.add(ActorManager.ACTOR_NONE_ID);
	}
	
	public UserStory(String name, String description, String absolutePath, String parentAbsolutePath)
	{
		super(name, description, absolutePath, parentAbsolutePath);
		actors = new ArrayList<Integer>();
		testCases = new ArrayList<TestCase>();
		links = new ArrayList<Link>();
		actors.add(ActorManager.ACTOR_NONE_ID);
	}
	
	public ArrayList<TestCase> getTestCases()
	{
		return this.testCases;
	}

	public void setTestCases(ArrayList<TestCase> testCases)
	{
		this.testCases = testCases;
	}

	public ArrayList<Integer> getActors() {
		return actors;
	}

	public void setActors(ArrayList<Integer> actors) {
		this.actors = actors;
	}
	
	public void addActor(int actorId)
	{
		if (!this.actors.contains(actorId))
		{
			this.actors.add(actorId);
		}
	}

	public ArrayList<Link> getLinks()
	{
		return links;
	}
	
	public void addLink(String sourceAbsolutePath, String memberName, String parentMemberName)
	{
		System.out.println(memberName+"???");
		System.out.println(sourceAbsolutePath+"???");
		System.out.println(parentMemberName+"???");
		
		String sourcePath = Controller.toRelativePath(sourceAbsolutePath);
		LogWriter.info("UserStory: addLink: New Path: " + sourcePath);
		
		Link classLink = null;
		boolean createClassLink = false;

		// If the Parent Member Name is not "", we're going to add a Method Link.
		// Thus, we need to make sure this class has not already been linked.
		if (!parentMemberName.equals(""))
		{
			LogWriter.info("UserStory: addLink: Started with Class Name: " + parentMemberName + " and Method Name: " + memberName);
			
			// Find existing Class Link (if it exists)
			for (Link l : this.getLinks())
			{
				if (parentMemberName.equals(l.getClassName()) && sourcePath.equals(l.getSourceFileRelativePath()))
				{
					LogWriter.info("UserStory: addLink: Detected existing Class Link (method: YES)");
					classLink = l;
				}
			}
			
			// If there is no existing Class Link, we need to create it (thus this becomes true)
			createClassLink = (classLink == null);
			if (createClassLink)
			{
				LogWriter.info("UserStory: addLink: Creating new Class Link " + parentMemberName + " with method");
				classLink = new Link(sourcePath, parentMemberName);
				LogWriter.info("UserStory: addLink: Adding method " + memberName + " to new class Link " + classLink.toString());
				classLink.addMethod(memberName);
				this.links.add(classLink);
				LogWriter.info("UserStory: addLink: About to save new ClassLink with Method");
				try { this.saveToDisk(); } catch (IOException e) {
					e.printStackTrace();
				}
				LogWriter.info("UserStory: addLink: User Story \""+this.getName()+"\" Saved with new Link: " + classLink.toString());
				return;
			}
			else
			{
				// Find whether method is already linked
				for (String method : classLink.getMethodNameList())
				{
					if (method.equals(memberName))
					{
						LogWriter.info("UserStory: addLink: Detected duplicate method name \"" + memberName + "\" on existing Class Link with name " + parentMemberName + ". Aborting");
						return;
					}
				}
				
				LogWriter.info("UserStory: addLink: Adding method " + memberName + " to existing class Link " + classLink.toString());
				classLink.addMethod(memberName);
				LogWriter.info("UserStory: addLink: About to save new ClassLink with Method");
				try { this.saveToDisk(); } catch (IOException e) {e.printStackTrace(); }
				LogWriter.info("UserStory: addLink: User Story \""+this.getName()+"\" Saved with new Link: " + classLink.toString());
				return;
			}
			
		}
		else
		{
			LogWriter.info("UserStory: addLink: Started with Class Name: " + memberName);
			// Find existing Class Link (if it exists)
			for (Link l : this.getLinks())
			{
				if (memberName.equals(l.getClassName()) && sourcePath.equals(l.getSourceFileRelativePath()))
				{
					LogWriter.info("UserStory: addLink: Detected existing Class Link (method: NO). Aborting");
					return;
				}
			}
			
			// If we haven't returned yet, there's no existing Class Link.
			LogWriter.info("UserStory: addLink: Creating new Class Link " + memberName);
			classLink = new Link(sourcePath, memberName);
			this.links.add(classLink);
			LogWriter.info("UserStory: addLink: About to save new ClassLink");
			try { this.saveToDisk(); } catch (IOException e) {e.printStackTrace(); }
			LogWriter.info("UserStory: addLink: User Story \""+this.getName()+"\" Saved with new Link: " + classLink.toString());
			return;
		}
		
		/*
		// If the Parent Member Name is not "", we're going to add a Method Link.
		// Thus, we need to make sure this class has not already been linked.
		if (!parentMemberName.equals(""))
		{
			Link foundExistingClassLink = null;
			LogWriter.info("UserStory: addLink: Found parent member name: " + parentMemberName);
			for (Link l : this.links)
			{
				// If the Class and Path matches, add this method here
				if (parentMemberName.equals(l.getClassName()) && sourcePath.equals(l.getSourceFileRelativePath()))
				{
					LogWriter.info("UserStory: addLink: This Class is already linked");
					foundExistingClassLink = l;
					//return;
				}
			}
			
			if (foundExistingClassLink != null)
			{
				for (String methodName : foundExistingClassLink.methodNameList)
				{
					if (methodName.equals(memberName))
					{
						LogWriter.info("UserStory: addLink: Attempted to add duplicate method '\'" + memberName + "\' to Class: " + parentMemberName);
						return;
					}
				}
 				LogWriter.info("UserStory: addLink: Adding method '\'" + memberName + "\' to existing Class: " + parentMemberName);
				foundExistingClassLink.addMethod(memberName);
				//links.add(foundExistingClassLink);
				try { this.saveToDisk(); } catch (IOException e) { }
				return;
			}

			LogWriter.info("UserStory: addLink: No existing class found with member name " + memberName + " parent member name: " + parentMemberName);
			Link addThisLink = new Link(sourcePath, parentMemberName);
			addThisLink.addMethod(memberName);
			this.links.add(addThisLink);
			try { this.saveToDisk(); } catch (IOException e) { }
		}
		else
		{
			for (Link l : this.links)
			{
				if (memberName.equals(l.getClassName()) && sourcePath.equals(l.getSourceFileAbsPath()))
				{
					LogWriter.warning("Attempted to create duplicate link");
					return;
				}
			}
			Link addThisLink = new Link(sourcePath, memberName);
			this.links.add(addThisLink);
			try { this.saveToDisk(); } catch (IOException e) { }
		}*/
	}

	public void setLinks(ArrayList<Link> links)
	{
		this.links = links;
	}
	
	@Override
	public Object[] getChildren()
	{
		ArrayList<Model> linkStubs = new ArrayList<Model>();
		for (Link l : links)
		{
			LinkStub stub = new LinkStub(l.getClassName(), l.getMethodNameList(), this.getAbsolutePath(), l);
			linkStubs.add(stub);
		}
		return linkStubs.toArray();
	}
	
	@Override
	public String toJson()
	{
		return JsonManager.toJson(this);

        //return new JSONSerializer().prettyPrint(true).exclude("hasError").exclude("absolutePath").exclude("parentAbsolutePath").include("links").include("actors").include("testCases").serialize( this );
	}
	
	@Override	
	public void saveToDisk() throws IOException
	{
		LogWriter.info("UserStory: saveToDisk: About to save");
		saveToDisk(this.getAbsolutePath());
		
		/*
		
		String absPath = this.getAbsolutePath();
		if(!absPath.endsWith(super.USERSTORY_FILE_EXTENSION))
		{
			absPath += super.USERSTORY_FILE_EXTENSION;
		}
		this.setAbsolutePath(absPath);
		try
			{File parent = new File(this.getParentAbsolutePath());
			if(!parent.exists()){
				parent.mkdirs();
			}
			if (absPath != null)
			{
				if (!absPath.equals(""))
				{
					File f = new File(absPath);
					if (!f.exists())
					{
						f.createNewFile();
					}
					else if (f.isDirectory())
					{
						return;
					}
					Writer bw = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(absPath), "UTF-8"));
					bw.write(JsonManager.toJson(this));
					bw.close();
				}
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Critical: Couldn't save to disk at path " + this.getAbsolutePath().toString());
			e.printStackTrace();
		}
		finally
		{
			for (Link l : this.links)
			{
				LogWriter.info("UserStory: saveToDisk: Saving Link " + l.toString());
			}
		}*/
	}
	
	@Override
	public void saveToDisk(String absolutePath) throws IOException
	{
		LogWriter.info("UserStory: saveToDisk: About to save");
		if(!absolutePath.endsWith(super.USERSTORY_FILE_EXTENSION))
		{
			absolutePath += super.USERSTORY_FILE_EXTENSION;
		}
		this.setAbsolutePath(absolutePath);
		try
		{
			File parent = new File(this.getParentAbsolutePath());
			if(!parent.exists()){
				parent.mkdirs();
			}
			if (absolutePath != null)
			{
				if (!absolutePath.equals(""))
				{
					File f = new File(absolutePath);
					if (!f.exists())
					{
						f.createNewFile();
					}
					else if (f.isDirectory())
					{
						return;
					}
					Writer bw = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(absolutePath), "UTF-8"));
					bw.write(JsonManager.toJson(this));
					bw.close();
				}
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Critical: Couldn't save to disk at path " + this.getAbsolutePath().toString());
			e.printStackTrace();
		}
		finally
		{
			for (Link l : this.links)
			{
				LogWriter.info("UserStory: saveToDisk: Saving Link " + l.toString());
			}
		}
	}
	
	@Override
	public void saveToDisk(String json, String absolutePath) throws IOException
	{
		if(!absolutePath.endsWith(super.USERSTORY_FILE_EXTENSION))
		{
			absolutePath += super.USERSTORY_FILE_EXTENSION;
		}
		this.setAbsolutePath(absolutePath);
		try
		{
			File parent = new File(this.getParentAbsolutePath());
			if(!parent.exists()){
				parent.mkdirs();
			}
			if (absolutePath != null)
			{
				if (!absolutePath.equals(""))
				{
					File f = new File(absolutePath);
					if (!f.exists())
					{
						f.createNewFile();
					}
					else if (f.isDirectory())
					{
						return;
					}
					Writer bw = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(absolutePath), "UTF-8"));
					bw.write(json);
					bw.close();
				}
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Critical: Couldn't save to disk at path " + this.getAbsolutePath().toString());
			e.printStackTrace();
		}
		finally
		{
			for (Link l : this.links)
			{
				LogWriter.info("UserStory: saveToDisk: Saving Link " + l.toString());
			}
		}
	}
	
	

	@Override
	public String toString() {
		String s = "UserStory - Name: ";
		s += this.getName();
		s += ", Description: ";
		s += this.getDescription();
		s+=", Actors: [";
		for (int a : this.actors)
		{
			s += Integer.toString(a);
			s += ", ";
		}
		s = s.substring(0, s.length()-2); //remove last ", "
		s+="], Test Cases: [";
		for (TestCase tf : this.testCases)
		{
			s += tf.toString();
		}
		s+="], Links: [";
		for (Link tf : this.links)
		{
			s += tf.toString();
		}
		s+="]";
		return s;
	}
	@Override
	public UserStory getCopy() {
		UserStory copy = new UserStory();
		copy.getActors().addAll(this.actors);
		copy.getLinks().addAll(this.links);
		copy.getTestCases().addAll(this.testCases);
		copy.setAbsolutePath(this.getAbsolutePath());
		copy.setDescription(this.getDescription());
		copy.setHasError(this.isHasError());
		copy.setName(this.getName());
		copy.setParentAbsolutePath(this.getParentAbsolutePath());
		return copy;
	}	
}
