package plugin;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jdt.core.IJavaElement;

import actors.ActorManager;
import model.Project;
import plugin.config.PreferencesManager;
//import plugin.config.UserStoryToolPreferenceManager;
import plugin.serialization.JsonManager;
import ui.UserStoryTreeView;

public class Controller implements Runnable{

	public static Thread THREAD_CONTROLLER;
	public static Controller instance;
	private Database database;
	final String workplace = ResourcesPlugin.getWorkspace().getRoot().getLocation().toString();
	public static String projectPath;

	public final static String PATH_TOOL   = "/userstoriestool/";
	public final static String PATH_FILE_PREFERENCES_LEGACY   = "/userstoriestool/preferences.txt";
	public final static String PATH_FILE_PREFERENCES   = "/userstoriestool/projectpreferences.txt";
	public final static String PATH_FILE_ACTORS        = "/userstoriestool/actors.txt";
	public final static String PATH_DIR_US_ROOT        = "/userstoriestool/stories";
	public final static String PATH_FILE_LOG           = "/userstoriestool/UserStoriesTool.log";
	
	public final static int NO_MEMBER = -1;
	
	public static Hashtable<Integer, String> javaElementDict;
	
	public Controller ()
	{
		instance = this;
		
		// Instantiate GSON instance
		new JsonManager();
		
		THREAD_CONTROLLER = new Thread(this, "Controller Thread");
		//THREAD_CONTROLLER.start();
	}
	
	public Controller (String newProjectPath)
	{
		instance = this;
		projectPath = newProjectPath;
		
		LogWriter.info("Instantiated with " + newProjectPath);
		
		THREAD_CONTROLLER = new Thread(this, "Controller Thread");
		//THREAD_CONTROLLER.start();
	}
	
	public Project checkDefaultProject()
	{
		LogWriter.debug("Controller: checkDefaultProject(): Awake");
		new ProjectDatabase();
	    
		while (ProjectDatabase.controllerWaitForMe)
		{
			try
			{
				LogWriter.debug("Controller: checkDefaultProject(): Waiting for ProjectDatabase");
				Thread.sleep(100);
			}
			catch (InterruptedException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		for (Object o : ProjectDatabase.rootProject.getChildren())
		{
			Project p = (Project)o;
			if (p.isDefault())
			{
				LogWriter.debug("Controller: checkDefaultProject(): Returning Default " + p.getName());
				return p;
			}
		}

		LogWriter.debug("Controller: checkDefaultProject(): No default project was found");
		return null;
		
	}
	
	/**
	 * Single reference to a dictionary that takes the type of an IJavaElement and returns the corresponding String.
	 * @param x
	 * @return
	 */
	public static String getJavaElementDict(int x)
	{
		if (javaElementDict == null || javaElementDict.get(x) == null)
		{
				javaElementDict = new Hashtable<Integer, String>();
				javaElementDict.put(IJavaElement.JAVA_MODEL, "Java Model");
				javaElementDict.put(IJavaElement.JAVA_PROJECT, "Java Project");
				javaElementDict.put(IJavaElement.PACKAGE_FRAGMENT_ROOT, "Package Fragment Root");
				javaElementDict.put(IJavaElement.PACKAGE_FRAGMENT, "Package Fragment");
				javaElementDict.put(IJavaElement.COMPILATION_UNIT, "Compilation Unit");
				javaElementDict.put(IJavaElement.CLASS_FILE, "Class File");
				javaElementDict.put(IJavaElement.TYPE, "Type");
				javaElementDict.put(IJavaElement.FIELD, "Field");
				javaElementDict.put(IJavaElement.METHOD, "Method");
				javaElementDict.put(IJavaElement.INITIALIZER, "Initializer");
				javaElementDict.put(IJavaElement.PACKAGE_DECLARATION, "Package Declaration");
				javaElementDict.put(IJavaElement.IMPORT_CONTAINER, "Import Container");
				javaElementDict.put(IJavaElement.IMPORT_DECLARATION, "Import Declaration");
				javaElementDict.put(IJavaElement.LOCAL_VARIABLE, "Local Variable");
				javaElementDict.put(IJavaElement.TYPE_PARAMETER, "Type Parameter");
				javaElementDict.put(IJavaElement.ANNOTATION, "Annotation");
				javaElementDict.put(NO_MEMBER, "");
		}
		return javaElementDict.get(x);
	}
	
	public static boolean javaElementIsType(int elementType)
	{
		return getJavaElementDict(elementType).equals("Type");
	}
	
	public static boolean javaElementIsMethod(int elementType)
	{
		return getJavaElementDict(elementType).equals("Method");
	}
	
	public File getFile(String fileDir)
	{
		return new File(projectPath + fileDir);
	}
	
	public void run()
	{
		
		if (projectPath==null)
		{
			if (UserStoryTreeView.currentProjectPath.equals("") && UserStoryTreeView.currentProjectPath!=null)
			{			
				LogWriter.info("Controller: run(): Attempted to create controller without Project");
				return;
			}
			
			if (projectPath.equals(UserStoryTreeView.currentProjectPath))
			{
				LogWriter.info("Controller: run(): Tried to run an already selected project");
				return;
			}
		}
		LogWriter.info("Controller: Spawning for path " + projectPath);
		
		
		File projectPathToolFolder = new File(projectPath  + PATH_TOOL);
		if (! (projectPathToolFolder.exists() || projectPathToolFolder.isDirectory()) )
		{
			LogWriter.info("Controller.run(): Setting up UserStoriesTool for path " + projectPath);
			firstTimeSetup();
		}
		else
		{
			createMissingFiles();
		}

		/*MyPreferencesLegacy.preferencesFilePath = projectPath + PATH_FILE_PREFERENCES_LEGACY;
		MyPreferencesLegacy.readFromFile(projectPath + PATH_FILE_PREFERENCES_LEGACY);*/

		//UserStoryToolPreferenceManager.readFromDisk(projectPath);
		
		this.database = new Database(projectPath + PATH_DIR_US_ROOT);
	}
	
	/**
	 * Creates a userstoriestool folder in a new project
	 */
	public void firstTimeSetup()
	{
		try
		{
			File temp = new File(projectPath  + PATH_TOOL);
			temp.mkdir();
			
			temp = new File(projectPath  + PATH_DIR_US_ROOT);
			temp.mkdir();
	
			//temp = new File(projectPath  + PATH_FILE_PREFERENCES_LEGACY);
			//temp.createNewFile();
			
			temp = new File(projectPath  + PATH_FILE_LOG);
			temp.createNewFile();
			LogWriter.logFilePath = projectPath  + PATH_FILE_LOG;
	
			//temp = new File(projectPath  + PATH_FILE_ACTORS);
			//temp.createNewFile();
			File actorsFile = new File(projectPath + PATH_FILE_ACTORS);
			if(!actorsFile.exists())
			{
				try
				{
					actorsFile.createNewFile();
					ActorManager.actorsFile = actorsFile;
					ActorManager.addActor(ActorManager.ACTOR_NONE_ID, ActorManager.ACTOR_NONE_NAME);
				}
				catch (IOException e)
				{
					LogWriter.high("UserStoryController: Controller: Couldn't create actors file : " + projectPath + PATH_FILE_ACTORS + " " + e.getMessage());
				}
			}
			else
			{
				ActorManager.actorsFile = actorsFile;
			}
		}
		catch (IOException e)
		{
			LogWriter.high("Controller.firstTimeSetup(): I/O Error: Couldn't setup project files");
			e.printStackTrace();
		}
	}
	
	/**
	 * Re-creates any missing important files from the userstoriestool of the current project.
	 */
	public void createMissingFiles()
	{
		try
		{
			File temp = new File(projectPath  + PATH_DIR_US_ROOT);
			if (!temp.exists())
			{
				temp.mkdir();
			}
	/*
			temp = new File(projectPath  + PATH_FILE_PREFERENCES_LEGACY);
			if (!temp.exists())
			{
				temp.createNewFile();
			}
		*/	
			temp = new File(projectPath  + PATH_FILE_LOG);
			if (!temp.exists())
			{
				temp.createNewFile();
			}
			LogWriter.logFilePath = projectPath  + PATH_FILE_LOG;
			
			temp = new File(projectPath + PATH_FILE_ACTORS);
			if(!temp.exists())
			{
				try
				{
					temp.createNewFile();
					ActorManager.actorsFile = temp;
					ActorManager.addActor(ActorManager.ACTOR_NONE_ID, ActorManager.ACTOR_NONE_NAME);
				}
				catch (IOException e)
				{
					LogWriter.high("UserStoryController: Controller: Couldn't create actors file : " + projectPath + PATH_FILE_ACTORS + " " + e.getMessage());
				}
			}
			else
			{
				ActorManager.actorsFile = temp;
			}
			
		}
		catch(IOException e)
		{
			System.err.println("Controller: Critical: IOException during createMissingFiles");
		}
	}
	
	public static Controller getInstance()
	{
		if (instance == null)
		{
			instance = new Controller();
		}
		return instance;
	}
	
	public static Database getDatabase()
	{
		return getInstance().database;
	}
	
	public void switchProject(Project selectedNode)
	{
		LogWriter.info("Controller: switchProject(): Initialized");
		if (projectPath!=null)
		{
			if (projectPath.equals(selectedNode.getProjectPath()))
			{
				LogWriter.info("Controller: switchProject(): Tried to switch to already selected project");
				return;
			}
		}
		projectPath = selectedNode.getProjectPath();
		UserStoryTreeView.switchProject(selectedNode.getName());
		this.run();
	}
	
	//Will not update the UserStoryTreeView
	public void switchProjectTestingPorpuses(Project selectedNode)
	{
		LogWriter.info("Controller: switchProject(): Initialized");
		if (projectPath!=null)
		{
			if (projectPath.equals(selectedNode.getProjectPath()))
			{
				LogWriter.info("Controller: switchProject(): Tried to switch to already selected project");
				return;
			}
		}
		projectPath = selectedNode.getProjectPath();
		//UserStoryTreeView.switchProject(selectedNode.getName());
		this.run();
	}
	
	public static String toAbsolutePath(String relativePath)
	{
		return projectPath+relativePath;
	}
	
	public static String toRelativePath(String absolutePath)
	{
		if ( absolutePath.split(projectPath)[1] != null)
		{
			LogWriter.info("Controller: toRelativePath: \'"+absolutePath+"\' converted to " + absolutePath.split(projectPath)[1]);
			return absolutePath.split(projectPath)[1];
		}
		else
		{
			LogWriter.info("Controller: toRelativePath: returning \'"+absolutePath+"\' as is");
			return absolutePath;
		}
	}

}
