package plugin;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.NoSuchElementException;
import java.util.Scanner;

import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import flexjson.JSONDeserializer;
import history.HistoryManager;
import model.*;
import plugin.config.Preference;
import plugin.config.PreferencesManager;
import plugin.serialization.JsonManager;
import ui.UserStoryTreeView;
import ui.dialogs.ModelConvertFormatDialog;

public class Database implements Runnable
{
	public static Thread THREAD_DATABASE;
	
	private ArrayList<Model> db;
	private static UsSystem rootSystem;
	public static final String SYSTEM_FILE_NAME = "_system_";
	
	public static final String TYPE_SYSTEM = "SYSTEM";
	public static final String TYPE_USERSTORY = "USERSTORY";
	/*
	private static final String FIELD_DESCRIPTION = "description";
	private static final String FIELD_NAME = "name";*/
	
	private static boolean dontSpawn;
	
	/*private static final ModelStub loading = new ModelStub("Loading", "", ModelStub.ICON_LOADING);
	private static final UsSystem loading_root = new UsSystem("Loading Root", "Loading Root System");*/
	
	private String storiesAbsolutePath;
	public static ArrayList<Model> newModels = new ArrayList<Model>();
	
	// Conversion
	public static Model convert_model;
	public static boolean convert_lock;
	public static int convert_result;
	public static final int convert_result_default = -9999;
	
	public Database(String storiesAbsolutePath)
	{
		this.storiesAbsolutePath = storiesAbsolutePath.replace('/', '\\');

		THREAD_DATABASE = new Thread(this, "Database Thread");
		THREAD_DATABASE.start();
		dontSpawn = false;
		
		convert_lock = true;
	}
	public ArrayList<Model> getDbCopy(){
		ArrayList<Model> copy = new ArrayList<Model>();
		for(Model temp : db){
			copy.add(temp.getCopy());			
		}
		return copy;
	}
	public void run()
	{
		//showLoadingStub();
		if (dontSpawn)
		{
			return;
			/*try
			{
				LogWriter.info("Prevented new database");
				Thread.sleep(3000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}*/
		}
		
		try
		{
			LogWriter.info("Database: run(): Waiting for DB");
			Thread.sleep(100);
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
		

		dontSpawn = true;
		LogWriter.info("Database: Spawning for path " + this.storiesAbsolutePath);
		
		db = new ArrayList<Model>();	
		
		rootSystem = new UsSystem("Root", "Root System");
		rootSystem.setAbsolutePath(UsSystem.ROOT_ABSPATH);
		db.add(rootSystem);
		
		scanDir(this.storiesAbsolutePath, UsSystem.ROOT_ABSPATH);
		assignChildren();
		
		//LogWriter.info("Database: run(): Root: " + this.toString());
		
		//Asynchronously Update the Tree Viewer
		//https://wiki.eclipse.org/FAQ_Why_do_I_get_an_invalid_thread_access_exception%3F
		new Thread(new Runnable() {
			public void run()
			{
				Display.getDefault().asyncExec(new Runnable()
				{
					public void run()
					{
						if(UserStoryTreeView.viewer != null){
							UserStoryTreeView.viewer.setInput(rootSystem);
							if(UserStoryTreeView.expandedElements != null && UserStoryTreeView.expandedTreePaths != null){
								UserStoryTreeView.viewer.setExpandedElements(UserStoryTreeView.expandedElements);
								UserStoryTreeView.viewer.setExpandedTreePaths(UserStoryTreeView.expandedTreePaths);
							}
						}
					}
				});
			}
		}).start();
		dontSpawn=false;
	}
	/*
	public void showLoadingStub()
	{
		loading_root.addChild(loading);
		
		new Thread(new Runnable() {
			public void run()
			{
				Display.getDefault().asyncExec(new Runnable()
				{
					public void run()
					{
						LogWriter.info("!!!!!Loading");
						UserStoryTreeView.viewer.setInput(loading_root);
					}
				});
			}
		}).start();
	}*/
	
	
	/**
	 * Starts scanning a directory for Systems and User Stories.
	 * @param directoryAbsPath
	 *//*
	public void scanDir(String directoryAbsPath)
	{
		File directoryToScan = new File(directoryAbsPath);
		if (directoryToScan.exists() && directoryToScan.isDirectory())
		{
			for (File f : directoryToScan.listFiles())
			{
				if (f.getName().equals(SYSTEM_FILE_NAME))
				{
					UsSystem sys = (UsSystem)deserializeFile(f, TYPE_SYSTEM);
					sys.setAbsolutePath(directoryAbsPath);
					db.add(sys);
				}
				else if (f.isDirectory())
				{
					//Recursively search into the new directory
					scanDir(f.getAbsolutePath(), directoryAbsPath);
				}
				else
				{
					UserStory us = (UserStory)deserializeFile(f, TYPE_USERSTORY);
					us.setAbsolutePath(f.getAbsolutePath());
				}
			}
		}
		else
		{
			LogWriter.high("Database: scanDir: Failed before start of recursion");
		}
	}*/

	public UsSystem getRoot()
	{
		return rootSystem;
	}
	
	/**
	 * Recursive method that continues scanning a directory for Systems and User Stories.
	 * @param directoryAbsPath
	 * @param parentAbsPath
	 */
	public void scanDir(String directoryAbsPath, String parentAbsPath)
	{
		File directoryToScan = new File(directoryAbsPath);
		if (directoryToScan.exists() && directoryToScan.isDirectory())
		{
			for (File f : directoryToScan.listFiles())
			{
				// If file is Directory, scan inside it
				if (f.isDirectory())
				{
					//Recursively search into the new directory
					scanDir(f.getAbsolutePath(), directoryAbsPath);
				}
				// If it is a _system_ file, scan it
				else if (f.getName().equals(SYSTEM_FILE_NAME))
				{
					UsSystem sys = (UsSystem)deserializeFile(f, TYPE_SYSTEM);
					sys.setAbsolutePath(directoryAbsPath);
					//LogWriter.info("TOOT TOOT " + f.getParentFile().getParentFile().getAbsolutePath() + " == " + this.storiesAbsolutePath);
					if (f.getParentFile().isDirectory() && f.getParentFile().getParentFile().getAbsolutePath().equals(this.storiesAbsolutePath))
					{
						//LogWriter.info("adding as parent to "+f.getName()+": " + UsSystem.ROOT_ABSPATH);
						sys.setParentAbsolutePath(UsSystem.ROOT_ABSPATH);
					}
					else
					{
						sys.setParentAbsolutePath(parentAbsPath);
					}
					
					if (sys!=null)
					{
						db.add(sys);
					}
				}
				// Else, it is a User Story
				else
				{
					UserStory us = (UserStory)deserializeFile(f, TYPE_USERSTORY);
					us.setAbsolutePath(f.getAbsolutePath());
					us.setParentAbsolutePath(directoryAbsPath);
					db.add(us);
				}
			}
		}
		else
		{
			LogWriter.high("Database: scanDir: Failed during recursion on " + directoryAbsPath);
		}
	}
	
	
	
	public Model deserializeFile(File file, String type)
	{
		// Conversion stuff
		boolean showdialog = PreferencesManager.instance.getShowConversionDialog();
		convert_lock = showdialog;
		convert_result = convert_result_default;
		
		String fileContents = "";
		Model deserializedObject;
		try
		{
			fileContents = new String(Files.readAllBytes(Paths.get(file.getPath())), StandardCharsets.UTF_8);
		}
		catch (IOException e)
		{
			LogWriter.high("Database: deserializeFile: IOException occured while reading File: \"" + file.getAbsolutePath() + "\"");
			e.printStackTrace();
		}

		if (JsonManager.isFlexJson(fileContents))
		{
			LogWriter.info("Database: deserializeFile: Detected FlexJson file");
			if (type.equals(TYPE_SYSTEM))
			{
				deserializedObject = new JSONDeserializer<UsSystem>().deserialize( fileContents );
			}
			else
			{
				deserializedObject = new JSONDeserializer<UserStory>().deserialize( fileContents );
			}
			
			convertModel(deserializedObject, file, showdialog);
		}
		else
		{
			deserializedObject = JsonManager.readModelFromJson(fileContents, type);
		}

		LogWriter.debug("Database: deserializeFile: Deserialized " + type + " with name " + deserializedObject.getName());
		return deserializedObject;
	}
	
	public void convertModel(final Model deserializedObject, final File file, final boolean showdialog)
	{
		//Asynchronously Convert Model to new JSON
		new Thread(new Runnable() {
			public void run()
			{
				Display.getDefault().asyncExec(new Runnable()
				{
					public void run()
					{
						ModelConvertFormatDialog convertDialog = null;
						convert_result = -99999;
						
						if(showdialog)
						{
							convertDialog = new ModelConvertFormatDialog(UserStoryTreeView.shell, deserializedObject);
							convert_result = convertDialog.open();
							
							// Change preferences if user presses OK
							if (convert_result != convert_result_default)
							{			
								PreferencesManager.instance.savePluginSettings();
							}
						}
						
						if (!showdialog || convert_result != convert_result_default)
						{
							// Convert file
							convert_lock = false;
						}
					}
				});
			}
		}).start();
		
		while (convert_lock)
		{
			try
			{
				LogWriter.debug("Database: convertModel(): Waiting for user to convert file");
				Thread.sleep(100);
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}
		
		// Convert file here
		if ( !showdialog || convert_result == Window.OK)
		{			
			LogWriter.info("Database: convertModel: About to convert " + deserializedObject.getName() + " to new format");
			JsonManager.saveToDisk(deserializedObject, file);
			LogWriter.info("Database: convertModel: Finished converting " + deserializedObject.getName() + " to new format");
		}
	}
	
	public ArrayList<Model> getAllUS(){
		return this.db;
	}
	//All user stories, the children too
	public ArrayList<Model> getAllUS(ArrayList<Model> x){
		ArrayList<Model> ret = new ArrayList<Model>();
		for(Model m : x)
		{
			if (m instanceof UsSystem)
			{
				ret.addAll(getAllUS(((UsSystem)m).getChildrenList()));
			}
			//ret.addAll(getAllUS(x.get(i).getNodes()));
		}
		ret.addAll(x);
		return ret;
	}
	
	public void assignChildren()
	{
		// For every Model m in the Database
		for (Model m : db)
		{
			// If they have a Parent Abs Path,
			if (m.getParentAbsolutePath() != null)
			{
				// Store it.
				String parentPath = m.getParentAbsolutePath();
				// If it's equal to the Root Abs Path, add this as a child of the root.
				if (parentPath.equals(UsSystem.ROOT_ABSPATH))
				{
					LogWriter.debug("Database: assignChildren(): Model " + m.getName() + " is root child");
					rootSystem.addChild(m);
				}
				else
				{
					//LogWriter.info("Model " + m.getName() + " NOT a root child");
					for (Model n : db)
					{
						// For every System with an Absolute Path,
						if (n instanceof UsSystem && n.getAbsolutePath()!=null)
						{
							// If it equals the stored Parent Abs Path,
							if (n.getAbsolutePath().equals(parentPath))
							{
								//Add the Model m to the System's Children list.
								((UsSystem)n).addChild(m);
								//LogWriter.info(m.getName() + " is a child of " + n.getName());
							}
						}
					}
				}
			}
		}
	}
	
	
	
	// Creates a new System in the Root Stories dir
	public void createRootSystem(String name)
	{
		UsSystem newSystem = new UsSystem(name, "");
		String absPath = storiesAbsolutePath + "\\" + name;
		name = this.uniqueName(name, rootSystem.getAbsolutePath());
		newSystem.setAbsolutePath(absPath);
		newSystem.setParentAbsolutePath(UsSystem.ROOT_ABSPATH);
		rootSystem.addChild(newSystem);
		try
		{
			newSystem.saveToDisk(absPath);			
			HistoryManager.createHistory(newSystem, HistoryManager.addHistory);
		}
		catch (IOException e)
		{
			LogWriter.high("IOException during Database.createRootSystem("+name+")");
		}
		finally
		{
			newModels.add(newSystem);
			db.add(newSystem);
			//this.run();
		}
	}

	// Creates a new System under the selected System
	public void createSystem(String name, String parentAbsPath)
	{
		name = this.uniqueName(name, parentAbsPath);
		Model parent = getModel(parentAbsPath);
		if (parent == null || !(parent instanceof UsSystem))
		{
			LogWriter.medium("Database.createSystem("+name+"): parent is null or not a System!");
			return;
		}
		UsSystem newSystem = new UsSystem(name, "");
		String absPath = parentAbsPath + "\\" + name;
		newSystem.setAbsolutePath(absPath);
		newSystem.setParentAbsolutePath(parentAbsPath);	
		try
		{
			newSystem.saveToDisk(absPath);
			HistoryManager.createHistory(newSystem, HistoryManager.addHistory);
		}
		catch (IOException e)
		{
			LogWriter.high("IOException during Database.createSystem("+name+")");
		}
		finally
		{
			((UsSystem)parent).addChild(newSystem);
			newModels.add(newSystem);
			db.add(newSystem);
		}
	}
	
	/**
	 * Edit a User Story's properties
	 * @author kristo
	 * @param oldUs
	 * @param newUs
	 */
	public void edit(Model oldUs,Model newUs)
	{
		this.deleteModel(oldUs); // Deletes from DB and disk.
		Model parent = null;
		
		try
		{
			newUs.setName(uniqueName(newUs.getName(),newUs.getParentAbsolutePath())); // give it a unique name			
			String absPath = "";
			
			//If parent is the root node
			if(newUs.getParentAbsolutePath().equals(UsSystem.ROOT_ABSPATH))
			{ 
				absPath = storiesAbsolutePath + "\\" + newUs.getName(); // change the file name
				parent = rootSystem; //sets parent
			}
			else
			{
				absPath = newUs.getParentAbsolutePath() + "\\" + newUs.getName(); // change the file name
				parent = getModel(newUs.getParentAbsolutePath()); // sets parent
			}
			newUs.setAbsolutePath(absPath); // Sets the new path if name changed
			
			newUs.saveToDisk(newUs.getAbsolutePath()); // Save to Disk
			HistoryManager.createHistory(newUs.getCopy(), oldUs.getCopy(), HistoryManager.editHistory); // Create an edit history
			
			// If Model is a System (Folder with UserStories as children) 
			if(newUs instanceof UsSystem)
			{ 
				adjustPathsToParentChange((UsSystem) newUs); // change Nodes Paths to correspond to parent path
			}
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		
		// Set the old UserStory to new. In this way we will just update the old UserStory in view
		oldUs.setName(newUs.getName()); 
		oldUs.setAbsolutePath(newUs.getAbsolutePath());
		oldUs.setDescription(newUs.getDescription());
		
		// UserStory objects got some extra variables to set
		if(newUs instanceof UserStory)
		{ 
			((UserStory)oldUs).setActors(((UserStory)newUs).getActors());
			((UserStory)oldUs).setTestCases(((UserStory)newUs).getTestCases());
		}
		
		db.add(oldUs); // Add the updated UserStory to DB (was deleted)
		
		if(parent != null && parent instanceof UsSystem)
		{
			((UsSystem)parent).addChild(oldUs);
		}
	}
	
	public void adjustPathsToParentChange(UsSystem parent)
	{
		for(Model child : parent.getChildrenList())
		{
			child.setParentAbsolutePath(parent.getAbsolutePath());
			child.setAbsolutePath(parent.getAbsolutePath() + "\\" + child.getName());
			
			try
			{
				child.saveToDisk(parent.getAbsolutePath() + "\\" + child.getName());
				if(getModel(parent.getAbsolutePath() + "\\" + child.getName()) == null){
					db.add(child);
				}
			} 
			catch (IOException e)
			{
				e.printStackTrace();
			}
			
			if(child instanceof UsSystem)
			{
				adjustPathsToParentChange((UsSystem)child);
			}
		}
	}
	/*
	public void sort(){
		if(rootSystem.getChildrenList() == null)
		{
			return;
		}
		alphabeticalSort(sorter,rootSystem.getChildrenList());
		sorter = !sorter;
		UserStoryTreeView.viewer.setInput(rootSystem);
	}
	
	private void alphabeticalSort(boolean sort,ArrayList<Model> list)
	{
		if(list.size() == 0)
		{
			return;
		}
		
		ArrayList<Model> sorted = new ArrayList<Model>();
		int j = 0;
		int i = 0;
		int removeThis = -1;
		
		while( i < list.size())
		{
			Model first = list.get(i);
			removeThis = 0;
			if(first instanceof UsSystem)
			{
				alphabeticalSort(sort,((UsSystem)first).getChildrenList());
			}
			j = i + 1;
			while( j < list.size() )
			{				
				if(sort)
				{
					if(first.getName().compareTo(list.get(j).getName()) > 0)
					{
						removeThis = j;
						first = list.get(j); 
					}
				}
				else
				{
					if(first.getName().compareTo(list.get(j).getName()) < 0
							){
						removeThis = j;
						first = list.get(j);
					}					
				}
				j++;
			}
			//i++;
			sorted.add(first);
			list.remove(removeThis);
		}
		list.clear();
		list.addAll(sorted);
	}
	*/
	
	/**
	 * Gives a unique name to a User Story (Unique name in the folder)
	 * @author kristo
	 * @param name
	 * @param parentPath
	 */
	private String uniqueName(String name, String parentPath)
	{ 
		if(name.matches(".*\\(([0-9]+)\\)"))
		{
			name = name.substring(0, name.lastIndexOf("("));
		}
		
		int k = 0;	//ex: name(1), k = 1	
		Model parent;
		
		if( parentPath.equals(rootSystem.getAbsolutePath()) )
		{
			parent = rootSystem;
		}
		else
		{
			parent = this.getModel(parentPath);
		}
		
		if ( parent == null || !(parent instanceof UsSystem) )
		{
			LogWriter.medium("Database.createSystem("+name+"): parent is null or not a System!");
			return name;
		}
		
		String temp = name;
		int i = 0;
		while(i < ((UsSystem)parent).getChildrenList().size())
		{
			//LogWriter.info(name + " check " + ((UsSystem)parent).getChildrenList().get(i).getName());
			
			// If name exists
			if(temp.equals(((UsSystem)parent).getChildrenList().get(i).getName()))
			{ 
				// Try a different one
				temp = name+"("+k+")"; 
				k++;
				// Start searching again for a new name
				i = -1; 								
			}
			
			i++;
		}	
		return temp;
		
	}
	public void restoreModel(Model add,String parentAbsPath)
	{
		try {
			if(add instanceof LinkStub){				
				Model parent = null;
				if(((LinkStub)add).getLinkType() == LinkStub.TYPE_METHOD){
					parent = getModel(((LinkStub)add).parentUserStoryPath);	
					String methodName = ((LinkStub)add).getName();
					for(Link temp : ((UserStory)parent).getLinks()){
						if(temp.getClassName().equals(((LinkStub) add).getLinkReference().getClassName())){
							temp.getMethodNameList().add(methodName);
							try {
								((UserStory)parent).saveToDisk();
							} catch (IOException e) {
								e.printStackTrace();
							}
							break;
						}
					}
				}else{
					parent = getModel(((LinkStub)add).getParentAbsolutePath());
					((UserStory)parent).getLinks().add(((LinkStub)add).getLinkReference());
					try {
						((UserStory)parent).saveToDisk();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
			else{
				Model parent;
				if(!parentAbsPath.equals("")){
					add.setName(uniqueName(add.getName(),parentAbsPath));
					if(!parentAbsPath.equals(rootSystem.getAbsolutePath())){
						add.setAbsolutePath(parentAbsPath + "\\" + add.getName());
					}else{
						add.setAbsolutePath(storiesAbsolutePath + "\\" + add.getName());
					}					
					add.setParentAbsolutePath(parentAbsPath);
					parent = this.getModel(parentAbsPath);	
				}else{
					add.setName(uniqueName(add.getName(),add.getParentAbsolutePath()));
					if(!add.getParentAbsolutePath().equals(rootSystem.getAbsolutePath())){
						add.setAbsolutePath(add.getParentAbsolutePath() + "\\" + add.getName());
					}else{
						add.setAbsolutePath(storiesAbsolutePath + "\\" + add.getName());
					}
					parent = this.getModel(add.getParentAbsolutePath());				
				}
				
				if(parent != null && parent instanceof UsSystem){
					((UsSystem)parent).addChild(add);	
					db.add(add);				
					//UserStoryTreeView.viewer.add(parent, add);
				}else{
					LogWriter.info("Database(): restoreModel: Something went wrong");
				}
				add.saveToDisk(add.getAbsolutePath());
				if(add instanceof UsSystem)
					adjustPathsToParentChange((UsSystem) add);			
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		/*if(keepHistory){
			HistoryManager.createHistory(add, HistoryManager.addHistory);
		}*/
		//newNames.add(add.getName());
		//this.run();
	}
	
	public void deleteModel(ArrayList<Model> remove)
	{
		for(Model deleteThis : remove){
			if(deleteThis instanceof LinkStub){
				Model parent = null;
				if(((LinkStub)deleteThis).getLinkType() == LinkStub.TYPE_METHOD){
					parent = getModel(((LinkStub)deleteThis).parentUserStoryPath);	
					String methodName = ((LinkStub)deleteThis).getName();
					for(Link temp : ((UserStory)parent).getLinks()){
						System.out.println("Link : " + temp);
						if(temp.getMethodNameList().contains(((LinkStub)deleteThis).getName())){
							temp.getMethodNameList().remove(methodName);
							try {
								((UserStory)parent).saveToDisk();
							} catch (IOException e) {
								e.printStackTrace();
							}
							break;
						}
					}
				}else{
					parent = getModel(((LinkStub)deleteThis).getParentAbsolutePath());
					((UserStory)parent).getLinks().remove(((LinkStub)deleteThis).getLinkReference());
					try {
						((UserStory)parent).saveToDisk();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}else{
				File f = new File(deleteThis.getAbsolutePath());		
				if(f.isDirectory()){			
					deleteFilesInDir(deleteThis.getAbsolutePath());
				}else{
					f.delete();
				}
				ArrayList<Model> children = new ArrayList<Model>();
				children.add(deleteThis);
				
				children = getAllUS(children);
				db.removeAll(children);
				if(UserStoryTreeView.viewer != null)
					UserStoryTreeView.viewer.remove(deleteThis);
				Model parent = this.getModel(deleteThis.getParentAbsolutePath());
				if(deleteThis.getParentAbsolutePath().equals(rootSystem.getAbsolutePath())){
					parent = rootSystem;
				}
				if(parent != null){
					if(parent instanceof UsSystem){
						((UsSystem)parent).getChildrenList().remove(deleteThis);
					}
				}
			}
		}
	}
	
	public void deleteModel(Model remove)
	{
		ArrayList<Model> temp = new ArrayList<Model>();
		temp.add(remove);
		deleteModel(temp);
	}
	
	public static void deleteFilesInDir(String path)
	{
		File f = new File(path);
		if(f.exists() && f.isDirectory()){
			for(int i = 0; i < f.listFiles().length; i++){
				if(f.listFiles()[i].isDirectory()){
					deleteFilesInDir(f.listFiles()[i].getAbsolutePath());
				}else{
					f.listFiles()[i].delete();
				}
			}
		}		
		if(!f.delete()){
			deleteFilesInDir(path);
		}
	}
	
	// Creates a new System under the selected System
	public void createUserStory(String name, String parentAbsPath)
	{
		name = this.uniqueName(name, parentAbsPath);
		UserStory newUserStory;
		Model parent = getModel(parentAbsPath);
		if (parent == null || !(parent instanceof UsSystem))
		{
			LogWriter.medium("Database.createSystem("+name+"): parent is null or not a System!");
			return;
		}
		newUserStory = new UserStory(name, "");
		String absPath = parentAbsPath + "\\" + name;
		newUserStory.setAbsolutePath(absPath);
		newUserStory.setParentAbsolutePath(parentAbsPath);
		try
		{
			newUserStory.saveToDisk(absPath);
			HistoryManager.createHistory(newUserStory, HistoryManager.addHistory);
		}
		catch (IOException e)
		{
			LogWriter.high("IOException during Database.createSystem("+name+")");
		}
		finally
		{
			((UsSystem)getModel(parentAbsPath)).addChild(newUserStory);
			newModels.add(newUserStory);
			db.add(newUserStory);
		}
	}
	
	public void refresh()
	{
		this.run();
	}
	
	public Model getModel(String abs_path)
	{
		if(db == null){
			return null;
		}
		for (Model m : db)
		{
			if (m.getAbsolutePath().equals(abs_path))
			{
				return m;
			}
		}
		return null;
	}
	
	@Override
	public String toString()
	{
		return rootSystem.toString();
	}	
}
