package plugin.config;

import java.util.ArrayList;

import org.eclipse.core.runtime.preferences.InstanceScope;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;

import plugin.Activator;
import plugin.LogWriter;

public class PreferencesManager
{
	public static PreferencesManager instance;

	public ArrayList<Preference> settings;
	
	public PreferencesManager()
	{
		instance = this;
		settings = new ArrayList<Preference>();
		settings.add(new Preference(Preference.default_project, Preference.default_default_project));
		settings.add(new Preference(Preference.show_conversion_confirmation_dialog, Preference.default_show_conversion_confirmation_dialog));
		settings.add(new Preference(Preference.show_delete_confirmation_dialog, Preference.default_show_delete_confirmation_dialog));
		settings.add(new Preference(Preference.show_test_case_cancel_confirmation_dialog, Preference.default_show_test_case_cancel_confirmation_dialog));
		settings.add(new Preference(Preference.open_editor_on_add, Preference.default_open_editor_on_add));
		settings.add(new Preference(Preference.table_show_systems, Preference.default_table_show_systems));
		settings.add(new Preference(Preference.table_show_linked_classes, Preference.default_table_show_linked_classes));
		settings.add(new Preference(Preference.table_show_linked_methods, Preference.default_table_show_linked_methods));
	}
	
	public static void defaults()
	{
		instance = new PreferencesManager();
		instance.savePluginSettings();
	}
	
	public Preference getSetting(String id)
	{
		for (Preference s : settings)
		{
			if (s.id.equals(id))
			{
				return s;
			}
		}
		return null;
	}
	
	public boolean getCurrentValueBool(String id)
	{
		loadPluginSettings();
		Preference s = getSetting(id);
		return Boolean.parseBoolean(s.currentValue.toString());
	}
	
	public void setCurrentValueBool(String id, boolean value)
	{
		Preference s = getSetting(id);
		s.currentValue = value;
	}


	public String getCurrentValue(String id)
	{
		loadPluginSettings();
		Preference s = getSetting(id);
		return (s.currentValue.toString());
	}
	
	public void setCurrentValue(String id, String value)
	{
		Preference s = getSetting(id);
		s.currentValue = value;
	}
	
	public void savePluginSettings()
	{
		// saves plugin preferences at the workspace level
		Preferences prefs = InstanceScope.INSTANCE.getNode(Activator.getDefault().getBundle().getSymbolicName()); // does all the above behind the scenes

		LogWriter.debug("PluginSettings: savePluginSettings: About to save prefs to " + Activator.getDefault().getBundle().getSymbolicName());
		for (Preference s : settings)
		{
			prefs.put(s.getId(), s.getValue());
			LogWriter.debug("PluginSettings: savePluginSettings: wrote id " + s.getId() + " with value " + s.getValue());
		}
		
		try
		{
			LogWriter.debug("PluginSettings: savePluginSettings: About to flush prefs");
			prefs.flush();
		}
		catch(BackingStoreException e)
		{
			LogWriter.debug("PluginSettings: savePluginSettings(): BackingStoreException occured");
			e.printStackTrace();
		}
		finally
		{
			LogWriter.debug("PluginSettings: savePluginSettings: Finished");
		}
	}

	public void loadPluginSettings()
	{
		Preferences prefs = InstanceScope.INSTANCE.getNode(Activator.PLUGIN_ID);
		
		for (Preference s : settings)
		{
			s.currentValue = prefs.get(s.getId(), s.getDefaultValue());
		}
	}

	public static String getDefaultProject()
	{
		return instance.getCurrentValue(Preference.default_project);
	}
	
	public static boolean getShowDeleteDialog()
	{
		return instance.getCurrentValueBool(Preference.show_delete_confirmation_dialog);
	}
	
	public static boolean getShowConversionDialog()
	{
		return instance.getCurrentValueBool(Preference.show_conversion_confirmation_dialog);
	}
	
	public static boolean getShowTestCancelDialog()
	{
		return instance.getCurrentValueBool(Preference.show_test_case_cancel_confirmation_dialog);
	}
	
	public static boolean getOpenEditorOnAdd()
	{
		return instance.getCurrentValueBool(Preference.open_editor_on_add);
	}
	
	public static boolean getTableShowClasses()
	{
		return instance.getCurrentValueBool(Preference.table_show_linked_classes);
	}

	public static boolean getTableShowMethods()
	{
		return instance.getCurrentValueBool(Preference.table_show_linked_methods);
	}
	
	public static boolean getTableShowSystems()
	{
		return instance.getCurrentValueBool(Preference.table_show_systems);
	}
	
	public static void setDefaultProject(String value)
	{
		instance.setCurrentValue(Preference.default_project, value);
	}
	
	public static void setShowDeleteDialog(boolean value)
	{
		instance.setCurrentValueBool(Preference.show_delete_confirmation_dialog, value);
	}
	
	public static void setShowConversionDialog(boolean value)
	{
		instance.setCurrentValueBool(Preference.show_conversion_confirmation_dialog, value);
	}

	public static void setShowTestCancelDialog(boolean value)
	{
		instance.setCurrentValueBool(Preference.show_test_case_cancel_confirmation_dialog, value);
	}

	public static void setOpenEditorOnAdd(boolean value)
	{
		instance.setCurrentValueBool(Preference.open_editor_on_add, value);
	}

	public static void setTableShowClasses(boolean value)
	{
		instance.setCurrentValueBool(Preference.table_show_linked_classes, value);
	}

	public static void setTableShowMethods(boolean value)
	{
		instance.setCurrentValueBool(Preference.table_show_linked_methods, value);
	}

	public static void setTableShowSystems(boolean value)
	{
		instance.setCurrentValueBool(Preference.table_show_systems, value);
	}
	
}
