package plugin.config;

import java.io.IOException;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import plugin.Activator;
import plugin.LogWriter;

public class PreferencesPage extends PreferencePage  implements IWorkbenchPreferencePage{

	public static final String LABEL_GLOBAL_PREFERENCES = "Global Preferences";
	public static final String LABEL_PROJECT_PREFERENCES = "Project Preferences";
	public static final String LABEL_NO_PROJECT = "No Project Selected.\nPlease close the Preferences dialog, select a Project on the User Story Tree and re-open the dialog.";

	public static USPreferenceButton btn_show_convert_message;
	public static USPreferenceButton btn_show_delete_message;
	public static USPreferenceButton btn_show_test_case_warning;
	public static USPreferenceButton btn_open_editor_on_add;
	public static USPreferenceButton btn_table_show_systems;
	public static USPreferenceButton btn_table_show_classes;
	public static USPreferenceButton btn_table_show_methods;
	
	public static final String QUESTIONMARK_HELP_LABEL = "[?]";
	
	public static final String DEFAULT_PROJECT_LABEL = "Default Project";
	public static final String DEFAULT_PROJECT_TOOLTIP = "When a default project is set, it is automatically loaded when the User Story Tree is opened.";
	public static final String DEFAULT_PROJECT_TOOLTIP_2 = "To set a default project, go to the Project Select dialog, select a project and choose \"Set as Default Project\"";
	
	public static final String CONVERT_CONFIRMATION_LABEL = "Do Not Automatically Convert Systems and User Stories";
	public static final String CONVERT_CONFIRMATION_TOOLTIP = "When enabled, a confirmation dialog is displayed when an old format User Story needs to be converted. When disabled, they are automatically converted.";
	
	public static final String DELETE_CONFIRMATION_LABEL = "Show Delete Confirmation Dialog";
	public static final String DELETE_CONFIRMATION_TOOLTIP = "When enabled, a warning dialog is displayed when deleting a System or User Story.";

	public static final String TESTCASE_CONFIRMATION_LABEL = "Show Test Case Cancelation Confirmation Dialog";
	public static final String TESTCASE_CONFIRMATION_TOOLTIP = "When enabled, a warning dialog is displayed when cancelling changes to a User Story's Test Cases.";

	public static final String OPEN_EDITOR_ON_ADD_LABEL = "Open Editor after adding a System / User Story";
	public static final String OPEN_EDITOR_ON_ADD_TOOLTIP = "When enabled, the User Story Editor opens after adding a System or User Story.";

	public static final String TABLE_SHOW_SYSTEMS_LABEL = "Show Systems in the \"Selected User Stories\" view";
	public static final String TABLE_SHOW_SYSTEMS_TOOLTIP = "When enabled, Systems are displayed in the \"Selected User Stories\" view.";

	public static final String TABLE_SHOW_CLASSES_LABEL = "Show Linked Classes in the \"Selected User Stories\" view";
	public static final String TABLE_SHOW_CLASSES_TOOLTIP = "When enabled, Linked Classes are displayed in the \"Selected User Stories\" view.";

	public static final String TABLE_SHOW_METHODS_LABEL = "Show Linked Methods in the \"Selected User Stories\" view";
	public static final String TABLE_SHOW_METHODS_TOOLTIP = "When enabled, Linked Methods are displayed in the \"Selected User Stories\" view.";

	
	public static boolean projectActive = false;
	
	@Override
	public void init(IWorkbench arg0) {}
	
	@Override
	protected Control createContents(Composite parent)
	{
		//this.noDefaultAndApplyButton();

		
		PreferencesManager.instance.loadPluginSettings();
		
		GridLayout layout = new GridLayout(1,false);
		//layout.marginRight = 5;
		//layout.marginLeft = 10;
		final Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(layout);
        
        
		/*
		 * Global Preferences
		 */
        final Composite defaultProjectComp = new Composite(container, SWT.NONE);
        defaultProjectComp.setLayout(new GridLayout(3,false));
        
        final Label lblDefault = new Label(defaultProjectComp, SWT.NONE);
        lblDefault.setText(DEFAULT_PROJECT_LABEL);
        lblDefault.setToolTipText(DEFAULT_PROJECT_TOOLTIP);
        
        final Text lblProjectName = new USPreferenceText(defaultProjectComp, "", Preference.default_project);
        lblProjectName.setEditable(false);
        lblProjectName.setEnabled(false);
        if (lblProjectName.getText().equals(""))
        {
        	lblProjectName.setText("No project selected.");
        }
        
        final Label lblQuestionMark = new Label(defaultProjectComp, SWT.NONE);
        //lblQuestionMark.setText(QUESTIONMARK_HELP_LABEL);
        lblQuestionMark.setImage(new Image(null, Activator.getImageDescriptor("icons/help.png").getImageData()));
        lblQuestionMark.setToolTipText(DEFAULT_PROJECT_TOOLTIP_2);
		
		// Show User Story Conversion Message 
		btn_show_convert_message = new USPreferenceButton(container,
				CONVERT_CONFIRMATION_LABEL,
				CONVERT_CONFIRMATION_TOOLTIP,
				Preference.show_conversion_confirmation_dialog);
		
		// Show Warning on Delete
		btn_show_delete_message = new USPreferenceButton(container,
				DELETE_CONFIRMATION_LABEL,
				DELETE_CONFIRMATION_TOOLTIP,
				Preference.show_delete_confirmation_dialog);

		// Show Test Case Warning 
		btn_show_test_case_warning = new USPreferenceButton(container,
				TESTCASE_CONFIRMATION_LABEL,
				TESTCASE_CONFIRMATION_TOOLTIP,
				Preference.show_test_case_cancel_confirmation_dialog);

		//  Open Editor on Add 
		btn_open_editor_on_add = new USPreferenceButton(container,
				OPEN_EDITOR_ON_ADD_LABEL,
				OPEN_EDITOR_ON_ADD_TOOLTIP,
				Preference.open_editor_on_add);

		// Show Systems on Table Viewer 
		btn_table_show_systems = new USPreferenceButton(container,
				TABLE_SHOW_SYSTEMS_LABEL,
				TABLE_SHOW_SYSTEMS_TOOLTIP,
				Preference.table_show_systems);
		
		// Show Classes on Table Viewer
		btn_table_show_classes = new USPreferenceButton(container,
				TABLE_SHOW_CLASSES_LABEL,
				TABLE_SHOW_CLASSES_TOOLTIP,
				Preference.table_show_linked_classes);
		
		// Show Methods on Table Viewer
		btn_table_show_methods = new USPreferenceButton(container,
				TABLE_SHOW_METHODS_LABEL,
				TABLE_SHOW_METHODS_TOOLTIP,
				Preference.table_show_linked_methods);
		
/*
		// Separator Label
		Label separator = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// Project Preferences Label
		Label lblProj = new Label(container, SWT.NONE);
		lblProj.setText(LABEL_PROJECT_PREFERENCES);
		descriptor = FontDescriptor.createFrom(lblProj.getFont());
		// setStyle method returns a new font descriptor for the given style
		descriptor = descriptor.setStyle(SWT.BOLD);
		lblProj.setFont(descriptor.createFont(lblProj.getDisplay()));
*/
	    return container;
	}
	
	@Override
    public boolean performOk()
	{
		performApply();
        return super.performOk();
    }
	
	@Override
    public void performApply()
	{
		PreferencesManager.instance.savePluginSettings();
	}
	
	@Override
	public Point computeSize()
	{
		return new Point(650,400);
	}
	
	@Override
    protected void performDefaults()
	{
		PreferencesManager.defaults();
		btn_show_convert_message.setSelection(PreferencesManager.getShowConversionDialog());
		btn_show_delete_message.setSelection(PreferencesManager.getShowDeleteDialog());
		btn_show_test_case_warning.setSelection(PreferencesManager.getShowTestCancelDialog());
		btn_open_editor_on_add.setSelection(PreferencesManager.getOpenEditorOnAdd());
		btn_table_show_systems.setSelection(PreferencesManager.getTableShowSystems());
		btn_table_show_classes.setSelection(PreferencesManager.getTableShowClasses());
		btn_table_show_methods.setSelection(PreferencesManager.getTableShowMethods());
        
		
		super.performDefaults();
    }
	
}

class USPreferenceButton extends Button
{
	public USPreferenceButton(Composite parent, String name, String tooltip, String id)
	{
		super(parent, SWT.CHECK);
		
		setSelection(PreferencesManager.instance.getCurrentValueBool(id));
		setText(name);
		if (!tooltip.equals(""))
		{
			setToolTipText(tooltip);	
		}
		addSelectionListener(new USPreferenceButtonSelectionListener(id));
	}
	
	@Override
	public void checkSubclass()
	{
		return;
	}
}


class USPreferenceText extends Text
{
	public USPreferenceText(Composite parent, String tooltip, String id)
	{
		super(parent, SWT.SINGLE);
		
		//setText(PreferencesManager.instance.getCurrentValueBool(id));
		setText(PreferencesManager.instance.getCurrentValue(id));
		setToolTipText(tooltip);
		//addSelectionListener(new USPreferenceButtonSelectionListener(id));
	}
	
	@Override
	public void checkSubclass()
	{
		return;
	}
}

class USPreferenceButtonSelectionListener implements SelectionListener
{
	public String reference;
	
	public USPreferenceButtonSelectionListener(String reference)
	{
		this.reference = reference;
	}
	
	@Override
	public void widgetSelected(SelectionEvent event)
	{
		Button btn = (Button) event.getSource();
		PreferencesManager.instance.setCurrentValueBool(reference, btn.getSelection());
	}

	@Override
	public void widgetDefaultSelected(SelectionEvent e)
	{}
	
}