package plugin.serialization;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.StandardCharsets;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import model.Model;
import model.UsSystem;
import model.UserStory;
import plugin.Database;
import plugin.LogWriter;
//import plugin.config.UserStoryToolPreferenceManager;

public class JsonManager
{
	public static final String flexString = "\u001a";
	public static Gson gson_instance;
	
	public JsonManager()
	{
		gson_instance = new GsonBuilder().setPrettyPrinting().create();
	}
	
	public static void writeJsonToFile(Jsonable j) throws IOException
	{
		LogWriter.info("JsonManager: writeJsonToFile: Awake");
		//Gson gson = new GsonBuilder().setPrettyPrinting().create();
		String json = gson_instance.toJson(j);
		j.saveToDisk();
		LogWriter.info("JsonManager: writeJsonToFile: Done");
	}
	

	
	public static void saveToDisk(Jsonable j, File f)
	{
		String json = gson_instance.toJson(j);
		//Writer writer = new OutputStreamWriter( new FileOutputStream ( f.getAbsolutePath()  , "UTF-8")_) );
		try {
			LogWriter.debug("JsonManager: saveToDisk(): Started Writer");
			Writer writer = new OutputStreamWriter ( new FileOutputStream( f.getAbsolutePath() ), StandardCharsets.UTF_8);
			try {
				writer.write(json);
				writer.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		LogWriter.debug("JsonManager: saveToDisk(): Closed Writer");
	}
	
	public static String toJson(Jsonable j)
	{
		String json = gson_instance.toJson(j);
		return json;
	}
	
	public static Model readModelFromJson(String json, String modelType)
	{
		switch (modelType)
		{
			case Database.TYPE_SYSTEM:
				return (UsSystem) gson_instance.fromJson(json, UsSystem.class);
			case Database.TYPE_USERSTORY:
				return (UserStory) gson_instance.fromJson(json, UserStory.class);
			default:
				return null;
		}
	}
	/*
	public static UserStoryToolPreferenceManager readPreferencesFromJson(String json)
	{
		return gson_instance.fromJson(json, UserStoryToolPreferenceManager.class);
	}
	*/
	public static boolean isFlexJson(String flexJson)
	{
		return flexJson.contains(flexString);
	}
	
	public static void convertFromFlex(Model m)
	{
		try
		{
			String json = m.toJson();
			m.saveToDisk(json, m.getAbsolutePath());
		}
		catch (IOException e)
		{
			LogWriter.high("JsonManager: convertFromFlex: IOException occured while converting model " + m.getName() + " of type " + m.getClass());
			e.printStackTrace();
		}
	}

}
