package tests;
import org.junit.runners.MethodSorters;
import static org.junit.Assert.*;

import java.io.File;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;

import model.Model;
import model.Project;
import model.UsSystem;
import plugin.Controller;
import plugin.Database;
import plugin.ProjectDatabase;

public class DatabaseTest {
	String path = "";
	int filesInProject = 0;
	@Before
	public void beforeTest() throws Exception {
		System.out.println("beforeTest");
	}

	@After
	public void afterTest() throws Exception {
		System.out.println("AfterTest");
	}
	@Test 
	public void createDBTest() { //ProjectDatabase,Controller,Database constructors
		path = System.getProperty("user.dir");
		File userstoriestool = new File(path+"\\userstoriestool");
		if(userstoriestool.exists() && userstoriestool.isDirectory()){
			Database.deleteFilesInDir(path+"\\userstoriestool");
			userstoriestool.delete();
		}
		File projectPath = new File(path);
		assertFalse(projectPath.listFiles() == null);
		filesInProject = projectPath.listFiles().length;
		new ProjectDatabase();
		
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		
		Project temp = new Project("com.userstoriestool", path);
		Controller ctrl = new Controller();
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		ctrl.switchProjectTestingPorpuses(temp);
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		assertNotNull(Controller.getDatabase().getRoot());

		//Thread testing is pain in the eyes :P 
		//This method will run before the createModelsTests() ends, this is critical
		//We could Threadh.sleep() for every method... or just run them through this @Test
		createModelsTests();
		editTest();
		deleteModelTest();
		deleteFiles();
	}

	
	public void createModelsTests(){//Create UsSystem and UserStory
		Controller.getDatabase().createRootSystem("RootSystemBoss");
		Controller.getDatabase().createSystem("SystemOP", path+"\\userstoriestool\\stories\\RootSystemBoss");
		Controller.getDatabase().createUserStory("UserStoryq", path+"\\userstoriestool\\stories\\RootSystemBoss\\SystemOP");
		//uniqueName() will change the name!
		Controller.getDatabase().createUserStory("UserStoryq", path+"\\userstoriestool\\stories\\RootSystemBoss\\SystemOP");
		
		Model temp1 = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss"); 
		Model temp2 = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss\\SystemOP");
	    Model temp3 = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss\\SystemOP\\UserStoryq.us");
	    Model temp4 = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss\\SystemOP\\UserStoryq(0).us");
	    
	    
		assertNotNull(temp1);
		assertNotNull(temp2);
		assertNotNull(temp3);
		assertNotNull("uniqueName() failed to change the name or expected name != returned name",temp4);
	}
	public void editTest(){
		Model temp2 = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss\\SystemOP");
		
		Model newTemp2 = temp2.getCopy();		
		newTemp2.setDescription("new Description");
		newTemp2.setName("qq");
		
		//Old file should be deleted
		//New file path = old parent's path + new name
		//DB shouldn't contains old file and DB contains new file
		//Children's parentAbsPath should change to new Model's file path
		//Children's path should change to parent's path + child's name
		//Unique name should give a different name if the new Model's edited name exists
		Controller.getDatabase().edit(temp2, newTemp2);
		
		File oldFile = new File(path+"\\userstoriestool\\stories\\RootSystemBoss\\SystemOP");
		assertFalse("The old file still exists after edit",oldFile.exists());
		File newFile = new File(path+"\\userstoriestool\\stories\\RootSystemBoss\\qq");
		assertTrue("The new file still wasn't created",newFile.exists());
		assertNull(Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss\\SystemOP"));
		Model newTempAfterEdit = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss\\qq");
		assertNotNull(newTempAfterEdit);
		assertEquals("new Description",newTempAfterEdit.getDescription());
		assertTrue(newTempAfterEdit instanceof UsSystem);
		//This UsSystem contains only 2 UserStory
		assertEquals(path+"\\userstoriestool\\stories\\RootSystemBoss\\qq",((UsSystem)newTempAfterEdit).getChildrenList().get(0).getParentAbsolutePath());
		assertEquals(path+"\\userstoriestool\\stories\\RootSystemBoss\\qq\\" + ((UsSystem)newTempAfterEdit).getChildrenList().get(0).getName()+".us",((UsSystem)newTempAfterEdit).getChildrenList().get(0).getAbsolutePath());
		assertEquals(path+"\\userstoriestool\\stories\\RootSystemBoss\\qq",((UsSystem)newTempAfterEdit).getChildrenList().get(1).getParentAbsolutePath());
		assertEquals(path+"\\userstoriestool\\stories\\RootSystemBoss\\qq\\" + ((UsSystem)newTempAfterEdit).getChildrenList().get(1).getName()+".us",((UsSystem)newTempAfterEdit).getChildrenList().get(1).getAbsolutePath());
		
		for(Model t : Controller.getDatabase().getDbCopy()){
			System.out.println(t.getAbsolutePath());
		}
		
		Model temp3 = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss\\qq\\UserStoryq(0).us");
		System.out.println(temp3 == null);
		Model newTemp3 = temp3.getCopy();
		newTemp3.setName("UserStoryq"); 
		Controller.getDatabase().edit(temp3, newTemp3);
		
		//The changed name will be UserStoryq(0)
		//If the was a UserStoryq(0) then the name would be UserStoryq(1)
		Model newTemp3AfterEdit = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss\\qq\\UserStoryq(0).us"); 
		
		assertNotNull(newTemp3AfterEdit);		
	}
	
	public void deleteModelTest(){
		Model temp1 = Controller.getDatabase().getModel(path+"\\userstoriestool\\stories\\RootSystemBoss");
		Controller.getDatabase().deleteModel(temp1); //temp2 and temp3 are its children! deleteModel(Model) should delete them too
		//The default size of DB 1 (root) 
		assertEquals("After delete some Model weren't deleted",1,Controller.getDatabase().getDbCopy().size());		
	}
	
	public void deleteFiles(){
		File projectPath = new File(path);
		Database.deleteFilesInDir(path+"\\userstoriestool");
		new File(path+"\\userstoriestool").delete();
		assertEquals("",filesInProject,projectPath.listFiles().length);	
	}
}
