package tests;

import static org.junit.Assert.*;

import java.io.File;
import java.io.IOException;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import model.Model;
import model.UsSystem;
import model.UserStory;

public class ModelTest {

	@Before
	public void setUpBeforeClass() throws Exception {
		System.out.println("Running test..");
	}

	@After
	public void tearDownAfterClass() throws Exception {
		System.out.println("test ended..");
	}

	@Test
	public void testModelConstuctors() {
		Model x = new UserStory();
		x = new UserStory("name","description");
		
		assertTrue(x.getClass().isAssignableFrom(UserStory.class));
		assertEquals("name",x.getName());
		assertEquals("description",x.getDescription());
		
		x = new UserStory("name1","description1","thispath","parentPath");
		
		assertTrue(x.getClass().isAssignableFrom(UserStory.class));
		assertEquals("name1",x.getName());
		assertEquals("description1",x.getDescription());
		assertEquals("thispath",x.getAbsolutePath());
		assertEquals("parentPath",x.getParentAbsolutePath());
		
		x = new UsSystem("name11","description11","thispath11","parentPath11");
		assertTrue(x.getClass().isAssignableFrom(UsSystem.class));
		assertEquals("name11",x.getName());
		assertEquals("description11",x.getDescription());
		assertEquals("thispath11",x.getAbsolutePath());
		assertEquals("parentPath11",x.getParentAbsolutePath());
	}
	@Test
	public void testEquals(){
	 //We cannot create a UserStory with the same file path because the UserStory will overwrite the old.
	 //So the file path is the factor that separates UserStories
	 Model x = new UserStory("name","description","thisPath","thisParentPath");
	 Model x1 = new UserStory("name","description","thisPath","thisParentPath");
	 Model x2 = new UsSystem("name","description","usSystemPath","UsSystemParentPath");
	 
	 assertTrue("UserStory equals to UserStory ",x1.equals(x));
	 assertFalse("UserStory should not be equals with UsSystem",x1.equals(x2));
	}
	@Test
	public void testSaveToDisk(){
		String path = System.getProperty("user.dir");
		File root = new File(path);
		int filesInDir = root.listFiles().length;
		
		Model x = new UserStory("name","description",path+"\\name","thisParentPath");
		try {
			x.saveToDisk(x.getAbsolutePath());
		} catch (IOException e) {
			fail("Couldn't save the UserStory. " + e.getMessage());
		}
		File userStoryFile = new File(x.getAbsolutePath());
		assertTrue(userStoryFile.exists());
		assertTrue(userStoryFile.isFile());
		assertTrue("Created UserStory File for testing puproses couldn't deleted",userStoryFile.delete());
		assertTrue("Created UserStory's parent directory for testing puproses couldn't deleted",new File(x.getParentAbsolutePath()).delete());
		
		Model usSystem = new UsSystem("name","description",path+"\\name","thisParentPath");
		try {
			usSystem.saveToDisk(x.getAbsolutePath());
		} catch (IOException e) {
			fail("Couldn't save the UserStory. " + e.getMessage());
		}
		File usSystemFile = new File(x.getAbsolutePath());
		File deleteThisBeforeDeleteTheUsSystemFolder = new File(x.getAbsolutePath()+"\\_system_");
		assertTrue(deleteThisBeforeDeleteTheUsSystemFolder.delete());
		assertTrue(usSystemFile.exists());
		assertTrue(usSystemFile.isDirectory());
		assertTrue("Created UsSystem File for testing puproses couldn't deleted",usSystemFile.delete());
		
		int filesLeftInDir = root.listFiles().length;
		assertEquals("The files left in directory should be equals to files at the Beginning",filesInDir,filesLeftInDir);
	}
	@Test
	public void testCopy(){
	
		Model userstory = new UserStory("name","description","usFilePath","usParentFilePath");
		Model copyUs = userstory.getCopy();
		
		Model system = new UsSystem("name1","description","systemFilePath","systemParentFilePath");
		Model copySystem = system.getCopy();
		
		assertEquals(userstory,copyUs);
		assertEquals(system,copySystem);
	}
	@Test
	public void testUsSystemgetChildren(){
		UsSystem system = new UsSystem("name1","description","systemFilePath","systemParentFilePath");
		assertEquals(0,system.getChildren().length);
		Model userstory = new UserStory("name","description","usFilePath","usParentFilePath");
		system.addChild(userstory);
		assertEquals(1,system.getChildren().length);
	}
}
