package ui;

import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.graphics.Image;

import actors.Actor;
import model.*;
import plugin.Activator;

public class UserStoryLabelProvider extends StyledCellLabelProvider 
{	
	
	public Image getImage(Object element)
	{
		if (element instanceof UsSystem)
		{
			return Activator.getImageDescriptor("icons/folder.png").createImage();
		}
		else if (element instanceof UserStory)
		{
			return Activator.getImageDescriptor("icons/userstory_default.png").createImage();
		} 
		else if (element instanceof LinkStub)
		{
			if ( ((LinkStub)element).getLinkType() == LinkStub.TYPE_CLASS )
			{
				return Activator.getImageDescriptor("icons/class_link.gif").createImage();
			}
			else
			{
				return Activator.getImageDescriptor("icons/method_link.png").createImage();
			}
			
		}
		else if (element instanceof ModelStub)
		{
			return Activator.getImageDescriptor( ((ModelStub)element).getIcon() ).createImage();
		}
		else if (element instanceof Actor)
		{
			return Activator.getImageDescriptor("icons/actor.png").createImage();
		}
		else
		{
			return Activator.getImageDescriptor("icons/sample.gif").createImage();
		}
	}
	
	public String getToolTipText(Object element)
	{
		return "OK";
	}
	
	public String getText(Object element)
	{
		if (element instanceof LinkStub)
		{
			return ((Model)element).getName();
		}
		else if (element instanceof Model)
		{
			return ((Model)element).getName();
		}
		else if(element instanceof Actor){
			return ((Actor)element).getName();
		}
		else
		{
			return "Unrecognized Object";
		}
	}
	
	public void update(ViewerCell cell) {
        Object obj = cell.getElement();
        StyledString styledString = new StyledString("");
        
        if(obj instanceof Actor){
        	styledString = new StyledString( ((Actor)obj).getName());	
        }
        else{
        	styledString = new StyledString( ((Model)obj).getName());
        }

        if(obj instanceof LinkStub)
        {
        	//LinkStub stub = (LinkStub) obj;
        	//LogWriter.info("Update found Stub, adding " + " - " +  stub.getLinkReference().getSourceFileAbsPath());
            styledString.append(" : " +  ((LinkStub)obj).getLinkReference().getSourceFilename(), StyledString.QUALIFIER_STYLER);
        }

       cell.setText(styledString.toString());
       cell.setStyleRanges(styledString.getStyleRanges());
       cell.setImage(getImage(obj));
       super.update(cell);
   }

	
	public void dispose()
	{
	}

	protected RuntimeException unknownElement(Object element) {
		return new RuntimeException("Unknown type of element in tree of type " + element.getClass().getName());
	}
}