package ui;


import java.util.ArrayList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import actors.ActorManager;
import model.LinkStub;
import model.Model;
import model.TestCase;
import model.UsSystem;
import model.UserStory;
import plugin.Activator;
import plugin.Controller;
import plugin.config.PreferencesManager;
import ui.dialogs.SearchDialog;


public class UserStoryTableViewer extends ViewPart {

	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String ID = "com.userstoriestool.ui.UserStoryTableViewer";

	private static TableViewer viewer;
	
	private boolean sorterUser = true;
	private boolean sortingUser = false;
	
	private boolean sorterName = true;
	private boolean sortingName = false;
	
	private boolean showAll = false;
	
	private static String searchName = "";
	private static String searchDesc = "";
	private static String searchActors = "";
	
	public static String searchInNames = "Names_Search";
	public static String searchInDescriptions = "Descriptions_Search";
	public static String searchInActors = "Actors_Search";
	
	private ArrayList<Model> allUS = new ArrayList<Model>();

	private IAction openSearch;
	
	class ViewContentProvider implements IStructuredContentProvider {
		
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		
		public Object[] getElements(Object inputElement) {
			if(UserStoryTreeView.currentProjectPath==null || UserStoryTreeView.currentProjectPath == ""){
				Object[] x=new Object[1];
				x[0] = new UserStory();
				((UserStory)x[0]).setName("Nothing_Selected");
				((UserStory)x[0]).setDescription("You haven't select any object from the Requirement Tree");
				return x;				
			}
			if(!showAll){
				if(UserStoryTreeView.lastSelectionIndex != null && !UserStoryTreeView.lastSelectionIndex.getName().equals("") ){
					allUS.clear();
					if(UserStoryTreeView.lastSelectionIndex instanceof UserStory)
						allUS.add(UserStoryTreeView.lastSelectionIndex);
					else if(UserStoryTreeView.lastSelectionIndex instanceof UsSystem){
						allUS.addAll(getAll(UserStoryTreeView.lastSelectionIndex));
					}
				}else{
					Object[] x=new Object[1];
					x[0] = new UserStory();
					((UserStory)x[0]).setName("Nothing_Selected");
					((UserStory)x[0]).setDescription("You haven't select any object from the Requirement Tree");
					return x;
				}
			}else{
				allUS.clear();
				allUS.addAll(getAll(Controller.getDatabase().getRoot()));				
			}
			
			if(!searchActors.equals("")){
				allUS = search(allUS,searchActors,searchInActors);						
			}
			if(!searchName.equals("")){
				allUS = search(allUS,searchName,searchInNames);
			}
			if(!searchDesc.equals("")){
				allUS = search(allUS,searchDesc,searchInDescriptions);
			}
			
			if(sortingUser){
				allUS = sort(allUS,"actors",sorterUser);

			}else if(sortingName){
				allUS = sort(allUS,"names",sorterName);
			}

			
			ArrayList<Model> onlyUs = new ArrayList<Model>();
			for(int i = 0; i<allUS.size();i++){
				if(allUS.get(i) instanceof UserStory){				
					onlyUs.add(allUS.get(i));
				}			
				if( PreferencesManager.getTableShowSystems() ){
					if(allUS.get(i) instanceof UsSystem){	
						onlyUs.add(allUS.get(i));
					}				
				}
				if( PreferencesManager.getTableShowClasses() ){
					if(allUS.get(i) instanceof LinkStub){
						if(((LinkStub)allUS.get(i)).getLinkType() == LinkStub.TYPE_CLASS ){
							onlyUs.add(allUS.get(i));
						}
					}				
				}
				if( PreferencesManager.getTableShowMethods() ){
					if(allUS.get(i) instanceof LinkStub){	
						if(((LinkStub)allUS.get(i)).getLinkType() == LinkStub.TYPE_METHOD ){
							onlyUs.add(allUS.get(i));
						}
					}				
				}
			}
			return onlyUs.toArray();
		}
		
		public ViewContentProvider getInstance()
		{
			return this;
		}
	}
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			return getText(obj);
		}
		public Image getColumnImage(Object obj, int index) {
			return getImage(obj);
		}
		public Image getImage(Object obj) {
			return Activator.getImageDescriptor("icons/userstory_default.png").createImage();
		}
	}
	/**
	 * The constructor.
	 */
	public UserStoryTableViewer()	{	
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	@SuppressWarnings("deprecation")
	public void createPartControl(Composite Tcontainer) {
		Tcontainer.setLayout(new GridLayout(1,false));

		GridData gd = new GridData();
		gd = new GridData(GridData.FILL, GridData.FILL, true, true);

		
		Composite container = new Composite(Tcontainer,SWT.NONE);
		container.setLayout(new GridLayout(3,true));		
		container.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
		
		Composite viewercontainer = new Composite(Tcontainer,SWT.NONE);
		viewercontainer.setLayout(new GridLayout(SWT.FILL,false));
		viewercontainer.setLayoutData(gd);
		
		viewer = new TableViewer(viewercontainer, SWT.MULTI|SWT.V_SCROLL|SWT.H_SCROLL);
		viewer.setSorter(null);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.addDoubleClickListener(new IDoubleClickListener(){

			@Override
			public void doubleClick(DoubleClickEvent arg0) {
				if(arg0.getSelection() instanceof StructuredSelection){
					Model temp  = ((Model)((StructuredSelection)arg0.getSelection()).getFirstElement());
					UserStoryTreeView.setSelection(temp);					
				}
			}
			
		});
		viewer.setInput(new Object());
		TableViewerColumn vcl = new TableViewerColumn(viewer,SWT.WRAP);
		
		TableColumn column = vcl.getColumn();
		column.setText("User Stories");
		column.setToolTipText("The name of the User Story (click to sort)");
		column.setWidth(200);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(new SelectionListener(){
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
		
			}
	
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				sortingName = true;
				viewer.setInput(new Object());					
				sorterName = !sorterName;
				sortingName = false;
			}
			
		});
		vcl.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Model p = (Model) element;
					return p.getName();
			}
		});
		
		vcl = new TableViewerColumn(viewer,SWT.WRAP);
		column = vcl.getColumn();
		column.setText("Actor");
		//column.setToolTipText("The actor of the User Story(click to sort)");
		column.setWidth(200);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(new SelectionListener(){
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
		
			}
	
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				sortingUser = true;
				viewer.setInput(new Object());
				sorterUser = !sorterUser;
				sortingUser = false;
			}
			
		});
		vcl.setLabelProvider(new ColumnLabelProvider() {
			 @Override
			 public String getText(Object element) {
				if(!(element instanceof UserStory)){					
					return "-";
				}
				UserStory p = (UserStory) element;
				String actors = "";
				for(int actorID : p.getActors())
				{
					actors += ActorManager.getActorById(actorID).getName() + " ";
				}
				actors = actors.trim();
					return actors;
			 }
		});

		vcl = new TableViewerColumn(viewer,SWT.WRAP);
		column = vcl.getColumn();
		column.setText("Test Cases");
		column.setToolTipText("The Test Cases of the User Story (Only User Stories have Test Cases)");
		column.setWidth(200);
		column.setResizable(true);
		column.setMoveable(true);
		vcl.setLabelProvider(new ColumnLabelProvider() {
			 @Override
			 public String getText(Object element) {
				if(!(element instanceof UserStory)){
					return "-";					
				}
				UserStory p = (UserStory) element;
				String returned = "";
				for(TestCase tf : p.getTestCases()){
					returned += tf.getId() + " ";
				}
				
				return returned;
			}
		});
		
		vcl = new TableViewerColumn(viewer,SWT.WRAP);
		column = vcl.getColumn();
		column.setText("Description");
		column.setToolTipText("The description of the User Story");
		column.setWidth(2000);
		column.setResizable(true);
		column.setMoveable(true);
		vcl.setLabelProvider(new ColumnLabelProvider() {
			 @Override
			 public String getText(Object element) {
				 Model p = (Model) element;
				 String returned = p.getDescription().replaceAll("\n", " ");
				 return returned;
			}
		});
		
		
		viewer.getTable().setHeaderVisible(true);
		viewer.getTable().setLinesVisible(true);
		//viewer.refresh();
		Composite searchActorContainer = new Composite(container,SWT.NONE);
		searchActorContainer.setLayout(new GridLayout(2,false));		
		searchActorContainer.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
		
		Label lblSearchActor = new Label(searchActorContainer, SWT.NONE);
		lblSearchActor.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, false, false));
		lblSearchActor.setText("Search Actor:");
	
		final Text txtSearchActor = new Text(searchActorContainer,SWT.BORDER );
		txtSearchActor.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
		txtSearchActor.setText("");
		/*txtSearchActor.addFocusListener(new FocusListener(){

			@Override
			public void focusGained(FocusEvent arg0) {
				viewer.setInput(new Object());		
			}

			@Override
			public void focusLost(FocusEvent arg0) {
				
			}
			
		});*/
		txtSearchActor.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e)
			{
				Text textWidget = (Text) e.getSource();
				String actorText = textWidget.getText();
				searchActors = actorText;
				viewer.setInput(new Object());				
			}
		});	

		Composite searchNameContainer = new Composite(container,SWT.NONE);
		searchNameContainer.setLayout(new GridLayout(2,false));		
		searchNameContainer.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
		
		Label lblSearchName = new Label(searchNameContainer, SWT.NONE);
		lblSearchName.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, false, false));
		lblSearchName.setText("Search Name:");
		
		final Text txtSearchName = new Text(searchNameContainer,SWT.BORDER );
		txtSearchName.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		txtSearchName.setText("");
		/*txtSearchName.addFocusListener(new FocusListener(){

			@Override
			public void focusGained(FocusEvent arg0) {
				viewer.setInput(new Object());
			}

			@Override
			public void focusLost(FocusEvent arg0) {
				
			}
			
		});*/
		txtSearchName.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e)
			{
				Text textWidget = (Text) e.getSource();
				String userText = textWidget.getText();
				searchName = userText;
				viewer.setInput(new Object());
			}
		});

		Composite searchDescContainer = new Composite(container,SWT.NONE);
		searchDescContainer.setLayout(new GridLayout(2,false));		
		searchDescContainer.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
		
		Label lblSearchDesc = new Label(searchDescContainer, SWT.NONE);
		lblSearchDesc.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, false, false));
		lblSearchDesc.setText("Search Description:");
		 
		final Text txtSearchDesc = new Text(searchDescContainer,SWT.BORDER );
		txtSearchDesc.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
		txtSearchDesc.setText("");
		/*txtSearchDesc.addFocusListener(new FocusListener(){

			@Override
			public void focusGained(FocusEvent arg0) {
				viewer.setInput(new Object());
			}

			@Override
			public void focusLost(FocusEvent arg0) {
				
			}
			
		});*/
		txtSearchDesc.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e)
			{
				Text textWidget = (Text) e.getSource();
				String descText = textWidget.getText();
				searchDesc = descText;
				viewer.setInput(new Object());				
			}
		});
		Button showAll = new Button(container,SWT.CHECK);
		showAll.setToolTipText("Show all user stories.The search texts will be reset.The preferences will not be affected");
		showAll.setText("Show All");
		showAll.addSelectionListener(new SelectionAdapter(){			
			@Override
			public void widgetSelected(SelectionEvent event) {
				if(((Button)event.getSource()).getSelection()){
					searchName = "";
					searchDesc = "";
					searchActors = "";
					txtSearchActor.setText("");
					txtSearchName.setText("");
					txtSearchDesc.setText("");
					UserStoryTableViewer.this.showAll = true;
					viewer.setInput(new Object());
				}else{
					UserStoryTableViewer.this.showAll = false;
					viewer.setInput(new Object());
				}
			}
			
		});
			makeActions();
			contributeToActionBars();
			PlatformUI.getWorkbench().getHelpSystem().setHelp(viewer.getControl(), "com.example.helloworld.viewer");
			getSite().setSelectionProvider(viewer);
			viewer.getControl().addKeyListener(new KeyListener(){
				private int keyPressed = -1;
				@Override
				public void keyPressed(KeyEvent event) {
					if(event.keyCode == 102 && keyPressed == 262144){
						openSearch.run();
					}
					keyPressed = event.keyCode;					
				}

				@Override
				public void keyReleased(KeyEvent event) {
					if(event.keyCode == 262144)
						keyPressed = -1;
				}
				
			});
			viewer.getTable().setLayout(new GridLayout());
			viewer.getTable().setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,true));
	}

	private void makeActions(){
		openSearch = new Action(){
			public void run(){
				SearchDialog x4 = new SearchDialog(getSite().getShell());
				x4.create();
				if(x4.open()==Window.OK){
					viewer.setInput(new Object());
				}
			}
		};
		openSearch.setText("Open Search");
		openSearch.setToolTipText("Open Search");
		openSearch.setImageDescriptor(Activator.getImageDescriptor("icons/search.gif"));
	}
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(openSearch);
	}
	/**
	 * Passing the focus request to the viewer's control.
	 */
	public static void Refresh(){
		//if user hasn't open the view the variable "viewer" will be null
		if(viewer != null){
			viewer.setInput(new Object());
		}
	}
	public static void Refresh(String actor,String name,String desc){
		searchActors = actor;
		searchName = name;
		searchDesc = desc;
		if(viewer!=null){
			viewer.setInput(new Object());
		}
	}
	public void setFocus() {
		if(viewer!=null)viewer.getControl().setFocus();
	}
	
	private ArrayList<Model> sort(ArrayList<Model> db,String sortBy,boolean sorter){				
		ArrayList<Model> returnThis = new ArrayList<Model>();
		if(db.size() == 0){
			return returnThis;
		}		
		while(db.size() > 0){
			int i = 0;
			Model first = db.get(0);
			while(i < db.size()){
				if(sortBy.equals("names")){
					if(sorter){
						if(first.getName().compareTo(db.get(i).getName()) < 0){				
							first = db.get(i);
						}
					}else{				
						if(first.getName().compareTo(db.get(i).getName()) > 0){				
							first = db.get(i);
						}
					}
				}else if(sortBy.equals("actors")){					
					if(sorter){
						if(first instanceof UserStory){
							if(db.get(i) instanceof UserStory){
								if(((UserStory)first).getActors().get(0).compareTo(((UserStory)db.get(i)).getActors().get(0)) < 0){				
									first = db.get(i);
								}
							}else{
								first = db.get(i);
							}
						}
					}else{				
						if(first instanceof UserStory){
							if(db.get(i) instanceof UserStory){
								if(((UserStory)first).getActors().get(0).compareTo(((UserStory)db.get(i)).getActors().get(0)) > 0){				
									first = db.get(i);
								}
							}else{
								first = db.get(i);
							}
						}
					}
					
				}
				i++;
			}
			db.remove(first);
			returnThis.add(first);
		}
		return returnThis;
	}
	private ArrayList<Model> getAll(Model us){
		ArrayList<Model> returnThis = new ArrayList<Model>();		
		if(!us.equals(Controller.getDatabase().getRoot())){
			returnThis.add(us);	
		}
		if(us instanceof UserStory){
			return returnThis;
		}
		if(us instanceof UsSystem){
			for(Model temp : ((UsSystem)us).getChildrenList()){
				if(temp instanceof UsSystem){
					returnThis.addAll(getAll(temp));
				}else{
					returnThis.add(temp);
				}			
			}
		}
		return returnThis;
	}
	
	private ArrayList<Model> search(ArrayList<Model> db, String searchThis,String searchIn){
		ArrayList<Model> returnThis = new ArrayList<Model>();
		searchThis = searchThis.toLowerCase();
		for(int i = 0; i < db.size(); i++){
			if(searchIn.equals(searchInNames)){
				if(db.get(i).getName().toLowerCase().contains(searchThis)){
					returnThis.add(db.get(i));
				}
			}else if(searchIn.equals(searchInDescriptions)){
				if(db.get(i).getDescription().toLowerCase().contains(searchThis)){
					returnThis.add(db.get(i));
				}
			}else if(searchIn.equals(searchInActors)){
				if(db.get(i) instanceof UserStory){
					int k = 0;				 
					while(k < ((UserStory)db.get(i)).getActors().size()){
						int id =  ((UserStory)db.get(i)).getActors().get(k);
						if(ActorManager.getActorById(id).getName().toLowerCase().contains(searchThis)){
							returnThis.add(db.get(i));
							k = ((UserStory)db.get(i)).getActors().size();//If an actor matches stop searching in actors
						}
						k++;
					}
				}
			}
		}		
		return returnThis;
	}
}
