package ui;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.*;
import org.eclipse.ui.part.*;

import actions.userstorytree.*;
import actions.userstorytree.actors.*;
import actions.userstorytree.history.*;
import actions.userstorytree.link.*;
import actions.userstorytree.model.*;
import actions.userstorytree.system.*;
import actions.userstorytree.userstory.*;

import actors.Actor;
import copypaste.CopyPasteDB;
import dragAndDrop.UserStoryDragListener;
import dragAndDrop.UserStoryTransferType;
import dragAndDrop.UserStoryTreeDropAdapter;
import links.RightClickContextManager;
import links.FoundJavaElement;
import model.*;
import plugin.Activator;
import plugin.Controller;
import plugin.LogWriter;
import ui.dialogs.projectselection.ProjectTreeDialog;

public class UserStoryTreeView extends ViewPart 
{
	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String ID = "ui.UserStoryTreeView";
	public static TreeViewer viewer;
	public static Shell shell;
	public static Model lastSelectionIndex;
	public static boolean showActors = false;
	public static StyledText currentProjectName;
	public static Object[] expandedElements;
	public static TreePath[] expandedTreePaths;
	
	private final static String MESSAGE_DIALOG_TITLE = "UserStoriesTool Message";

	/* Tree Actions */
	Action treeCollapseAction;
	Action treeExpandAction;
	Action treeDoubleClickAction;
	Action treeRefreshAction;
	Action treeShowActorsAction;
	Action treeSortAction;
	
	/* Actor Actions */
	Action actorManagerOpenAction;

	/* History Actions */
	Action historyRedoAction;
	Action historyUndoAction;

	/* Link Actions */
	Action linkSelectedMemberAction;

	/* Model Actions */
	Action modelCopyAction;
	Action modelDeleteAction;
	Action modelPasteAction;
	Action modelShowOnDiskAction;
	Action modelViewInfoAction;

	/* System Actions */
	Action systemCreateTopLevelAction;
	Action systemCreateUnderSelectionAction;

	/* User Story Actions */
	Action userStoryCreateAction;
	public static Action userStoryEditAction;

	public static String currentProjectPath;
	
	public UserStoryTreeView()
	{
		// Current Project Path is Empty on Start
		currentProjectPath = "";

	}

	@SuppressWarnings("deprecation")
	public void createPartControl(Composite parent)
	{
		shell = this.getSite().getShell();
		
		// Set the Layout for the entire View
		GridLayout parentLayout = new GridLayout();
		parentLayout.numColumns = 1;
		parent.setLayout(parentLayout);
		
		// Composite for Project Selection		
		Composite projectContainer = new Composite(parent, SWT.NONE);
		projectContainer.setLayout(new GridLayout(2, false));
		// projectContainer.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		
		// Grid Data for Project Selection	
		GridData projectContainerGridData = new GridData(SWT.FILL, SWT.BEGINNING, true, false);		
		projectContainer.setLayoutData(projectContainerGridData);

		// StyledText Label that displays the name of the current project.
		currentProjectName = new StyledText(projectContainer, SWT.NONE);
		StyledString styledProjectName = new StyledString();
        styledProjectName.append("Current Project: ", StyledString.QUALIFIER_STYLER);
        styledProjectName.append("None");
		currentProjectName.setText(styledProjectName.toString());
		currentProjectName.setStyleRanges(styledProjectName.getStyleRanges());
		
		// Project Switcher Button
		Button projectSwitcher = new Button(projectContainer, SWT.NONE);
		//projectSwitcher.setText("Select Project");
		projectSwitcher.setImage(Activator.getImageDescriptor("icons/change_project.png").createImage());
		projectSwitcher.setToolTipText("Change the working Project for User Stories Tool.");
		
		// Project Switcher Grid Data
		GridData projectSwitcherGridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		projectSwitcherGridData.horizontalAlignment = GridData.END;
		projectSwitcher.setLayoutData(projectSwitcherGridData);
		
		// Selection Listener that initiates Project Database and Dialog
		projectSwitcher.addSelectionListener(new SelectionListener()
			{
				public void widgetSelected(SelectionEvent event)
				{
					ProjectTreeDialog selectProjectDialog = new ProjectTreeDialog(viewer.getControl().getShell());
					selectProjectDialog.create();

					if (selectProjectDialog.open() == Window.OK)
					{ }
					viewer.getTree().deselectAll();
				}

				public void widgetDefaultSelected(SelectionEvent event)
				{
					widgetSelected(event);
				}
		    });

		// Composite for Tree Viewer
		Composite treeViewerContainer = new Composite(parent, SWT.NONE);
		treeViewerContainer.setLayout(new GridLayout(1, false));
		
		// Grid Data for Tree Viewer
		GridData treeViewerGridData = new GridData(SWT.FILL, SWT.FILL, true, true);		
		treeViewerContainer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		// Creation of TreeViewer
		viewer = new TreeViewer(treeViewerContainer, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		viewer.setContentProvider(new UserStoryContentProvider());
		viewer.setLabelProvider(new UserStoryLabelProvider());
		viewer.setSorter(new UserStoryTreeViewerSorter());
		viewer.getTree().addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				if (viewer.getTree().getSelectionCount() > 0)
				{
					if( viewer.getTree().getSelection()[0].getData() instanceof Actor){
						return;
					}
					lastSelectionIndex = (Model) viewer.getTree().getSelection()[0].getData();
					UserStoryTableViewer.Refresh();
					LogWriter.debug("UserStoryTreeView: viewer.SelectionListener(): changed selection to " + lastSelectionIndex.getName());
					//UserStoryTableViewer.Refresh(controller);
				}
				else
				{
					lastSelectionIndex = Controller.getDatabase().getRoot();
				}
			}
		});
		
		// Important! Makes the TreeViewer expand to all of its available space.
		viewer.getTree().setLayoutData(treeViewerGridData);
		
		// This listener resizes elements when the tree is resized		
		viewer.getTree().addControlListener(new ControlAdapter() {
		      public void controlResized(ControlEvent e) {
		    	  	//currentProjectName.setBounds(shell.getClientArea());
					currentProjectName.pack();
					//currentProjectName.open();
		      }
		});
		viewer.getControl().addKeyListener(new org.eclipse.swt.events.KeyListener() {
			private int previousKey = -1;
			@Override
			public void keyReleased(KeyEvent arg0) {				
			}
			
			@Override
			public void keyPressed(KeyEvent arg0) {	
				if(previousKey == 262144){ //262144 = left control
					if(arg0.keyCode == 122){//122 = z
						historyUndoAction.run();
					}
					if(arg0.keyCode == 121){//121 = y
						historyRedoAction.run();
					}					
				}
				if(arg0.keyCode == 127){//127 = delete
					modelDeleteAction.run();
				}
				previousKey = arg0.keyCode;
			}
		});		
		//viewer.setSorter(new NameSorter());
		//viewer.setInput(getViewSite());
		
		//controller = Controller.getInstance();
		int ops = DND.DROP_MOVE; //operations  
		Transfer[] transfers = new Transfer[] { UserStoryTransferType.getInstance(), PluginTransfer.getInstance() };
		viewer.addDragSupport(ops, transfers, new UserStoryDragListener(viewer));
		transfers = new Transfer[] { UserStoryTransferType.getInstance() };
		UserStoryTreeDropAdapter dropAdapter = new UserStoryTreeDropAdapter(viewer);
		viewer.addDropSupport(ops, transfers, dropAdapter);
		
		// Create the help context id for the viewer's control
		PlatformUI.getWorkbench().getHelpSystem().setHelp(viewer.getControl(), "UserStoryTreeView.viewer");
		getSite().setSelectionProvider(viewer);
		makeActions();
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();
		

		// Switch to default project if it is set
		Project defaultProject = Controller.getInstance().checkDefaultProject();
		if (defaultProject != null)
		{
			Controller.getInstance().switchProject(defaultProject);
		}
	}

	private void hookContextMenu()
	{
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager manager)
			{
				UserStoryTreeView.this.fillContextMenu(manager);
				for (int i = 0; i < viewer.getControl().getMenu().getItems().length; i++) {
					if (viewer.getControl().getMenu().getItems()[i].getText().equals("Paste")) {
						if (CopyPasteDB.copied == null || CopyPasteDB.copied.size() == 0 || !(lastSelectionIndex instanceof UsSystem)){
							viewer.getControl().getMenu().getItems()[i].setEnabled(false);
						}
						else
							viewer.getControl().getMenu().getItems()[i].setEnabled(true);
					} else if (viewer.getControl().getMenu().getItems()[i].getText().equals("Copy")) {
						if (lastSelectionIndex instanceof UsSystem || lastSelectionIndex instanceof UserStory) {
							viewer.getControl().getMenu().getItems()[i].setEnabled(true);
						}else
							viewer.getControl().getMenu().getItems()[i].setEnabled(false);
					}else if (viewer.getControl().getMenu().getItems()[i].getText().equals("Edit Requirement")) {
						if (!(lastSelectionIndex instanceof UsSystem) || !(lastSelectionIndex instanceof UserStory)) {
							viewer.getControl().getMenu().getItems()[i].setEnabled(false);
						} else
							viewer.getControl().getMenu().getItems()[i].setEnabled(true);
					}
				}
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		/*manager.add(action1);
		manager.add(new Separator());
		manager.add(action2);*/
	}

	private void fillContextMenu(IMenuManager manager) {
		
		// Don't add context menu options if we have no project
		if (currentProjectPath==null || currentProjectPath == "")
		{
			return;
		}
		manager.add(systemCreateTopLevelAction);
		
		// If there is a selection that isn't the Root System, allow these actions to appear
		if (lastSelectionIndex!=null)
		{
			if (!lastSelectionIndex.getAbsolutePath().equals(UsSystem.ROOT_ABSPATH))
			{
				// You can only create Systems under other Systems
				if (lastSelectionIndex instanceof UsSystem)
				{
					manager.add(systemCreateUnderSelectionAction);
					manager.add(userStoryCreateAction);
				}			
				manager.add(new Separator());
				manager.add(modelCopyAction);
				if(lastSelectionIndex instanceof UsSystem){
					manager.add(modelPasteAction);
				}
				manager.add(userStoryEditAction);				
				manager.add(new Separator());
				manager.add(modelDeleteAction);
				manager.add(historyUndoAction);
				manager.add(historyRedoAction);
				manager.add(new Separator());
				manager.add(modelViewInfoAction);
				manager.add(new Separator());
				
				//Change name of Link Selected Member action
				try
				{
					FoundJavaElement selectedElement = RightClickContextManager.getSelectedMember();
					if(lastSelectionIndex instanceof UserStory){
						if ( (!(selectedElement.getElementType().equals(Controller.getJavaElementDict(Controller.NO_MEMBER)))))
						{
							linkSelectedMemberAction.setText("Link " + selectedElement.getElementType() + " \'" + selectedElement.getElementName() + "'");
							manager.add(linkSelectedMemberAction);
						}
					}
				}
				catch (NullPointerException e)
				{
					LogWriter.info("UserStoryTreeView: RightClickContextManager returned null");
				}
			}
		}
		/*manager.add(action1);
		manager.add(action2);
		manager.add(new Separator());
		//drillDownAdapter.addNavigationActions(manager);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));*/
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		//manager.add(action1);
		//manager.add(action2);
		manager.add(treeRefreshAction);
		manager.add(actorManagerOpenAction);
		manager.add(modelShowOnDiskAction);
		manager.add(treeShowActorsAction);
		manager.add(treeExpandAction);
		manager.add(treeCollapseAction);
		manager.add(treeSortAction);
		manager.add(new Separator());
		//drillDownAdapter.addNavigationActions(manager);
	}

	private void makeActions()
	{
		/* Tree Actions */
		treeCollapseAction = new TreeCollapseAction();
		treeExpandAction = new TreeExpandAction();
		treeDoubleClickAction = new TreeDoubleClickAction();
		treeRefreshAction = new TreeRefreshAction();
		treeShowActorsAction = new TreeShowActorsAction();
		treeSortAction = new TreeSortAction();

		/* Actor Actions */
		actorManagerOpenAction = new ActorManagerOpenAction();
		
		/* History Actions */
		historyRedoAction = new HistoryRedoAction();
		historyUndoAction = new HistoryUndoAction();
		
		/* Link Actions */
		linkSelectedMemberAction = new LinkSelectedMemberAction();
		
		/* Model Actions */
		modelCopyAction = new ModelCopyAction();
		modelDeleteAction = new ModelDeleteAction(this);
		modelPasteAction = new ModelPasteAction();
		modelShowOnDiskAction = new ModelShowOnDiskAction(this);
		modelViewInfoAction = new ModelViewInfoAction();
		
		/* System Actions */
		systemCreateTopLevelAction = new SystemCreateTopLevelAction();
		systemCreateUnderSelectionAction = new SystemCreateUnderSelectionAction();
		
		/* User Story Actions */
		userStoryCreateAction = new UserStoryCreateAction();
		userStoryEditAction = new UserStoryEditAction();		
	}

	private void hookDoubleClickAction()
	{
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				treeDoubleClickAction.run();
			}
		});
	}
	
	public static void showMessage(String message)
	{
		MessageDialog.openInformation( viewer.getControl().getShell(), MESSAGE_DIALOG_TITLE , message);
	}

	public static void switchProject(String projectName)
	{
		LogWriter.info("UserStoryTreeView: switchProject: Init with project name: "+ projectName);
		currentProjectPath = projectName;
		StyledString styledProjectName = new StyledString();
        styledProjectName.append("Current Project: ", StyledString.QUALIFIER_STYLER);
        styledProjectName.append(projectName);
		currentProjectName.setText(styledProjectName.toString());
		currentProjectName.setStyleRanges(styledProjectName.getStyleRanges());
		currentProjectName.pack();
	}
	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	/*public static ArrayList<TreeItem> getTreeItemsFromUserStoryName(ArrayList<String> names){
		ArrayList<TreeItem> x = getTreeItemsFromUserStoryName(viewer.getTree().getItems(),names);
		if(x.size() !=0){
			UserStoryTreeView.viewer.getTree().setSelection(x.toArray(new TreeItem[0]));
			editUserStoryAction.run();
		}
		return x;
	}
	private static ArrayList<TreeItem> getTreeItemsFromUserStoryName(TreeItem[] x ,ArrayList<String> names){
		ArrayList<TreeItem> selectThose = new ArrayList<TreeItem>();
		viewer.getControl().setFocus();
		for (int i = 0; i < x.length; i++) {	
			for(String name : names){
				if (x[i].getText().equals(name)) {
					selectThose.add(x[i]);		
				}
				selectThose.addAll(getTreeItemsFromUserStoryName(x[i].getItems(),names));				
			}
		}
		return selectThose;
	}*/
	//Set Selection from the bottom view
		private static boolean setSelection(TreeItem x, Model x1) {
			boolean found = false;
			for (int i = 0; i < x.getItemCount(); i++) {
				if (x.getItem(i).getText().equals(x1.getName())) {
					viewer.getTree().setSelection(x.getItem(i));
					Object[] expanedObjects = new Object[viewer.getExpandedElements().length + 1];
					for (int j = 0; j < viewer.getExpandedElements().length; j++) {
						expanedObjects[j] = viewer.getExpandedElements()[j];
					}				
					try {
						expanedObjects[expanedObjects.length - 1] = viewer.getTree().getItem(i);
						viewer.setExpandedElements(expanedObjects);
					} catch (Exception e) {
					}
					return true;
				} else {
					found = setSelection(x.getItem(i), x1);
				}
			}
			return found;
		}
		
	//Set Selection from the bottom view
	public static void setSelection(Model selectThis) {
		viewer.getControl().setFocus();
		for (int i = 0; i < viewer.getTree().getItemCount(); i++) {
			if (viewer.getTree().getItem(i).getText().equals(selectThis.getName())) {
				viewer.getTree().setSelection(viewer.getTree().getItem(i));
				Object[] expandedObjects = new Object[viewer.getExpandedElements().length + 1];
				for (int j = 0; j < viewer.getExpandedElements().length; j++) {
					expandedObjects[j] = viewer.getExpandedElements()[j];
				}				
				try {
					expandedObjects[expandedObjects.length - 1] = viewer.getTree().getItem(i);
					viewer.setExpandedElements(expandedObjects);
				} catch (Exception e) {					
				}
				break;
			} else if (setSelection(viewer.getTree().getItem(i), selectThis)) {
				break;
			}
		}
	}
}