package ui.dialogs;

import java.io.IOException;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import model.Model;
import model.UsSystem;
import plugin.Database;
import plugin.config.PreferencesManager;

public class ModelConvertFormatDialog extends Dialog{

	private static String LABEL_INFO;
	private static Model model;
	private static String DIALOG_TITLE = "Convert ";
	private static final String DIALOG_TITLE_DEFAULT = "Convert ";
	private static final String LABEL_CHECKBOX = "Do not show this message again, and automatically convert Systems and User Stories in the future.";
	private static final String[] LABEL_INFO_PARTS = {"We have detected that this " , " is using an old format.\nDo you wish to convert \"", "\" to the new format?"};
	
	public ModelConvertFormatDialog(Shell parentShell, Model m)
	{
		super(parentShell);
		model = m;
		DIALOG_TITLE = DIALOG_TITLE_DEFAULT + "\""+m.getName()+"\"?";
	}

	@Override
	protected boolean isResizable()
	{
	    return true;
	}
	@Override
	protected Point getInitialSize()
	{
		return new Point(600, 165);
	}
	
	@Override
	protected void configureShell(Shell newShell)
	{
	      super.configureShell(newShell);
	      newShell.setText(DIALOG_TITLE);
	}
	
	@Override
	protected Control createDialogArea(Composite parent)
	{		
		Composite container = (Composite) super.createDialogArea(parent);
        
		Label labelInfo = new Label(container,SWT.WRAP);
		
		if (model instanceof UsSystem)
		{
			LABEL_INFO = LABEL_INFO_PARTS[0] + "System" + LABEL_INFO_PARTS[1] + model.getName() + LABEL_INFO_PARTS[2];
		}
		else
		{
			LABEL_INFO = LABEL_INFO_PARTS[0] + "User Story" + LABEL_INFO_PARTS[1] + model.getName() + LABEL_INFO_PARTS[2];
		}
		
		labelInfo.setText(LABEL_INFO);
		
		Button dontShowAgain = new Button(container,SWT.CHECK | SWT.WRAP);
		dontShowAgain.setText(LABEL_CHECKBOX);
		dontShowAgain.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent event)
			{
				Button btn = (Button) event.getSource();
				PreferencesManager.setShowConversionDialog(!btn.getSelection());
				PreferencesManager.instance.savePluginSettings();
			}
			
		});
		return parent;
	}
	
}
