package ui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import plugin.config.PreferencesManager;
import ui.UserStoryTableViewer;

public class SearchDialog extends Dialog {
	private String searchDesc = "";
	private String searchName = "";
	private String searchActor = "";

	public SearchDialog(Shell parentShell) {
		super(parentShell);
		this.setShellStyle(this.getShellStyle() ^ SWT.APPLICATION_MODAL | SWT.MODELESS);
		setBlockOnOpen(false);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite parentContainer = (Composite) super.createDialogArea(parent);
		GridLayout gl = new GridLayout();
		gl.numColumns = 1;
		gl.verticalSpacing = 5;
		parentContainer.setLayout(gl);

		Composite container = new Composite(parentContainer, SWT.NONE);
		container.setLayout(new GridLayout(2, false));
		container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		Composite container1 = new Composite(parentContainer, SWT.NONE);
		container1.setLayout(new GridLayout(2, false));
		container1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		Composite container2 = new Composite(parentContainer, SWT.NONE);
		container2.setLayout(new GridLayout(2, false));
		container2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		Label lblSearchActor = new Label(container, SWT.NONE);
		lblSearchActor.setLayoutData(new GridData(SWT.NONE, SWT.NONE, false, false, 2, 1));
		lblSearchActor.setText("Search Actor:           ");

		Text x = new Text(container, SWT.BORDER);
		x.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));		
		x.setText("");
		x.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				Text textWidget = (Text) e.getSource();
				String userText = textWidget.getText();
				searchActor = userText;
				UserStoryTableViewer.Refresh(searchActor, searchName, searchDesc);

			}
		});
		Label lblSearchName = new Label(container1, SWT.NONE);
		lblSearchName.setLayoutData(new GridData(SWT.NONE, SWT.NONE, false, false, 2, 1));
		lblSearchName.setText("Search Name:          ");

		Text x1 = new Text(container1, SWT.BORDER);
		x1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		x1.setText("");
		x1.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				Text textWidget = (Text) e.getSource();
				String userText = textWidget.getText();
				searchName = userText;
				UserStoryTableViewer.Refresh(searchActor, searchName, searchDesc);
			}
		});

		Label lblSearchDesc = new Label(container2, SWT.NONE);
		lblSearchDesc.setLayoutData(new GridData(SWT.NONE, SWT.NONE, false, false, 2, 1));
		lblSearchDesc.setText("Search Description: ");

		Text x2 = new Text(container2, SWT.BORDER);
		x2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		x2.setText("");
		x2.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				Text textWidget = (Text) e.getSource();
				String descText = textWidget.getText();
				searchDesc = descText;
				UserStoryTableViewer.Refresh(searchActor, searchName, searchDesc);
			}
		});
		// --------------------------------------------------------
		Button showSystemAtViews = new Button(parentContainer, SWT.CHECK);
		showSystemAtViews.setSelection(PreferencesManager.getTableShowSystems());
		showSystemAtViews.setText("Show Systems at bottom View");
		showSystemAtViews.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub

			}

			@Override
			public void widgetSelected(SelectionEvent event) {

				Button btn = (Button) event.getSource();
				PreferencesManager.setTableShowSystems(btn.getSelection());
				UserStoryTableViewer.Refresh(searchActor, searchName, searchDesc);
			}

		});
		// --------------------------------------------------------
		Button showLinkedClassesAtViews = new Button(parentContainer, SWT.CHECK);
		showLinkedClassesAtViews.setSelection(PreferencesManager.getTableShowClasses());	
		showLinkedClassesAtViews.setText("Show Linked Classes at bottom View");
		showLinkedClassesAtViews.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub

			}

			@Override
			public void widgetSelected(SelectionEvent event) {

				Button btn = (Button) event.getSource();
				PreferencesManager.setTableShowClasses(btn.getSelection());
				UserStoryTableViewer.Refresh(searchActor, searchName, searchDesc);
			}

		});
		// --------------------------------------------------------
		Button showLinkedMethodsAtViews = new Button(parentContainer, SWT.CHECK);
		showLinkedMethodsAtViews.setSelection(PreferencesManager.getTableShowMethods());
		showLinkedMethodsAtViews.setText("Show Linked Methods at bottom View");
		showLinkedMethodsAtViews.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub

			}

			@Override
			public void widgetSelected(SelectionEvent event) {

				Button btn = (Button) event.getSource();
				PreferencesManager.setTableShowMethods(btn.getSelection());
				UserStoryTableViewer.Refresh(searchActor, searchName, searchDesc);
			}

		});
		return parentContainer;
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, "OK", true);
	}

	@Override
	protected boolean isResizable() {
		return true;
	}

	@Override
	protected Point getInitialSize() {
		return new Point(250, 450);
	}

	@Override
	protected void okPressed() {
		super.okPressed();
	}

	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Search Dialog");
	}

	@Override
	public boolean close() {
		//MyPreferences.DeleteInDisk = true;
		return super.close();
	}
}
