package ui.dialogs.actors;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import actors.Actor;
import actors.ActorManager;
import plugin.Activator;
import plugin.LogWriter;
public class ActorManagerDialog extends Dialog
{
	public static final String WINDOW_NAME = "Actor Manager";
	public static final String TEXT_ACTOR_ADD = "Add Actor";
	public static final String TEXT_ACTOR_REMOVE = "Remove Actor";
	public static final String LABEL_SAVE_CHANGES = "Save Changes";
	static Button buttonSaveChanges;
	static Button buttonCancel;
	
	//Stores the ID value so that we know which Actor was edited upon save
	private LinkedHashMap<Text,Integer> fieldsMap;
	
	//Custom Button IDs
	public static final int BUTTON_SAVE_ID = IDialogConstants.CLIENT_ID+1;
	
	private ArrayList<Integer> removedActors;
	
	public ActorManagerDialog(Shell parentShell) throws IOException
	{
		super(parentShell);
	}
	
	@Override
	protected boolean isResizable()
	{
	    return true;
	}
	
	@Override
	protected void configureShell(Shell newShell)
	{
	      super.configureShell(newShell);
	      newShell.setText(WINDOW_NAME);
	}
	
	//Huge thanks to GGrec - http://stackoverflow.com/a/20228619
	@Override
	protected Control createDialogArea(Composite parent)
	{	
		fieldsMap = new LinkedHashMap<Text,Integer>();
		removedActors = new ArrayList<Integer>();
		
		GridData gridData = new GridData(GridData.FILL, GridData.CENTER, true, true);
		gridData.minimumWidth = 500;
		gridData.minimumHeight = 400;
		GridLayout layout = new GridLayout(1, true);
		
		final Composite container = (Composite) super.createDialogArea(parent);
        container.setLayout(layout);
		container.setLayoutData(gridData);
        
        final Composite compositeAbove = new Composite(container, SWT.PUSH);
        compositeAbove.setLayout(new GridLayout(1, false));
		compositeAbove.setLayoutData(new GridData(GridData.END, GridData.CENTER, true, false));
        
        final ScrolledComposite scrolledComposite = new ScrolledComposite(container, SWT.PUSH |  SWT.H_SCROLL | SWT.V_SCROLL);
        //SWT.BORDER |
        
        scrolledComposite.setExpandHorizontal(true);
        scrolledComposite.setExpandVertical(true);
		scrolledComposite.setLayoutData(gridData);
        
        final Composite composite = new Composite(scrolledComposite, SWT.NONE);
        composite.setLayout(new GridLayout(1, false));
        scrolledComposite.setContent(composite);
        scrolledComposite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));

        final Composite aboveActorsComposite = new Composite(composite, SWT.NONE);
        aboveActorsComposite.setLayout(new GridLayout(2, false));
        aboveActorsComposite.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false));

        final Composite actorsComposite = new Composite(composite, SWT.NONE);
        actorsComposite.setLayout(new GridLayout(2, false));
        actorsComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        final Composite belowActorsComposite = new Composite(composite, SWT.NONE);
        belowActorsComposite.setLayout(new GridLayout());
        belowActorsComposite.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false));

        final Button buttonAddActor = new Button(compositeAbove, SWT.PUSH);
        buttonAddActor.setText(TEXT_ACTOR_ADD);
        buttonAddActor.setLayoutData(new GridData(GridData.END, SWT.CENTER, true, false));
        buttonAddActor.setImage(new Image(null, Activator.getImageDescriptor("icons/add.png").getImageData()));
        buttonAddActor.addSelectionListener(new SelectionAdapter() 
        {
            @Override public void widgetSelected(final SelectionEvent e) 
            {
                addActor(composite, actorsComposite, scrolledComposite, new Actor(ActorManager.ACTOR_NEW_ID, ""));
            }
        });
        
        //Get list of Actors
        ArrayList<Actor> actorsList = ActorManager.getEditableActors();
		
		//Show the rest
		for (Actor actor : actorsList)
		{
			addActor(composite, actorsComposite, scrolledComposite, actor);
		}
		//lastId = ActorManager.getFreshID();
		return container;
	}
	
	void addActor(final Composite parentComposite, Composite actorComposite, final ScrolledComposite scrolledComp, Actor actor)
	{
		/*final Label label2 = new Label(comp, SWT.NONE);
        label2.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        label2.setText( value );*/

        final Text textField = new Text(actorComposite, SWT.BORDER);
        textField.setText(actor.getName());
        GridData gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
        gd.widthHint = 3;
        textField.setLayoutData(gd);
        textField.setFocus();
        
        fieldsMap.put(textField, actor.getId());
        
        final Button btnRemove = new Button(actorComposite, SWT.NONE);
        btnRemove.setImage(new Image(null, Activator.getImageDescriptor("icons/delete.png").getImageData()));
        btnRemove.setText(TEXT_ACTOR_REMOVE);
        btnRemove.addSelectionListener(new SelectionAdapter() 
        {
            @Override public void widgetSelected(final SelectionEvent e) 
            {
            	//If we're deleting an existing Actor, remember it so we assign ACTOR_NONE to its assigned User Stories
            	if (fieldsMap.get(textField) != ActorManager.ACTOR_NEW_ID)
            	{
            		removedActors.add(fieldsMap.get(textField));
            	}
            	
                fieldsMap.remove(textField);
                textField.dispose();
                btnRemove.dispose();
                scrolledComp.layout(true, true);
                scrolledComp.setMinSize(parentComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
            }
        });

        // DO THIS:
        scrolledComp.layout(true, true);
        scrolledComp.setMinSize(parentComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}
	
	protected void createButtonsForButtonBar(final Composite parent)
	{
	  // Change parent layout data to fill the whole bar
	  parent.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

	  buttonSaveChanges = createButton(parent, BUTTON_SAVE_ID, LABEL_SAVE_CHANGES , false);
	  buttonSaveChanges.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
	  buttonSaveChanges.setImage(new Image(null, Activator.getImageDescriptor("icons/save.png").getImageData()));
	  buttonSaveChanges.addListener(SWT.Selection, new Listener() {
	      public void handleEvent(Event e) {
		        switch (e.type) {
		        case SWT.Selection:
		        	ArrayList<Actor> actorsList = new ArrayList<Actor>();
		        	
		        	//Add ACTOR_NONE first
		        	actorsList.add(new Actor(ActorManager.ACTOR_NONE_ID, ActorManager.ACTOR_NONE_NAME));
		        	for (Map.Entry<Text, Integer> entry : fieldsMap.entrySet())
		        	{
		        		if (entry.getKey().getText().replaceAll(" ", "").length() == 0)
		        		{
		        			LogWriter.info("ActorManagerDialog: Save Changes: Skipping unnamed actor");
		        		}
		        		else
		        		{
		        			actorsList.add(new Actor(entry.getValue() + "," + entry.getKey().getText()));
		        		}
		        	}

		        	//Update User Stories that will have actors removed
		        	ActorManager.resetUSWithDeletedActors(removedActors);
		        	
		        	//Update -1 IDs
		        	ActorManager.freshenIDs(actorsList);
		        	
		        	//Sort by ID
		        	Collections.sort(actorsList,Actor.ActorIDComparator);
		        	
		        	//Save
		        	ActorManager.replaceAllWithList(actorsList);
	        		parent.getShell().dispose();
		        }
		      }
		    });
	  
	  buttonCancel = createButton(parent, IDialogConstants.CANCEL_ID,"Cancel", false);
	  buttonCancel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
	  buttonCancel.setImage(new Image(null, Activator.getImageDescriptor("icons/cancel.png").getImageData()));
	}
}
