package ui.dialogs.actors;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import actors.Actor;
import actors.ActorManager;
import model.UserStory;
import plugin.Activator;

public class ActorPicker extends Dialog
{
	public static final String WINDOW_NAME = "Select Actors";
	public static final String LABEL_SAVE_CHANGES = "OK";
	
	static Button buttonSaveChanges;
	static Button buttonCancel;
	
	public static UserStory givenUS;
	public static ArrayList<Integer> editorActors;
	public static boolean dirtyEditor;
	
	//Stores the ID value so that we know which Actor was edited upon save
	private LinkedHashMap<Button,Actor> fieldsMap;
	
	//Custom Button IDs
	public static final int BUTTON_SAVE_ID = IDialogConstants.CLIENT_ID+1;
	
	@SuppressWarnings("unused")
	private ArrayList<Integer> removedActors;
	
	public ActorPicker(Shell parentShell) throws IOException
	{
		super(parentShell);
	}
	
	@Override
	protected boolean isResizable()
	{
	    return true;
	}
	
	@Override
	protected void configureShell(Shell newShell)
	{
	      super.configureShell(newShell);
	      newShell.setText(WINDOW_NAME);
	}
	
	@Override
	protected Control createDialogArea(Composite parent)
	{	
		dirtyEditor = false;
		fieldsMap = new LinkedHashMap<Button,Actor>();
		
		GridData gridData = new GridData(GridData.FILL, GridData.CENTER, true, true);
		//gridData.minimumWidth = 500;
		//gridData.minimumHeight = 400;
		GridLayout layout = new GridLayout(1, true);
		
		final Composite container = (Composite) super.createDialogArea(parent);
        container.setLayout(layout);
		container.setLayoutData(gridData);
        
		final Composite actorsComposite = new Composite(container, SWT.PUSH);
        actorsComposite.setLayout(new GridLayout(2, false));
		actorsComposite.setLayoutData(new GridData(GridData.CENTER, GridData.CENTER, true, false));
        
        //Get list of Actors
        ArrayList<Actor> actorsList = ActorManager.getEditableActors();
        
		//Show them
		for (Actor actor : actorsList)
		{
			addActor(container, actorsComposite, actor);
		}
		
		//Get a list of this User Story's actors to show them as checked
        ArrayList<Actor> usActors = new ArrayList<Actor>();
        for (int id : givenUS.getActors())
        {
        	usActors.add(ActorManager.getActorById(id));
        }
		
        //Checking logic
		for (Map.Entry<Button, Actor> entry : fieldsMap.entrySet())
    	{
			//For existing actors of the US
			for (Actor actor : usActors)
			{
				if (entry.getValue().getId() == actor.getId())
				{
					entry.getKey().setSelection(true);
				}
			}
			
			//For "dirty" Actors of the Editor
			for (Integer actor : editorActors)
			{
				if (entry.getValue().getId() == actor)
				{
					entry.getKey().setSelection(true);
				}
			}
    	}
		
		return container;
	}
	
	void addActor(Composite parentComposite, Composite actorComposite, Actor actor)
	{
		final Label label2 = new Label(actorComposite, SWT.NONE);
        label2.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        label2.setText( actor.getName() );

        final Button checkBox = new Button(actorComposite, SWT.CHECK);
        GridData gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
        checkBox.setLayoutData(gd);
        checkBox.setFocus();
        
        fieldsMap.put(checkBox, actor);
	}
	
	protected void createButtonsForButtonBar(final Composite parent)
	{
	  // Change parent layout data to fill the whole bar
	  parent.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

	  buttonSaveChanges = createButton(parent, BUTTON_SAVE_ID, LABEL_SAVE_CHANGES , false);
	  buttonSaveChanges.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
	  buttonSaveChanges.setImage(new Image(null, Activator.getImageDescriptor("icons/accept.png").getImageData()));
	  buttonSaveChanges.addListener(SWT.Selection, new Listener() {
	      public void handleEvent(Event e) {
		        switch (e.type) {
		        case SWT.Selection:
		        	
		        	int checked = 0;
		        	ArrayList<Integer> removeActors = new ArrayList<Integer>();
		        	//For every button
		        	for (Map.Entry<Button, Actor> entry : fieldsMap.entrySet())
		        	{
		        		//If it's checked
		        		if (entry.getKey().getSelection())
		        		{
		        			checked++;
		        			//And it's not in the US actors
		        			if (editorActors.indexOf(entry.getValue().getId()) == -1 && givenUS.getActors().indexOf(entry.getValue().getId()) == -1)
		        			{
		        				//Add it to the list
		        				editorActors.add(entry.getValue().getId());
		        				dirtyEditor=true;
		        			}
		        		}
		        		else
		        		{
		        			if (editorActors.indexOf(entry.getValue().getId()) != -1)
		        			{
		        				removeActors.add(entry.getValue().getId());
		        				dirtyEditor=true;
		        			}
		        		}
		        	}
		        	
		        	//If the user leaves no checked actors, default to ACTOR_NONE
		        	if (checked == 0)
		        	{
		        		editorActors.add(ActorManager.ACTOR_NONE_ID);
        				dirtyEditor=true;
		        	}
		        	else
		        	{
		        		if (editorActors.indexOf(ActorManager.ACTOR_NONE_ID) != -1)
	        			{
	        				removeActors.add(ActorManager.ACTOR_NONE_ID);
	        				dirtyEditor=true;
	        			}
		        	}

		        	//Remove removeActors. We're doing this to not break the index size
		        	editorActors.removeAll(removeActors);
		        	
	        		parent.getShell().dispose();
		        }
		      }
		    });
	  
	  buttonCancel = createButton(parent, IDialogConstants.CANCEL_ID,"Cancel", false);
	  buttonCancel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
	  buttonCancel.setImage(new Image(null, Activator.getImageDescriptor("icons/cancel.png").getImageData()));
	}
	
}
