package ui.dialogs.projectselection;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

import model.Project;
import plugin.Activator;
import plugin.Controller;
import plugin.LogWriter;
import plugin.ProjectDatabase;
import plugin.config.PreferencesManager;

public class ProjectTreeDialog extends Dialog {

	public static final String WINDOW_NAME = "Select Project for User Stories Tool";
	public static final String BUTTON_PART = "Select ";
	public static final String BUTTON_NO_PROJECT = "No project selected";
	public static final String BUTTON_SET_AS_DEFAULT = "Set As Default Project";
	public static final String BUTTON_ALREADY_DEFAULT = "Default Project";
	public static TreeViewer projectTreeViewer;
	public static Composite parentRef;
	public static Project selection;
	public static Button btnOk;
	public static Button btnSetDefault;
	
	public ProjectTreeDialog(Shell parentShell)
	{
		super(parentShell);
	}
	
	protected boolean isResizable()
	{
	    return true;
	}
	
	protected void configureShell(Shell newShell)
	{
	      super.configureShell(newShell);
	      newShell.setText(WINDOW_NAME);
	}
	
	protected void initializeBounds() 
	{ 
		super.initializeBounds(); 
		Shell shell = this.getShell(); 
		Monitor primary = shell.getMonitor(); 
		Rectangle bounds = primary.getBounds (); 
		Rectangle rect = shell.getBounds (); 
		int x = bounds.x + (bounds.width - rect.width) / 2; 
		int y = bounds.y + (bounds.height - rect.height) / 2; 
		shell.setLocation (x, y); 
	} 

	@Override
	protected Point getInitialSize()
	{
		return new Point(500, 400);
	}
	
	protected Control createContents(final Composite parent)
	{
	    //parent.setSize(910, 300);

		parentRef = parent;
		final Composite container = (Composite) super.createDialogArea(parent);
		
		GridData gridData = new GridData(GridData.FILL, GridData.FILL, true, true);
		//gridData.minimumWidth = 675;
		//gridData.minimumHeight = 300;
		
		GridLayout layout = new GridLayout(1,true);
		layout.marginHeight = 10;
		layout.marginWidth = 10;
		
		container.setLayout(layout);
		container.setLayoutData(gridData);
	
	    projectTreeViewer = new TreeViewer(container, SWT.SINGLE);
	    projectTreeViewer.getTree().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	    projectTreeViewer.setContentProvider(new ProjectTreeContentProvider());
	    projectTreeViewer.setLabelProvider(new ProjectTreeLabelProvider());
	    
	    new ProjectDatabase();
	    
	    final Composite projectInfoContainer = new Composite(container, SWT.NONE);
	    projectInfoContainer.setLayout(new GridLayout(2, false));
	    projectInfoContainer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	    
	    Label projectNameInfo = new Label(projectInfoContainer, SWT.NONE);
	    projectNameInfo.setText("Name: ");
	    final Label nameLabel = new Label(projectInfoContainer, SWT.WRAP | SWT.LEFT);
	    nameLabel.setText("No Project Selected");
	    //projectNameLabel.setText("No Project Selected");
	    
	    Label projectPathInfo = new Label(projectInfoContainer, SWT.WRAP);
	    projectPathInfo.setText("Absolute Path: ");
	    final Label pathLabel = new Label(projectInfoContainer, SWT.WRAP | SWT.LEFT);
	    pathLabel.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ));
	    pathLabel.setText("No Project Selected");
	    //projectNameLabel.setText("");
	    
	    final GridData data = new GridData(SWT.HORIZONTAL, SWT.TOP, true, false, 1, 1);
	    nameLabel.setLayoutData(data);
	    pathLabel.setLayoutData(data);
	    
	    final Composite buttonBar = new Composite(container, SWT.FILL);
	    buttonBar.setLayout(new GridLayout(3, true));

	    final GridData buttonBarData = new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1);
	    buttonBar.setLayoutData(buttonBarData);
	    buttonBar.pack();
	    
	    // Listener that changes the Name / Path fields depending on the selected Project
	    projectTreeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
	    	   public void selectionChanged(SelectionChangedEvent event) {
	    	       // if the selection is empty clear the label
	    	       if( (projectTreeViewer.getTree().getSelection().length>0) )
	    	       {
	    	    	   selection = (Project)projectTreeViewer.getTree().getSelection()[0].getData();
	    	    	   LogWriter.debug("ProjectTreeViewer: SelectionChangedListener: " + selection.getName());
	    	    	   //ProjectTreeDialog.projectNameLabel.setText(selection.getName());
	    	    	   nameLabel.setText(selection.getName());
	    	    	   pathLabel.setText(selection.getProjectPath());
	    	    	   nameLabel.pack();
	    	    	   pathLabel.pack();
	    	    	   projectInfoContainer.pack();
	    	    	   btnOk.setEnabled(true);
	    	    	   btnOk.setText(BUTTON_PART + selection.getName());
	    	    	   btnSetDefault.setEnabled(!selection.isDefault());
	    	       }
	    	   }
	    	});
	    
	    parent.addControlListener(new ControlAdapter() {
		      public void controlResized(ControlEvent e) {
		    	  	//parent.pack();
		    	  	//container.pack();
		    	  	//projectTreeViewer.getTree().pack();
		    	  	//projectInfoContainer.pack();
		      }
		    });
	    
	    //currentProjectPath
	    projectTreeViewer.addDoubleClickListener(new IDoubleClickListener() {
	        @Override
	        public void doubleClick(DoubleClickEvent event)
	        {
	        	LogWriter.debug("ProjectTreeViewer: IDoubleClickListener Activated");
                @SuppressWarnings("unused")
				TreeViewer viewer = (TreeViewer) event.getViewer();
                IStructuredSelection thisSelection = (IStructuredSelection) event.getSelection();
                Object selectedNode = thisSelection.getFirstElement();
                
                // Switch project
                Controller.getInstance().switchProject(((Project)selectedNode));
                parent.getShell().dispose();	                
	        }
	});
	
	    container.pack();
	    
	    createButtonsForButtonBar(buttonBar);
	    return container;
  }
	
	protected void createButtonsForButtonBar(Composite parent)
	{
	  btnOk = createButton(parent, IDialogConstants.OK_ID,BUTTON_NO_PROJECT, false);
	  btnOk.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
	  btnOk.setImage(new Image(null, Activator.getImageDescriptor("icons/change_project.png").getImageData()));
	  btnOk.setEnabled(false);
	  
	  // Selection Listener that initiates Project Switching
	  btnOk.addSelectionListener(new SelectionListener()
		{
			public void widgetSelected(SelectionEvent event)
			{
				LogWriter.debug("ProjectTreeViewer: Switch Project Button Activated");

                // Switch project
                Controller.getInstance().switchProject(selection);
                try{
                parentRef.getShell().dispose();	      
                }
                catch (SWTException e)
                {
                	LogWriter.medium("ProjectTreeDialog: createButtonsForButtonBar: btnOk widgetSelected: SWTException occured");
                }
			}
	
			public void widgetDefaultSelected(SelectionEvent event)
			{
				widgetSelected(event);
			}
	    });
	  
	  btnSetDefault = createButton(parent, IDialogConstants.DETAILS_ID, BUTTON_SET_AS_DEFAULT, false);
	  btnSetDefault.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
	  btnSetDefault.setImage(new Image(null, Activator.getImageDescriptor("icons/default_project_button.png").getImageData()));
	  btnSetDefault.setEnabled(false);


	  btnSetDefault.addSelectionListener(new SelectionListener()
		{
			public void widgetSelected(SelectionEvent event)
			{
				LogWriter.debug("ProjectTreeViewer: Set Default Project Button Activated");
				PreferencesManager.setDefaultProject(selection.getName());
				PreferencesManager.instance.savePluginSettings();
				projectTreeViewer.refresh();
				btnSetDefault.setEnabled(false);
				LogWriter.debug("ProjectTreeViewer: Successfuly set " + selection.getName() + " as default project");

			}
	
			public void widgetDefaultSelected(SelectionEvent event)
			{
				widgetSelected(event);
			}
	    });
	  
	  Button btnCancel = createButton(parent, IDialogConstants.CANCEL_ID,"Cancel", false);
	  btnCancel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
	  btnCancel.setImage(new Image(null, Activator.getImageDescriptor("icons/cancel.png").getImageData()));
	  
	}
	
}


