package ui.dialogs.projectselection;

import java.util.List;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import model.Project;
import plugin.Activator;

// https://stackoverflow.com/a/30356349/
class BoldFontStyler extends Styler {
	public Font fontReference;
	
	public BoldFontStyler(Font reference)
	{
		fontReference = reference;
	}
    @Override
    public void applyStyles(final TextStyle textStyle)
    {
        FontDescriptor boldDescriptor = FontDescriptor.createFrom(fontReference);
        boldDescriptor.setStyle(SWT.BOLD);
        Font boldFont = boldDescriptor.createFont(Display.getCurrent());
        textStyle.font = boldFont;
    }
}

public class ProjectTreeLabelProvider extends StyledCellLabelProvider {
	  @SuppressWarnings({ "rawtypes", "unused" })
	private List listeners;

	  private Image file;

	  private Image dir;

	  public ProjectTreeLabelProvider() {

	  }

	  public Image getImage(Object arg0) {

		  if ( ((Project)arg0).isDefault() )
		  {
			  return Activator.getImageDescriptor("icons/default_project.png").createImage();
		  }
		  else if ( ((Project)arg0).getHasUserStories() )
		  {
			  return Activator.getImageDescriptor("icons/project_userstory.png").createImage();
		  }
		  else
		  {
			  return Activator.getImageDescriptor("icons/project.png").createImage();
		  }
	  }

	  public String getText(Object arg0) {
	    String text = ((Project) arg0).getProjectPath();

	    return text;
	  }

	  public void dispose() {
	    // Dispose the images
	    if (dir != null)
	      dir.dispose();
	    if (file != null)
	      file.dispose();
	  }

	  
	  public void update(ViewerCell cell)
	  {
	        Object obj = cell.getElement();
	        StyledString styledString;
	        if ( ((Project)obj).isDefault() )
	        {
		        styledString = new StyledString();
		        styledString.append("[default] ", StyledString.COUNTER_STYLER);
		        styledString.append(((Project)obj).getName());
	        }
	        else
	        {
		        styledString = new StyledString( ((Project)obj).getName());
	        }
	
	        
	        styledString.append(" : " +  ((Project)obj).getProjectPath(), StyledString.QUALIFIER_STYLER);
      
	
	        cell.setText(styledString.toString());
	        cell.setStyleRanges(styledString.getStyleRanges());
	        cell.setImage(getImage(obj));
	        super.update(cell);
	   }

	@Override
	public void addListener(ILabelProviderListener listener) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void removeListener(ILabelProviderListener listener) {
		// TODO Auto-generated method stub
		
	}

	}